@@ -16,8 +16,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -26,28 +26,16 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- *****************************************************************************
- *
- * $Log$
- * Revision 1.6  2005/12/21 05:34:09  sdlime
- * Fixed small problem in msRemoveStyle where a temporary style was not being initialized after being malloc'd.
- *
- * Revision 1.5  2005/02/18 03:06:44  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.4  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
 #ifdef USE_GDAL
 #  include "gdal.h"
 #  include "cpl_conv.h"
 #endif
 
-MS_CVSID("$Id: classobject.c 5080 2005-12-21 05:34:09Z sdlime $")
+MS_CVSID("$Id: classobject.c 6434 2007-08-01 14:23:54Z dmorissette $")
 
 /**
  * Move the style up inside the array of styles.
@@ -60,12 +48,12 @@ int msMoveStyleUp(classObj *class, int nStyleIndex)
         psTmpStyle = (styleObj *)malloc(sizeof(styleObj));
         initStyle(psTmpStyle);
         
-        msCopyStyle(psTmpStyle, &class->styles[nStyleIndex]);
+        msCopyStyle(psTmpStyle, class->styles[nStyleIndex]);
 
-        msCopyStyle(&class->styles[nStyleIndex], 
-                    &class->styles[nStyleIndex-1]);
+        msCopyStyle(class->styles[nStyleIndex], 
+                    class->styles[nStyleIndex-1]);
         
-        msCopyStyle(&class->styles[nStyleIndex-1], psTmpStyle);
+        msCopyStyle(class->styles[nStyleIndex-1], psTmpStyle);
 
         return(MS_SUCCESS);
     }
@@ -87,12 +75,12 @@ int msMoveStyleDown(classObj *class, int nStyleIndex)
         psTmpStyle = (styleObj *)malloc(sizeof(styleObj));
         initStyle(psTmpStyle);
         
-        msCopyStyle(psTmpStyle, &class->styles[nStyleIndex]);
+        msCopyStyle(psTmpStyle, class->styles[nStyleIndex]);
 
-        msCopyStyle(&class->styles[nStyleIndex], 
-                    &class->styles[nStyleIndex+1]);
+        msCopyStyle(class->styles[nStyleIndex], 
+                    class->styles[nStyleIndex+1]);
         
-        msCopyStyle(&class->styles[nStyleIndex+1], psTmpStyle);
+        msCopyStyle(class->styles[nStyleIndex+1], psTmpStyle);
 
         return(MS_SUCCESS);
     }
@@ -115,28 +103,29 @@ int msInsertStyle(classObj *class, styleObj *style, int nStyleIndex) {
         return -1;
     }
 
-    /* Possible to add another style? */
-    if (class->numstyles == MS_MAXSTYLES) {
-        msSetError(MS_CHILDERR, "Maximum number of class styles, %d, has been reached", "insertStyle()", MS_MAXSTYLES);
+    /* Ensure there is room for a new style */
+    if (msGrowClassStyles(class) == NULL) {
         return -1;
     }
     /* Catch attempt to insert past end of styles array */
-    else if (nStyleIndex >= MS_MAXSTYLES) {
-        msSetError(MS_CHILDERR, "Cannot insert style beyond index %d", "insertStyle()", MS_MAXSTYLES-1);
+    else if (nStyleIndex >= class->numstyles) {
+        msSetError(MS_CHILDERR, "Cannot insert style beyond index %d", "insertStyle()", class->numstyles-1);
         return -1;
     }
     else if (nStyleIndex < 0) { /* Insert at the end by default */
-        msCopyStyle(&(class->styles[class->numstyles]), style);
+        class->styles[class->numstyles]=style;
+	MS_REFCNT_INCR(style);
         class->numstyles++;
         return class->numstyles-1;
     }
-    else if (nStyleIndex >= 0 && nStyleIndex < MS_MAXSTYLES) {
+    else if (nStyleIndex >= 0 && nStyleIndex < class->numstyles) {
         /* Move styles existing at the specified nStyleIndex or greater */
         /* to a higher nStyleIndex */
         for (i=class->numstyles-1; i>=nStyleIndex; i--) {
             class->styles[i+1] = class->styles[i];
         }
-        msCopyStyle(&(class->styles[nStyleIndex]), style);
+        class->styles[nStyleIndex]=style;
+	MS_REFCNT_INCR(style);
         class->numstyles++;
         return nStyleIndex;
     }
@@ -158,18 +147,13 @@ styleObj *msRemoveStyle(classObj *class, int nStyleIndex) {
         return NULL;
     }
     else {
-        style = (styleObj *)malloc(sizeof(styleObj));
-        if (!style) {
-            msSetError(MS_MEMERR, "Failed to allocate styleObj to return as removed style", "msRemoveStyle");
-            return NULL;
-        }
-	initStyle(style);
-        msCopyStyle(style, &(class->styles[nStyleIndex]));
-        style->isachild = MS_FALSE;
+        style=class->styles[nStyleIndex];
         for (i=nStyleIndex; i<class->numstyles-1; i++) {
-             msCopyStyle(&class->styles[i], &class->styles[i+1]);
+             class->styles[i]=class->styles[i+1];
         }
+	class->styles[class->numstyles-1]=NULL;
         class->numstyles--;
+	MS_REFCNT_DECR(style);
         return style;
     }
 }
@@ -183,10 +167,13 @@ int msDeleteStyle(classObj *class, int nStyleIndex)
     int i = 0;
     if (class && nStyleIndex < class->numstyles && nStyleIndex >=0)
     {
+        if (freeStyle(class->styles[nStyleIndex]) == MS_SUCCESS)
+            msFree(class->styles[nStyleIndex]);
         for (i=nStyleIndex; i< class->numstyles-1; i++)
         {
-             msCopyStyle(&class->styles[i], &class->styles[i+1]);
+            class->styles[i] = class->styles[i+1];
         }
+        class->styles[class->numstyles-1] = NULL;
         class->numstyles--;
         return(MS_SUCCESS);
     }
