@@ -16,8 +16,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -26,28 +26,16 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- *****************************************************************************
- *
- * $Log$
- * Revision 1.11  2006/09/01 02:30:15  sdlime
- * Dan beat me to the bug 1428 fix. I took a bit futher by removing msLayerGetFilterString() from layerobject.c and refer to that in the mapscript getFilter/getFilterString methods.
- *
- * Revision 1.10  2005/02/18 03:06:44  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.9  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
 #ifdef USE_GDAL
 #  include "gdal.h"
 #  include "cpl_conv.h"
 #endif
 
-MS_CVSID("$Id: layerobject.c 5722 2006-09-01 02:30:15Z sdlime $")
+MS_CVSID("$Id: layerobject.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 /* ===========================================================================
    msInsertClass
@@ -65,49 +53,43 @@ int msInsertClass(layerObj *layer, classObj *classobj, int nIndex)
         return -1;
     }
         
-    /* Possible to add another? */
-    if (layer->numclasses == MS_MAXCLASSES) {
-        msSetError(MS_CHILDERR, "Max number of classes, %d, has been reached",
-                   "msInsertClass()", MS_MAXCLASSES);
+    /* Ensure there is room for a new class */
+    if (msGrowLayerClasses(layer) == NULL) {
         return -1;
     }
     /* Catch attempt to insert past end of styles array */
-    else if (nIndex >= MS_MAXCLASSES) {
+    else if (nIndex >= layer->numclasses) {
         msSetError(MS_CHILDERR, "Cannot insert class beyond index %d",
-                   "msInsertClass()", MS_MAXCLASSES-1);
+                   "msInsertClass()", layer->numclasses-1);
         return -1;
     }
     else if (nIndex < 0) { /* Insert at the end by default */
 #ifndef __cplusplus
-        initClass(&(layer->class[layer->numclasses]));
-        msCopyClass(&(layer->class[layer->numclasses]), classobj, layer);
+        layer->class[layer->numclasses]=classobj;
 #else
-        initClass(&(layer->_class[layer->numclasses]));
-        msCopyClass(&(layer->_class[layer->numclasses]), classobj, layer);
+        layer->_class[layer->numclasses]=classobj;
 #endif
+	MS_REFCNT_INCR(classobj);
         layer->numclasses++;
         return layer->numclasses-1;
     }
-    else if (nIndex >= 0 && nIndex < MS_MAXCLASSES) {
+    else if (nIndex >= 0 && nIndex < layer->numclasses) {
     
         /* Copy classes existing at the specified nIndex or greater */
         /* to an index one higher */
 
 #ifndef __cplusplus
-        initClass(&(layer->class[layer->numclasses]));
         for (i=layer->numclasses-1; i>=nIndex; i--)
             layer->class[i+1] = layer->class[i];
-        initClass(&(layer->class[nIndex]));
-        msCopyClass(&(layer->class[nIndex]), classobj, layer);
+        layer->class[nIndex]=classobj;
 #else
-        initClass(&(layer->_class[layer->numclasses]));
         for (i=layer->numclasses-1; i>=nIndex; i--)
             layer->_class[i+1] = layer->_class[i];
-        initClass(&(layer->_class[nIndex]));
-        msCopyClass(&(layer->_class[nIndex]), classobj, layer);
+        layer->_class[nIndex]=classobj;
 #endif
 
-        /* increment number of layers and return */
+	MS_REFCNT_INCR(classobj);
+        /* increment number of classes and return */
         layer->numclasses++;
         return nIndex;
     }
@@ -136,38 +118,27 @@ classObj *msRemoveClass(layerObj *layer, int nIndex)
     }
     else 
     {
-        classobj = (classObj *) malloc(sizeof(classObj));
-        if (!classobj) {
-            msSetError(MS_MEMERR, 
-                       "Failed to allocate classObj to return as removed Class",
-                       "msRemoveClass");
-            return NULL;
-        }
-        initClass(classobj);
 #ifndef __cplusplus
-        msCopyClass(classobj, &(layer->class[nIndex]), NULL);
+        classobj=layer->class[nIndex];
 #else
-        msCopyClass(classobj, &(layer->_class[nIndex]), NULL);
+        classobj=layer->_class[nIndex];
 #endif
+	classobj->layer=NULL;
+	MS_REFCNT_DECR(classobj);
 
         /* Iteratively copy the higher index classes down one index */
         for (i=nIndex; i<layer->numclasses-1; i++)
         {
 #ifndef __cplusplus
-            freeClass(&(layer->class[i]));
-            initClass(&(layer->class[i]));
-            msCopyClass(&layer->class[i], &layer->class[i+1], layer);
+            layer->class[i]=layer->class[i+1];
 #else
-            freeClass(&(layer->_class[i]));
-            initClass(&(layer->_class[i]));
-            msCopyClass(&layer->_class[i], &layer->_class[i+1], layer);
+            layer->_class[i]=layer->_class[i+1];
 #endif
         }
-        /* Free the extra class at the end */
 #ifndef __cplusplus
-        freeClass(&(layer->class[layer->numclasses-1]));
-#else  
-        freeClass(&(layer->_class[layer->numclasses-1]));
+        layer->class[i]=NULL;
+#else
+        layer->_class[i]=NULL;
 #endif
         
         /* decrement number of layers and return copy of removed layer */
@@ -184,15 +155,11 @@ int msMoveClassUp(layerObj *layer, int nClassIndex)
     classObj *psTmpClass = NULL;
     if (layer && nClassIndex < layer->numclasses && nClassIndex >0)
     {
-        psTmpClass = (classObj *)malloc(sizeof(classObj));
-        initClass(psTmpClass);
-        
-        msCopyClass(psTmpClass, &layer->class[nClassIndex], layer);
+        psTmpClass=layer->class[nClassIndex];
 
-        msCopyClass(&layer->class[nClassIndex], 
-                    &layer->class[nClassIndex-1], layer);
+        layer->class[nClassIndex] = layer->class[nClassIndex-1];
         
-        msCopyClass(&layer->class[nClassIndex-1], psTmpClass, layer);
+        layer->class[nClassIndex-1] = psTmpClass;
 
         return(MS_SUCCESS);
     }
@@ -210,15 +177,11 @@ int msMoveClassDown(layerObj *layer, int nClassIndex)
     classObj *psTmpClass = NULL;
     if (layer && nClassIndex < layer->numclasses-1 && nClassIndex >=0)
     {
-        psTmpClass = (classObj *)malloc(sizeof(classObj));
-        initClass(psTmpClass);
-        
-        msCopyClass(psTmpClass, &layer->class[nClassIndex], layer);
+        psTmpClass=layer->class[nClassIndex];
 
-        msCopyClass(&layer->class[nClassIndex], 
-                    &layer->class[nClassIndex+1], layer);
-        
-        msCopyClass(&layer->class[nClassIndex+1], psTmpClass, layer);
+        layer->class[nClassIndex] = layer->class[nClassIndex+1];
+
+        layer->class[nClassIndex+1] = psTmpClass;
 
         return(MS_SUCCESS);
     }
