@@ -24,20 +24,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.3  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.2  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapbits.c 4695 2005-06-14 16:03:36Z dan $")
+MS_CVSID("$Id: mapbits.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 #include <limits.h>
 
