@@ -42,7 +42,7 @@
 #include "mapserver.h"
 #include "mapsymbol.h"
 
-MS_CVSID("$Id: mapcopy.c 6434 2007-08-01 14:23:54Z dmorissette $")
+MS_CVSID("$Id: mapcopy.c 7255 2008-01-10 15:32:46Z sdlime $")
 
 #include "mapcopy.h"
 
@@ -255,6 +255,7 @@ int msCopyLabel(labelObj *dst, labelObj *src)
     MS_COPYSTRING(dst->bindings[i].item, src->bindings[i].item);
     dst->bindings[i].index = src->bindings[i].index; /* no way to use the macros */
   }
+  MS_COPYSTELEM(numbindings);
 
     MS_COPYSTRING(dst->font, src->font);
     MS_COPYSTELEM(type);
@@ -355,6 +356,7 @@ int msCopyStyle(styleObj *dst, styleObj *src)
     MS_COPYSTRING(dst->bindings[i].item, src->bindings[i].item);
     dst->bindings[i].index = src->bindings[i].index; /* no way to use the macros */
   }
+  MS_COPYSTELEM(numbindings);
 
     MS_COPYCOLOR(&(dst->color), &(src->color));
     MS_COPYCOLOR(&(dst->outlinecolor),&(src->outlinecolor));
@@ -837,6 +839,7 @@ int msCopyLayer(layerObj *dst, layerObj *src)
     MS_COPYSTELEM(numjoins);
 
     for (i = 0; i < dst->numjoins; i++) {
+        initJoin(&(dst->joins[i]));
         return_value = msCopyJoin(&(dst->joins[i]), &(src->joins[i]));
         if (return_value != MS_SUCCESS)
             return MS_FAILURE;
