@@ -24,35 +24,16 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.4  2006/06/02 20:34:24  dan
- * REmoved msDebug() call in msReadEncryptionKeyFromFile()
- *
- * Revision 1.3  2006/06/02 18:58:09  dan
- * Doc update: Buffer passed to msEncryptStringWithKey() has to be
- * strlen(in)+16+1 bytes and not strlen(in)+8+1 to account for hex encoding of
- * padding bytes
- *
- * Revision 1.2  2006/06/01 20:51:02  dan
- * Fixed possible buffer overflow in msDecryptStringTokens() and improved
- * error message if key file could not be read.
- *
- * Revision 1.1  2006/06/01 19:56:31  dan
- * Added ability to encrypt tokens (passwords, etc.) in database connection
- * strings (MS-RFC-18, bug 1792)
- *
- */
+ ****************************************************************************/
 
 #include <assert.h>
 #include <ctype.h>    /* isxdigit() */
 #include <stdlib.h>   /* rand() */
 #include <time.h>     /* time() */
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapcrypto.c 5483 2006-06-02 20:34:24Z dan $")
+MS_CVSID("$Id: mapcrypto.c 6373 2007-07-24 02:37:18Z dmorissette $")
 
 
 /**********************************************************************
@@ -103,10 +84,10 @@ MS_CVSID("$Id: mapcrypto.c 5483 2006-06-02 20:34:24Z dan $")
  *
  **********************************************************************/
 
-static void encipher(const unsigned long *const v,unsigned long *const w,
-                     const unsigned long *const k)
+static void encipher(const ms_uint32 *const v, ms_uint32 *const w,
+                     const ms_uint32 *const k)
 {
-   register unsigned long       y=v[0],z=v[1],sum=0,delta=0x9E3779B9,n=32;
+   register ms_uint32   y=v[0],z=v[1],sum=0,delta=0x9E3779B9,n=32;
 
    while(n-->0)
    {
@@ -118,11 +99,10 @@ static void encipher(const unsigned long *const v,unsigned long *const w,
    w[0]=y; w[1]=z;
 }
 
-static void decipher(const unsigned long *const v,unsigned long *const w,
-                     const unsigned long *const k)
+static void decipher(const ms_uint32 *const v, ms_uint32 *const w,
+                     const ms_uint32 *const k)
 {
-   register unsigned long       y=v[0],z=v[1],sum=0xC6EF3720,
-				delta=0x9E3779B9,n=32;
+   register ms_uint32       y=v[0],z=v[1],sum=0xC6EF3720, delta=0x9E3779B9,n=32;
 
    /* sum = delta<<5, in general sum = delta * n */
 
@@ -322,14 +302,14 @@ static int msLoadEncryptionKey(mapObj *map)
 
 void msEncryptStringWithKey(const unsigned char *key, const char *in, char *out)
 {
-    unsigned long v[4], w[4];
-    const unsigned long *k;
+    ms_uint32 v[4], w[4];
+    const ms_uint32 *k;
     int last_block = MS_FALSE;
     
     /* Casting the key this way is safe only as long as longs are 4 bytes
      * on this platform */
-    assert(sizeof(unsigned long) == 4);
-    k = (const unsigned long *) key;
+    assert(sizeof(ms_uint32) == 4);
+    k = (const ms_uint32 *) key;
     
     while(!last_block)
     {
@@ -384,14 +364,14 @@ void msEncryptStringWithKey(const unsigned char *key, const char *in, char *out)
 
 void msDecryptStringWithKey(const unsigned char *key, const char *in, char *out)
 {
-    unsigned long v[4], w[4];
-    const unsigned long *k;
+    ms_uint32 v[4], w[4];
+    const ms_uint32 *k;
     int last_block = MS_FALSE;
     
     /* Casting the key this way is safe only as long as longs are 4 bytes
      * on this platform */
-    assert(sizeof(unsigned long) == 4);
-    k = (const unsigned long *) key;
+    assert(sizeof(ms_uint32) == 4);
+    k = (const ms_uint32 *) key;
     
     while(!last_block)
     {
