@@ -30,7 +30,7 @@
 #include "mapserver.h"
 #include "maptime.h"
 
-MS_CVSID("$Id: mapdraw.c 6777 2007-09-03 03:54:47Z sdlime $")
+MS_CVSID("$Id: mapdraw.c 7019 2007-11-08 15:47:25Z warmerdam $")
 
 /*
  * Functions to reset any pen (color index) values previously set. Used primarily to reset things when
@@ -637,6 +637,10 @@ int msDrawLayer(mapObj *map, layerObj *layer, imageObj *image)
 
   if(layer->opacity == 0) return MS_SUCCESS; /* layer is completely transparent, skip it */
 
+  /* conditions may have changed since this layer last drawn, so set
+     layer->project true to recheck projection needs (Bug #673) */
+  layer->project = MS_TRUE;
+
   /* inform the rendering device that layer draw is starting. */
   msImageStartLayer(map, layer, image);
 
