@@ -24,79 +24,9 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.75.2.3  2006/12/06 05:51:18  sdlime
- * Make sure to declare errormsg at the top of the function for MSVC.
- *
- * Revision 1.75.2.2  2006/12/06 05:44:21  sdlime
- * Applied user supplied patch to fix long in-image error wrapping. (bug 1963)
- *
- * Revision 1.75.2.1  2006/11/24 22:11:27  frank
- * fix closing of stderr/stdout after writing error msg (bug 1970)
- *
- * Revision 1.75  2006/03/16 22:28:38  tamas
- * Fixed msGetErrorString so as not to truncate the length of the error messages
- * Added msAddErrorDisplayString to read the displayable messages separatedly
- *
- * Revision 1.74  2006/03/15 18:17:14  dan
- * Added SUPPORTS=SOS_SERVER to msGetVersion() (bug 1710)
- *
- * Revision 1.73  2006/03/14 03:17:19  assefa
- * Add SOS error code (Bug 1710).
- * Correct error codes numbers for MS_TIMEERR and MS_GMLERR.
- *
- * Revision 1.72  2006/02/18 21:11:35  hobu
- * make sure we cast the tv_sec to a long because on os x, the type is
- * really __darwin_suseconds_t, which is ultimately a long.
- *
- * Revision 1.71  2005/12/12 16:57:56  sean
- * write whether point/shape z and m is supported in version message
- *
- * Revision 1.70  2005/11/17 14:38:21  assefa
- * Correct function msGetVersion to indicate if mapserver was build with
- * MYGIS support.
- *
- * Revision 1.69  2005/09/13 23:43:31  frank
- * fix leak in threaded case of error object
- *
- * Revision 1.68  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.67  2005/04/07 21:51:22  frank
- * added SUPPORTS=THREADS to version info
- *
- * Revision 1.66  2005/04/07 17:23:16  assefa
- * Remove #ifdef USE_SVG. It was added during development.
- *
- * Revision 1.65  2005/04/07 13:46:46  frank
- * added report of SVG support
- *
- * Revision 1.64  2005/02/22 07:40:27  sdlime
- * A bunch of updates to GEOS integration. Can move many primatives between MapServer and GEOS, still need to do collections (e.g. multi-point/line/polygon). Added buffer method to mapscript (mapscript/shape.i).
- *
- * Revision 1.63  2005/02/18 03:06:45  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.62  2005/02/02 02:21:50  sdlime
- * Applied Jerry/Dan patch for bug 1194.
- *
- * Revision 1.61  2005/01/28 06:16:53  sdlime
- * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
- *
- * Revision 1.60  2005/01/26 14:42:13  frank
- * Removed msWebDebug() ... really this time!
- *
- * Revision 1.59  2005/01/07 18:51:09  sdlime
- * Added MS_GMLERR code.
- *
- * Revision 1.58  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "maperror.h"
 #include "mapthread.h"
 #include "maptime.h"
@@ -110,11 +40,7 @@
 #endif
 #include <stdarg.h>
 
-#ifdef NEED_NONBLOCKING_STDERR
-#include <fcntl.h>
-#endif
-
-MS_CVSID("$Id: maperror.c 5857 2006-12-06 05:51:18Z sdlime $")
+MS_CVSID("$Id: maperror.c 6757 2007-08-31 03:30:07Z tomkralidis $")
 
 static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"Unable to access file.",
@@ -153,7 +79,9 @@ static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"Invalid rectangle.",
 						"Date/time error.",
 						"GML encoding error.",
-                                                "SOS server error."
+            "SOS server error.",
+						"NULL parent pointer error.",
+            "AGG library error."
 };
 
 #ifndef USE_THREAD
@@ -358,11 +286,11 @@ char *msGetErrorCodeString(int code) {
 /* -------------------------------------------------------------------- */
 char *msAddErrorDisplayString(char *source, errorObj *error)
 {
-	if((source = strcatalloc(source, error->routine)) == NULL) return(NULL);
-	if((source = strcatalloc(source, ": ")) == NULL) return(NULL);
-	if((source = strcatalloc(source, ms_errorCodes[error->code])) == NULL) return(NULL);
-	if((source = strcatalloc(source, " ")) == NULL) return(NULL);
-	if((source = strcatalloc(source, error->message)) == NULL) return(NULL);
+	if((source = msStringConcatenate(source, error->routine)) == NULL) return(NULL);
+	if((source = msStringConcatenate(source, ": ")) == NULL) return(NULL);
+	if((source = msStringConcatenate(source, ms_errorCodes[error->code])) == NULL) return(NULL);
+	if((source = msStringConcatenate(source, " ")) == NULL) return(NULL);
+	if((source = msStringConcatenate(source, error->message)) == NULL) return(NULL);
 	return source;
 }
 
@@ -378,7 +306,7 @@ char *msGetErrorString(char *delimiter)
     if((errstr = msAddErrorDisplayString(errstr, error)) == NULL) return(NULL);
 	 
 	if(error->next && error->next->code != MS_NOERR) { /* (peek ahead) more errors, use delimiter */
-		if((errstr = strcatalloc(errstr, delimiter)) == NULL) return(NULL);
+		if((errstr = msStringConcatenate(errstr, delimiter)) == NULL) return(NULL);
 	}
     error = error->next;   
   }
@@ -388,9 +316,6 @@ char *msGetErrorString(char *delimiter)
 
 void msSetError(int code, const char *message_fmt, const char *routine, ...)
 {
-  char *errfile=NULL;
-  FILE *errstream;
-  time_t errtime;
   errorObj *ms_error = msInsertErrorObj();
   va_list args;
 
@@ -412,20 +337,9 @@ void msSetError(int code, const char *message_fmt, const char *routine, ...)
     va_end(args);
   }
 
-  errfile = getenv("MS_ERRORFILE");
-  if(errfile) {
-    if(strcmp(errfile, "stderr") == 0)
-      errstream = stderr;
-    else if(strcmp(errfile, "stdout") == 0)
-      errstream = stdout;
-    else
-      errstream = fopen(errfile, "a");
-    if(!errstream) return;
-    errtime = time(NULL);
-    fprintf(errstream, "%s - %s: %s %s\n", chop(ctime(&errtime)), ms_error->routine, ms_errorCodes[ms_error->code], ms_error->message);
-    if( errstream != stderr && errstream != stdout )
-        fclose(errstream);
-  }
+  /* Log a copy of errors to MS_ERRORFILE if set (handled automatically inside msDebug()) */
+  msDebug("%s: %s %s\n", ms_error->routine, ms_errorCodes[ms_error->code], ms_error->message);
+
 }
 
 void msWriteError(FILE *stream)
@@ -477,13 +391,13 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
   int nBlack = 0;   
   outputFormatObj *format = NULL;
   char *errormsg = msGetErrorString("; ");
-
-  if(map) {
-    if(map->width != -1 && map->height != -1) {
-      width = map->width;
-      height = map->height;
-    }
-    format = map->outputformat;
+  if (map) {
+      if( map->width != -1 && map->height != -1 )
+      {
+          width = map->width;
+          height = map->height;
+      }
+      format = map->outputformat;
   }
 
   if (format == NULL) format = msCreateDefaultOutputFormat( NULL, "GD/PC256" );
@@ -497,6 +411,7 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
   if (map->outputformat && map->outputformat->transparent)
     gdImageColorTransparent(img, 0);
 
+
   nTextLength = strlen(errormsg); 
   nWidthTxt  =  nTextLength * font->w;
   nUsableWidth = width - (nMargin*2);
@@ -582,6 +497,9 @@ char *msGetVersion() {
 #ifdef USE_PROJ
   strcat(version, " SUPPORTS=PROJ");
 #endif
+#ifdef USE_AGG
+  strcat(version, " SUPPORTS=AGG");
+#endif
 #ifdef USE_GD_FT
   strcat(version, " SUPPORTS=FREETYPE");
 #endif
@@ -643,41 +561,30 @@ char *msGetVersion() {
   strcat(version, " INPUT=MYGIS");
 #endif
   strcat(version, " INPUT=SHAPEFILE");
-#ifdef ENABLE_STDERR_DEBUG
-  strcat(version, " DEBUG=MSDEBUG");
-#endif
   return(version);
 }
 
-void msDebug( const char * pszFormat, ... )
+int msGetVersionInt() 
 {
-#ifdef ENABLE_STDERR_DEBUG
-    va_list args;
+    static int version = 0;
 
-#if defined(NEED_NONBLOCKING_STDERR) && !defined(USE_MAPIO) && !defined(_WIN32)
-    static char nonblocking_set = 0;
-    if (!nonblocking_set)
+    /* Parse MS_VERSION only once */
+    if (version == 0)
     {
-        fcntl(fileno(stderr), F_SETFL, O_NONBLOCK);
-        nonblocking_set = 1;
+        char **tokens = NULL;
+        int n = 0;
+        tokens = msStringSplit(MS_VERSION, '.', &n);
+        if (n<3)
+        {
+            msSetError(MS_MISCERR, "Invalid version string: %s", "msGetVersionInt()", MS_VERSION);
+        }
+        else
+        {
+            version = atoi(tokens[0])*10000 + atoi(tokens[1])*100 + atoi(tokens[2]);
+        }
+        if (tokens)
+            msFreeCharArray(tokens, n);
     }
-#endif
 
-#if !defined(USE_FASTCGI) && !defined(_WIN32)
-    /* It seems the FastCGI stuff inserts a timestamp anyways, so  */
-    /* we might as well skip this one.  And the struct timeval doesn't */
-    /* appear to exist on win32.  */
-    {
-        struct timeval tv;
-        msGettimeofday(&tv, NULL);
-        msIO_fprintf(stderr, "[%s].%ld ", 
-                     chop(ctime(&(tv.tv_sec))), (long)tv.tv_usec);
-    }
-#endif
-
-    va_start(args, pszFormat);
-    msIO_vfprintf(stderr, pszFormat, args);
-    va_end(args);
-#endif
+    return version;
 }
-
