@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maperror.h 5283 2006-03-16 22:28:38Z tamas $
+ * $Id: maperror.h 6749 2007-08-30 18:39:42Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Declarations for Error and Debug functions.
@@ -25,27 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.35  2006/03/16 22:28:38  tamas
- * Fixed msGetErrorString so as not to truncate the length of the error messages
- * Added msAddErrorDisplayString to read the displayable messages separatedly
- *
- * Revision 1.34  2006/03/14 03:17:19  assefa
- * Add SOS error code (Bug 1710).
- * Correct error codes numbers for MS_TIMEERR and MS_GMLERR.
- *
- * Revision 1.33  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.32  2005/01/07 18:51:09  sdlime
- * Added MS_GMLERR code.
- *
- * Revision 1.31  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
 #ifndef MAPERROR_H
 #define MAPERROR_H
@@ -54,6 +34,10 @@
 extern "C" {
 #endif
 
+/*====================================================================
+ *   maperror.c
+ *====================================================================*/
+
 #define MS_NOERR 0 /* general error codes */
 #define MS_IOERR 1 
 #define MS_MEMERR 2
@@ -92,11 +76,15 @@ extern "C" {
 #define MS_TIMEERR 35
 #define MS_GMLERR 36
 #define MS_SOSERR 37
-#define MS_NUMERRORCODES 38
+#define MS_NULLPARENTERR 38
+#define MS_AGGERR 39
+#define MS_NUMERRORCODES 40
 
 #define MESSAGELENGTH 2048
 #define ROUTINELENGTH 64
 
+#define MS_ERROR_LANGUAGE "en-US"
+
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #  define MS_DLL_EXPORT     __declspec(dllexport)
 #else
@@ -112,17 +100,13 @@ typedef struct error_obj {
 #endif
 } errorObj;
 
-/*
-** Global variables
-*/
-/* extern errorObj ms_error; */
-
 /*
 ** Function prototypes
 */
 MS_DLL_EXPORT errorObj *msGetErrorObj(void);
 MS_DLL_EXPORT void msResetErrorList(void);
 MS_DLL_EXPORT char *msGetVersion(void);
+MS_DLL_EXPORT int  msGetVersionInt(void);
 MS_DLL_EXPORT char *msGetErrorString(char *delimiter);
 
 #ifndef SWIG
@@ -135,8 +119,51 @@ MS_DLL_EXPORT char *msAddErrorDisplayString(char *source, errorObj *error);
 struct map_obj;
 MS_DLL_EXPORT void msWriteErrorImage(struct map_obj *map, char *filename, int blank);
 
+#endif /* SWIG */
+
+/*====================================================================
+ *   mapdebug.c (See also MS-RFC-28)
+ *====================================================================*/
+
+typedef enum { MS_DEBUGLEVEL_ERRORSONLY = 0,  /* DEBUG OFF, log fatal errors */
+               MS_DEBUGLEVEL_DEBUG      = 1,  /* DEBUG ON */
+               MS_DEBUGLEVEL_TUNING     = 2,  /* Reports timing info */
+               MS_DEBUGLEVEL_V          = 3,  /* Verbose */
+               MS_DEBUGLEVEL_VV         = 4,  /* Very verbose */
+               MS_DEBUGLEVEL_VVV        = 5   /* Very very verbose */
+} debugLevel;
+
+#ifndef SWIG
+
+typedef enum { MS_DEBUGMODE_OFF,
+               MS_DEBUGMODE_FILE,
+               MS_DEBUGMODE_STDERR,
+               MS_DEBUGMODE_STDOUT,
+               MS_DEBUGMODE_WINDOWSDEBUG
+} debugMode;
+
+typedef struct debug_info_obj 
+{
+    debugLevel  global_debug_level;
+    debugMode   debug_mode;
+    char        *errorfile;
+    FILE        *fp;
+    /* The following 2 members are used only with USE_THREAD (but we won't #ifndef them) */
+    int         thread_id;
+    struct debug_info_obj *next;
+} debugInfoObj;
+
+
 MS_DLL_EXPORT void msDebug( const char * pszFormat, ... );
-#endif
+MS_DLL_EXPORT int msSetErrorFile(const char *pszErrorFile);
+MS_DLL_EXPORT void msCloseErrorFile( void );
+MS_DLL_EXPORT const char *msGetErrorFile( void );
+MS_DLL_EXPORT void msSetGlobalDebugLevel(int level);
+MS_DLL_EXPORT debugLevel msGetGlobalDebugLevel( void );
+MS_DLL_EXPORT int msDebugInitFromEnv( void );
+MS_DLL_EXPORT void msDebugCleanup( void );
+
+#endif /* SWIG */
 
 #ifdef __cplusplus
 }
