@@ -1,6 +1,9 @@
 #ifndef MAPFILE_H
 #define MAPFILE_H
 
+enum MS_LEXER_STATES {MS_TOKENIZE_DEFAULT=0, MS_TOKENIZE_FILE, MS_TOKENIZE_STRING, MS_TOKENIZE_EXPRESSION, MS_TOKENIZE_URL_VARIABLE, MS_TOKENIZE_URL_STRING, MS_TOKENIZE_VALUE, MS_TOKENIZE_NAME};
+enum MS_TOKEN_SOURCES {MS_FILE_TOKENS=0, MS_STRING_TOKENS, MS_URL_TOKENS};
+
 /*
 ** Keyword definitions for the mapfiles and symbolfiles (used by lexer)
 */
@@ -38,12 +41,10 @@
 #define KEYSIZE 1031
 #define KEYSPACING 1032
 #define LABEL 1033
-#define LABELANGLEITEM 1034
 #define LABELCACHE 1035
 #define LABELITEM 1036
 #define LABELMAXSCALE 1037
 #define LABELMINSCALE 1038
-#define LABELSIZEITEM 1039
 #define LAYER 1040
 #define LEGEND 1041
 
@@ -199,9 +200,6 @@
 #define CONFIG 1158
 #define BANDSITEM 1159
 
-#define ANGLEITEM 1160
-#define SIZEITEM 1161
-
 #define ENCODING 1162
 
 #define WIDTH 1163
@@ -222,4 +220,15 @@
 
 #define RELATIVETO 1192
 
+#define OPACITY 1193
+#define PRIORITY 1194
+#define PATTERN 1195
+
+#define MAXSCALEDENOM 1196
+#define MINSCALEDENOM 1197
+#define LABELMAXSCALEDENOM 1198
+#define LABELMINSCALEDENOM 1199
+#define SYMBOLSCALEDENOM 1200
+#define SCALEDENOM 1201
+
 #endif /* MAPFILE_H */
