@@ -14,8 +14,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -24,111 +24,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.30  2005/10/31 15:58:11  frank
- * ensure TLOCK_GDAL released on error exits
- *
- * Revision 1.29  2005/09/13 23:51:04  frank
- * Pop all outstanding error handler in msGDALCleanup()
- *
- * Revision 1.28  2005/02/18 03:06:45  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.27  2005/01/15 20:03:14  frank
- * Fixed byte swapping error for 24bit images on bigendian systems.
- *
- * Revision 1.26  2004/11/04 21:06:09  frank
- * centralize 'stdout binary mode setting' for win32, add for gdal output
- *
- * Revision 1.25  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.24  2004/10/01 19:16:04  frank
- * use msIO_fwrite to write file to stdout
- *
- * Revision 1.23  2004/07/13 20:39:36  dan
- * Made msTmpFile() more robust using msBuildPath() to return absolute paths (bug 771)
- *
- * Revision 1.22  2004/05/25 15:56:26  frank
- * added rotation/geotransform support
- *
- * Revision 1.21  2004/05/18 14:42:51  frank
- * Removed unused variable.
- *
- * Revision 1.20  2004/05/06 21:06:08  frank
- * temporary filenames should not be altered with msBuildPath()
- *
- * Revision 1.19  2004/03/15 05:52:07  frank
- * only call CPLFreeConfig() with recent GDALs
- *
- * Revision 1.18  2004/03/13 20:11:42  frank
- * Added CONFIG option support on the mapObj
- *
- * Revision 1.17  2004/03/05 22:56:55  frank
- * improve GDAL cleanup
- *
- * Revision 1.16  2004/03/05 05:57:04  frank
- * support multi-band rawmode output
- *
- * Revision 1.15  2004/03/04 20:08:28  frank
- * added IMAGEMODE_BYTE (raw mode)
- *
- * Revision 1.14  2003/04/10 17:24:02  frank
- * Use format extension for file created in the temp directory.  This is mostly
- * intended to work around the fact that some GDAL drivers (such as ECW) will
- * replace the supplied extension with their own preferred extension.  If we
- * don't happen to use the preferred extension we will have a hard time opening
- * the file that is created.
- *
- * Revision 1.13  2002/12/16 18:59:58  assefa
- * Make sure map object is not null.
- *
- * Revision 1.12  2002/11/19 17:56:18  frank
- * don't call GDALDestroyDriverManager() if GDAL is old
- *
- * Revision 1.11  2002/11/19 05:47:29  frank
- * centralize GDAL initialization and cleanup
- *
- * Revision 1.10  2002/11/16 21:39:45  frank
- * fix support for generating colormaped files with transparency
- *
- * Revision 1.9  2002/09/20 03:44:06  sdlime
- * Swapped map_path for mappath for consistency.
- *
- * Revision 1.8  2002/09/17 13:08:28  julien
- * Remove all chdir() function and replace them with the new msBuildPath function.
- * This have been done to make MapServer thread safe. (Bug 152)
- *
- * Revision 1.7  2002/07/11 17:09:45  frank
- * corrected error reporting
- *
- * Revision 1.6  2002/06/21 18:34:31  frank
- * added support for INT16 and FLOAT imagemodes
- *
- * Revision 1.5  2002/06/13 19:55:57  frank
- * improve temporary file handling
- *
- * Revision 1.4  2002/06/12 21:20:32  frank
- * added automatic mimetype and extension setting
- *
- * Revision 1.3  2002/06/11 20:45:48  frank
- * fixed handling of temp files a bit
- *
- * Revision 1.2  2002/06/11 19:55:37  frank
- * added support for writing projection
- *
- * Revision 1.1  2002/06/11 13:49:52  frank
- * New
- *
- */
+ ****************************************************************************/
 
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapgdal.c 4968 2005-10-31 15:58:11Z frank $")
+MS_CVSID("$Id: mapgdal.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 #ifdef USE_GDAL
 
