@@ -24,10 +24,9 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- */
+ *****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
 #ifdef USE_GEOS
 
@@ -298,7 +297,7 @@ static shapeObj *msGEOSGeometry2Shape_point(GEOSGeom g)
   shape->line[0].numpoints = 1;
   shape->geometry = (GEOSGeom) g;
 
-  coords = GEOSGeom_getCoordSeq(g);
+  coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(g);
 
   GEOSCoordSeq_getX(coords, 0, &(shape->line[0].point[0].x));
   GEOSCoordSeq_getY(coords, 0, &(shape->line[0].point[0].y));
@@ -330,8 +329,8 @@ static shapeObj *msGEOSGeometry2Shape_multipoint(GEOSGeom g)
   shape->geometry = (GEOSGeom) g;
 
   for(i=0; i<numPoints; i++) {
-    point = GEOSGetGeometryN(g, i);
-    coords = GEOSGeom_getCoordSeq(point);
+    point = (GEOSGeom) GEOSGetGeometryN(g, i);
+    coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(point);
 
     GEOSCoordSeq_getX(coords, 0, &(shape->line[0].point[i].x));
     GEOSCoordSeq_getY(coords, 0, &(shape->line[0].point[i].y));
@@ -351,7 +350,7 @@ static shapeObj *msGEOSGeometry2Shape_line(GEOSGeom g)
 
   if(!g) return NULL;
   numPoints = GEOSGetNumCoordinates(g);
-  coords = GEOSGeom_getCoordSeq(g);
+  coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(g);
 
   shape = (shapeObj *) malloc(sizeof(shapeObj));
   msInitShape(shape);
@@ -392,9 +391,9 @@ static shapeObj *msGEOSGeometry2Shape_multiline(GEOSGeom g)
   shape->geometry = (GEOSGeom) g;
 
   for(j=0; j<numLines; j++) {
-    lineString = GEOSGetGeometryN(g, j);
+    lineString = (GEOSGeom) GEOSGetGeometryN(g, j);
     numPoints = GEOSGetNumCoordinates(lineString);
-    coords = GEOSGeom_getCoordSeq(lineString);
+    coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(lineString);
 
     line.point = (pointObj *) malloc(sizeof(pointObj)*numPoints);
     line.numpoints = numPoints;
@@ -429,9 +428,9 @@ static shapeObj *msGEOSGeometry2Shape_polygon(GEOSGeom g)
   shape->geometry = (GEOSGeom) g;
 
   /* exterior ring */
-  ring = GEOSGetExteriorRing(g);
+  ring = (GEOSGeom) GEOSGetExteriorRing(g);
   numPoints = GEOSGetNumCoordinates(ring);
-  coords = GEOSGeom_getCoordSeq(ring);
+  coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(ring);
   
   line.point = (pointObj *) malloc(sizeof(pointObj)*numPoints);
   line.numpoints = numPoints;
@@ -446,11 +445,11 @@ static shapeObj *msGEOSGeometry2Shape_polygon(GEOSGeom g)
   /* interior rings */
   numRings = GEOSGetNumInteriorRings(g);
   for(j=0; j<numRings; j++) {
-    ring = GEOSGetInteriorRingN(g, j);
+    ring = (GEOSGeom) GEOSGetInteriorRingN(g, j);
     if(GEOSisRing(ring) != 1) continue; /* skip it */
 
     numPoints = GEOSGetNumCoordinates(ring);
-    coords = GEOSGeom_getCoordSeq(ring);
+    coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(ring);
 
     line.point = (pointObj *) malloc(sizeof(pointObj)*numPoints);
     line.numpoints = numPoints;
@@ -485,12 +484,12 @@ static shapeObj *msGEOSGeometry2Shape_multipolygon(GEOSGeom g)
   shape->geometry = (GEOSGeom) g;
 
   for(k=0; k<numPolygons; k++) { /* for each polygon */
-    polygon = GEOSGetGeometryN(g, k);
+    polygon = (GEOSGeom) GEOSGetGeometryN(g, k);
 
     /* exterior ring */
-    ring = GEOSGetExteriorRing(polygon);
+    ring = (GEOSGeom) GEOSGetExteriorRing(polygon);
     numPoints = GEOSGetNumCoordinates(ring);
-    coords = GEOSGeom_getCoordSeq(ring);
+    coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(ring);
 
     line.point = (pointObj *) malloc(sizeof(pointObj)*numPoints);
     line.numpoints = numPoints;
@@ -506,11 +505,11 @@ static shapeObj *msGEOSGeometry2Shape_multipolygon(GEOSGeom g)
     numRings = GEOSGetNumInteriorRings(polygon);
 
     for(j=0; j<numRings; j++) {
-      ring = GEOSGetInteriorRingN(polygon, j);
+      ring = (GEOSGeom) GEOSGetInteriorRingN(polygon, j);
       if(GEOSisRing(ring) != 1) continue; /* skip it */      
 
       numPoints = GEOSGetNumCoordinates(ring);
-      coords = GEOSGeom_getCoordSeq(ring);	  
+      coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(ring);	  
 
       line.point = (pointObj *) malloc(sizeof(pointObj)*numPoints);
       line.numpoints = numPoints;
@@ -613,9 +612,11 @@ char *msGEOSShapeToWKT(shapeObj *shape)
 
   if(!shape) 
     return NULL;
+    
+  /* if we have a geometry, we should update it*/
+  msGEOSFreeGeometry(shape->geometry);
 
-  if(!shape->geometry) /* if no geometry for the shape then build one */
-    shape->geometry = (GEOSGeom) msGEOSShape2Geometry(shape);
+  shape->geometry = (GEOSGeom) msGEOSShape2Geometry(shape);
   g = (GEOSGeom) shape->geometry;
   if(!g) return NULL;
 
@@ -710,7 +711,7 @@ pointObj *msGEOSGetCentroid(shapeObj *shape)
 
   point = (pointObj *) malloc(sizeof(pointObj));
 
-  coords = GEOSGeom_getCoordSeq(g2);
+  coords = (GEOSCoordSeq) GEOSGeom_getCoordSeq(g2);
 
   GEOSCoordSeq_getX(coords, 0, &(point->x));
   GEOSCoordSeq_getY(coords, 0, &(point->y));
