@@ -24,188 +24,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.96  2006/08/22 23:14:13  hobu
- * throw away the const qualifier of msOWSLookupMetadata when we set
- * the namespace_prefix which is not defined as constant
- *
- * Revision 1.95  2006/08/22 04:45:06  sdlime
- * Fixed a bug that did not allow for seperate metadata namespaces to be used for WMS vs. WFS for GML transformations (e.g. WFS_GEOMETRIES, WMS_GEOMETRIES).
- *
- * Revision 1.94  2006/08/14 20:06:32  dan
- * Produce warning in WFS GetFeature output if ???_featureid is specified
- * but corresponding item is not found in layer (bug 1781)
- *
- * Revision 1.93  2006/08/14 18:47:28  sdlime
- * Some more implementation of external application schema support.
- *
- * Revision 1.92  2006/08/11 21:32:22  sdlime
- * The GML writer for WFS now allows a layer to override the default namespace via OWS_NAMESPACE_PREFIX at the layer level.
- *
- * Revision 1.91  2006/08/11 18:56:40  sdlime
- * Initial versions of namespace read/free functions.
- *
- * Revision 1.90  2006/08/01 13:48:31  sdlime
- * Added missing / in closing tag for multipoint (GML v2.1.2).
- *
- * Revision 1.89  2006/08/01 03:09:26  sdlime
- * Added missing pointMember tag to multipoint output for GML 2.1.2. (bug 1847)
- *
- * Revision 1.88  2006/05/02 20:52:06  dan
- * Output feature id as @fid instead of @gml:id in WFS 1.0.0 / GML 2.1.2
- * GetFeature requests (bug 1759)
- *
- * Revision 1.87  2006/05/02 19:38:39  dan
- * Allow use of wms/ows_include_items and wms/ows_exclude_items to control
- * which items to output in text/plain GetFeatureInfo. (bug 1761)
- *
- * Revision 1.86  2006/04/08 05:24:41  frank
- * Fixed itemList->items memory leak.
- *
- * Revision 1.85  2006/04/08 05:16:55  frank
- * Fixed memory leak of encoded entities in msGMLWriteItem().
- *
- * Revision 1.84  2006/03/22 21:40:47  sdlime
- * Added support to allow a service provider (WFS or WMS) tonot expose feature geometries. (bug 1718)
- *
- * Revision 1.83  2006/02/16 07:49:10  sdlime
- * Removed duplicate call to create outer ring list in writeGeometry_GML2().
- *
- * Revision 1.82  2006/02/07 17:40:23  sdlime
- * Reverting to Dan's original solution for the template member.
- *
- * Revision 1.81  2006/02/07 16:13:37  sdlime
- * Renamed gmlItemObj template to tmplate to avoid c++ compile errors.
- *
- * Revision 1.80  2006/02/06 19:50:41  sdlime
- * Added ability to define a item template for GML output, best for use with application schema. A templated attribute is: 1) not queryable and 2) not output in the server produced schema. The layer namespace and item value can be accessed via the template: e.g. gml_area_template '<$namespace:area>$value</$namespace:area>
- *
- * Revision 1.79  2006/01/23 22:42:53  julien
- * Add gml:lineStringMember in GML2 MultiLineString geometry (bug 1569)
- *
- * Revision 1.78  2005/11/21 23:22:15  sdlime
- * Trivial comment fixes.
- *
- * Revision 1.77  2005/11/01 16:55:18  sdlime
- * Updated the layer loops in the GML writers to repsect layer order.
- *
- * Revision 1.76  2005/10/26 21:22:04  sdlime
- * I changed my mind and removed the option to use the shape index as an ID. I think it is better practice to use a more stable key item. This should also make supporting featureid requests easier since it's just a shortcut for a filter. Normal filter processing does not support index queries.
- *
- * Revision 1.75  2005/10/26 21:07:35  sdlime
- * WFS GML output includes an option to set a gml:id for each feature. The default is no id (current behavior), but this can be overidden using the metadata value gml_featureid, which takes an item name to use to create the id or the special value '_index' to use the shape->index. This partially addresses bug 1413.
- *
- * Revision 1.74  2005/10/25 20:29:52  sdlime
- * Completed work to add constants to GML output. For example gml_constants 'aConstant'  gml_aConstant_value 'this is a constant', which results in output like <aConstant>this is a constant</aConstant>. Constants can appear in groups and can haves pecific types (default is string). Constants are NOT queryable so their use should be limited until some extensions to wfs 1.1 appear that will allow us to mark certain elements as queryable or not in capabilities output.
- *
- * Revision 1.73  2005/10/24 21:03:03  sdlime
- * Removed unused variable geom_name from WFS GML writer.
- *
- * Revision 1.72  2005/10/24 20:52:46  sdlime
- * Moved code to generate the layer_name outside of the feature loop. It was being executed for each feature when it only needed to be run once per layer.
- *
- * Revision 1.71  2005/10/24 20:46:46  sdlime
- * Updated GML transformation metadata handling to use the msOWSLookupMetadata functions with prefixes OWS_, WFS_ and GML_. Previously only the GML_ prefix was supported.
- *
- * Revision 1.70  2005/10/12 15:34:27  sdlime
- * Updated the gmlGroupObj to allow you to set the group type. This impacts schema location. If not set the complex type written by the schema generator is 'groupnameType' and via metadata you can override that (e.g. gml_groupname_type 'MynameType').
- *
- * Revision 1.69  2005/09/04 20:20:28  sdlime
- * GML writes now support multiple ways of defining a multipoint. 1) one line string with multiple points and 2) multiplle line strings with one point each. (bug 1490)
- *
- * Revision 1.68  2005/08/01 16:34:28  sdlime
- * Fixed a problem with the GML writer not properly closing the geometry container for MultiPoints. (bug 1409).
- *
- * Revision 1.67  2005/07/06 15:18:17  dan
- * Fixed missing space in GML MultiLineString output (bug 1408)
- *
- * Revision 1.66  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.65  2005/06/07 03:34:34  sdlime
- * Ported GML 2.1.2 writer to use geometry metadata like the GML 3.1 writer.
- *
- * Revision 1.64  2005/06/01 19:51:12  sdlime
- * Updated GML output to produce features in a default container (msGeometry) if no geometry metadata is supplied.
- *
- * Revision 1.63  2005/05/31 18:24:49  sdlime
- * Updated GML3 writer to use the new gmlGeometryListObj. This allows you to package geometries from WFS in a pretty flexible manner. Will port GML2 writer once testing on GML3 code is complete.
- *
- * Revision 1.62  2005/05/31 05:49:31  sdlime
- * Added geometry metadata processing functions to mapgml.c.
- *
- * Revision 1.61  2005/05/27 17:49:14  sdlime
- * Changed dimension to srsDimension for posList in GML3 writer.
- *
- * Revision 1.60  2005/05/26 21:19:05  sdlime
- * Fixed problem with GML outout when gml_geometry_name is not set.
- *
- * Revision 1.59  2005/05/26 16:09:15  sdlime
- * Updated mapwfs.c to produce schema compliant with the GML for Simple Feature Exchange proposed standard. Changes are relatively minor with the exception of naming the geometry container and handling of mixed geometry types. The previous version defaulted to a GML type. We need more control for application specific schema. We now package the GML geometry in an element named by default geometry, users can override using gml_geometry_name metadata. We also advertise a *very* generic GMLPropertyType by default again which can be overriden using gml_geometry_type. That metadata *can* contain a list of valid types which are offered as a xsd:choice.
- *
- * Revision 1.58  2005/05/23 17:31:27  sdlime
- * Move GML metadata structures and functions prototypes to mapows.h since they will be needed by mapwfs.c as well.
- *
- * Revision 1.57  2005/04/25 06:41:56  sdlime
- * Applied Bill's newest gradient patch, more concise in the mapfile and potential to use via MapScript.
- *
- * Revision 1.56  2005/04/21 21:10:38  sdlime
- * Adjusted WFS support to allow for a new output format (GML3).
- *
- * Revision 1.55  2005/04/21 04:46:35  sdlime
- * Fixed a small problem with last commit.
- *
- * Revision 1.54  2005/04/20 21:40:44  sdlime
- * Bug 950 transformations now apply to WMS output as well. You must explicitly request that items be exposed or set gml_include_items 'all'.
- *
- * Revision 1.53  2005/04/14 16:51:07  frank
- * Removed unused tab in printf.
- *
- * Revision 1.52  2005/04/01 23:02:45  frank
- * Avoid emitting extra whitespace in the gml:coordinates for the gml:Box.
- * Related to bug 1304.
- *
- * Revision 1.51  2005/03/25 20:39:24  sdlime
- * Fixed problem in WFS GML writer. Memory was being free'd in the wrong place. Should've been inside the if-then statement that tests if a layer is valid for WFS output.
- *
- * Revision 1.50  2005/03/16 17:19:30  sdlime
- * Updated GML group reading code to explicitly initialize all structure members.
- *
- * Revision 1.49  2005/02/18 19:15:23  sdlime
- * Added GML item-level transformation support for WFS. Includes inclusion/exclusion, column aliases and simple groups. (bug 950)
- *
- * Revision 1.48  2005/02/18 03:06:45  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.47  2004/12/29 22:49:57  sdlime
- * Added GML3 writing capabilities to mapgml.c. Not hooked up to anything yet.
- *
- * Revision 1.46  2004/12/14 21:30:43  sdlime
- * Moved functions to build lists of inner and outer rings to mapprimitive.c from mapgml.c. They are needed to covert between MapServer polygons and GEOS gemometries (bug 771).
- *
- * Revision 1.45  2004/11/16 21:57:49  dan
- * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
- * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
- *
- * Revision 1.44  2004/11/09 16:00:34  sean
- * move conditional compilation inside definition of msGMLWriteQuery and
- * msGMLWriteWFSQuery.  if OWS macros aren't defined, these functions set an
- * error and return MS_FAILURE (bug 1046).
- *
- * Revision 1.43  2004/10/28 18:54:05  dan
- * USe OWS_NOERR instead of MS_NOERR in calls to msOWSPrint*()
- *
- * Revision 1.42  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "maperror.h"
 
-MS_CVSID("$Id: mapgml.c 5630 2006-08-22 23:14:13Z hobu $")
+MS_CVSID("$Id: mapgml.c 6580 2007-08-14 14:53:32Z sdlime $")
 
 /* Use only mapgml.c if WMS or WFS is available */
 
@@ -903,7 +727,7 @@ gmlGeometryListObj *msGMLGetGeometries(layerObj *layer, const char *metadata_nam
   geometryList->numgeometries = 0;
 
   if((value = msOWSLookupMetadata(&(layer->metadata), metadata_namespaces, "geometries")) != NULL) {
-    names = split(value, ',', &numnames);
+    names = msStringSplit(value, ',', &numnames);
 
     /* allocation an array of gmlGeometryObj's */
     geometryList->numgeometries = numnames;
@@ -926,7 +750,7 @@ gmlGeometryListObj *msGMLGetGeometries(layerObj *layer, const char *metadata_nam
         char **occur;
         int numoccur;
 
-        occur = split(value, ',', &numoccur);
+        occur = msStringSplit(value, ',', &numoccur);
         if(numoccur == 2) { /* continue (TODO: throw an error if != 2) */
           geometry->occurmin = atof(occur[0]);
           if(strcasecmp(occur[1], "UNBOUNDED") == 0)
@@ -978,15 +802,15 @@ gmlItemListObj *msGMLGetItems(layerObj *layer, const char *metadata_namespaces)
 
   /* get a list of items that should be included in output */
   if((value = msOWSLookupMetadata(&(layer->metadata), metadata_namespaces, "include_items")) != NULL)  
-    incitems = split(value, ',', &numincitems);
+    incitems = msStringSplit(value, ',', &numincitems);
 
   /* get a list of items that should be excluded in output */
   if((value = msOWSLookupMetadata(&(layer->metadata), metadata_namespaces, "exclude_items")) != NULL)  
-    excitems = split(value, ',', &numexcitems);
+    excitems = msStringSplit(value, ',', &numexcitems);
 
   /* get a list of items that need don't get encoded */
   if((value = msOWSLookupMetadata(&(layer->metadata), metadata_namespaces, "xml_items")) != NULL)  
-    xmlitems = split(value, ',', &numxmlitems);
+    xmlitems = msStringSplit(value, ',', &numxmlitems);
 
   /* allocate memory and iinitialize the item collection */
   itemList = (gmlItemListObj *) malloc(sizeof(gmlItemListObj));
@@ -1106,8 +930,8 @@ static void msGMLWriteItem(FILE *stream, gmlItemObj *item, char *value, const ch
     char *tag = NULL;
 
     tag = strdup(item->template);
-    tag = gsub(tag, "$value", encoded_value);
-    if(namespace) tag = gsub(tag, "$namespace", namespace);
+    tag = msReplaceSubstring(tag, "$value", encoded_value);
+    if(namespace) tag = msReplaceSubstring(tag, "$namespace", namespace);
     msIO_fprintf(stream, "%s%s\n", tab, tag);
     free(tag);
   }
@@ -1134,9 +958,9 @@ gmlNamespaceListObj *msGMLGetNamespaces(webObj *web, const char *metadata_namesp
   namespaceList->namespaces = NULL;
   namespaceList->numnamespaces = 0; 
 
-  /* list of constants (TODO: make this automatic by parsing metadata) */
+  /* list of namespaces (TODO: make this automatic by parsing metadata) */
   if((value = msOWSLookupMetadata(&(web->metadata), metadata_namespaces, "external_namespace_prefixes")) != NULL) {
-    prefixes = split(value, ',', &numprefixes);
+    prefixes = msStringSplit(value, ',', &numprefixes);
 
     /* allocation an array of gmlNamespaceObj's */
     namespaceList->numnamespaces = numprefixes;
@@ -1198,7 +1022,7 @@ gmlConstantListObj *msGMLGetConstants(layerObj *layer, const char *metadata_name
 
   /* list of constants (TODO: make this automatic by parsing metadata) */
   if((value = msOWSLookupMetadata(&(layer->metadata), metadata_namespaces, "constants")) != NULL) {
-    names = split(value, ',', &numnames);
+    names = msStringSplit(value, ',', &numnames);
 
     /* allocation an array of gmlConstantObj's */
     constantList->numconstants = numnames;
@@ -1281,7 +1105,7 @@ gmlGroupListObj *msGMLGetGroups(layerObj *layer, const char *metadata_namespaces
 
   /* list of groups (TODO: make this automatic by parsing metadata) */
   if((value = msOWSLookupMetadata(&(layer->metadata), metadata_namespaces, "groups")) != NULL) {
-    names = split(value, ',', &numnames);
+    names = msStringSplit(value, ',', &numnames);
 
     /* allocation an array of gmlGroupObj's */
     groupList->numgroups = numnames;
@@ -1297,7 +1121,7 @@ gmlGroupListObj *msGMLGetGroups(layerObj *layer, const char *metadata_namespaces
 
       snprintf(tag, 64, "%s_group", group->name);
       if((value = msOWSLookupMetadata(&(layer->metadata), metadata_namespaces, tag)) != NULL)
-        group->items = split(value, ',', &group->numitems);
+        group->items = msStringSplit(value, ',', &group->numitems);
 
       snprintf(tag, 64, "%s_type", group->name);
       if((value = msOWSLookupMetadata(&(layer->metadata), metadata_namespaces, tag)) != NULL) 
@@ -1424,7 +1248,7 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 
   /* step through the layers looking for query results */
   for(i=0; i<map->numlayers; i++) {
-    lp = &(map->layers[map->layerorder[i]]);
+    lp = (GET_LAYER(map, map->layerorder[i]));
 
     if(lp->dump == MS_TRUE && lp->resultcache && lp->resultcache->numresults > 0) { /* found results */
 
@@ -1444,10 +1268,10 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
       if(status != MS_SUCCESS) return(status);
 
       /* populate item and group metadata structures (TODO: test for NULLs here, shouldn't happen) */
-      itemList = msGMLGetItems(lp, namespaces);
-      constantList = msGMLGetConstants(lp, namespaces);
-      groupList = msGMLGetGroups(lp, namespaces);      
-      geometryList = msGMLGetGeometries(lp, namespaces);
+      itemList = msGMLGetItems(lp, "G");
+      constantList = msGMLGetConstants(lp, "G");
+      groupList = msGMLGetGroups(lp, "G");      
+      geometryList = msGMLGetGeometries(lp, "G");
 
       for(j=0; j<lp->resultcache->numresults; j++) {
         status = msLayerGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);
@@ -1484,21 +1308,21 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 
         }
 
-        /* write the item/values */
+        /* write any item/values */
         for(k=0; k<itemList->numitems; k++) {
           item = &(itemList->items[k]);  
           if(msItemInGroups(item->name, groupList) == MS_FALSE) 
             msGMLWriteItem(stream, item, shape.values[k], NULL, "\t\t\t");
         }
 
-        /* write the constants */
+        /* write any constants */
         for(k=0; k<constantList->numconstants; k++) {
           constant = &(constantList->constants[k]);  
           if(msItemInGroups(constant->name, groupList) == MS_FALSE) 
             msGMLWriteConstant(stream, constant, NULL, "\t\t\t");
         }
 
-        /* write the groups */
+        /* write any groups */
         for(k=0; k<groupList->numgroups; k++)
           msGMLWriteGroup(stream, &(groupList->groups[k]), &shape, itemList, constantList, NULL, "\t\t\t");
 
@@ -1575,7 +1399,7 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
   /* step through the layers looking for query results */
   for(i=0; i<map->numlayers; i++) {
 
-    lp = &(map->layers[map->layerorder[i]]);
+    lp = GET_LAYER(map, map->layerorder[i]);
 
     if(lp->dump == MS_TRUE && lp->resultcache && lp->resultcache->numresults > 0)  { /* found results */
       char *layerName;      
@@ -1603,23 +1427,17 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
           }
         }
 
-        /* Produce a warning if a featureid was set but the corresponding
-         * item is not found. 
-         */
-        if (featureIdIndex == -1) {
-            msIO_fprintf(stream, "<!-- WARNING: FeatureId item '%s' not found in typename '%s'. -->\n", value, lp->name);
-        }
+        /* Produce a warning if a featureid was set but the corresponding item is not found. */
+        if (featureIdIndex == -1)
+          msIO_fprintf(stream, "<!-- WARNING: FeatureId item '%s' not found in typename '%s'. -->\n", value, lp->name);
       }
 
       /* populate item and group metadata structures (TODO: test for NULLs here, shouldn't happen) */
-      itemList = msGMLGetItems(lp, "OFG");
-      constantList = msGMLGetConstants(lp, "OFG");
-      groupList = msGMLGetGroups(lp, "OFG");
-      geometryList = msGMLGetGeometries(lp, "OFG");
+      itemList = msGMLGetItems(lp, "G");
+      constantList = msGMLGetConstants(lp, "G");
+      groupList = msGMLGetGroups(lp, "G");
+      geometryList = msGMLGetGeometries(lp, "G");
 
-      /* set the layer name */
-      /* value = msOWSLookupMetadata(&(lp->metadata), "OFG", "layername");
-         if(!value) value = lp->name; */
       if (namespace_prefix) {
         layerName = (char *) malloc(strlen(namespace_prefix)+strlen(lp->name)+2);
         sprintf(layerName, "%s:%s", namespace_prefix, lp->name);
@@ -1641,23 +1459,19 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
         /* 
         ** start this feature 
         */
-
         msIO_fprintf(stream, "    <gml:featureMember>\n");
         if(msIsXMLTagValid(layerName) == MS_FALSE)
           msIO_fprintf(stream, "<!-- WARNING: The value '%s' is not valid in a XML tag context. -->\n", layerName);
-        if(featureIdIndex != -1) 
-        {
-            if (outputformat == OWS_GML2)
-                msIO_fprintf(stream, "      <%s fid=\"%s\">\n", layerName, shape.values[featureIdIndex]);
-            else  /* OWS_GML3 */
-                msIO_fprintf(stream, "      <%s gml:id=\"%s\">\n", layerName, shape.values[featureIdIndex]);
-        }
-        else
+        if(featureIdIndex != -1) {
+          if(outputformat == OWS_GML2)
+            msIO_fprintf(stream, "      <%s fid=\"%s.%s\">\n", layerName, lp->name, shape.values[featureIdIndex]);
+          else  /* OWS_GML3 */
+            msIO_fprintf(stream, "      <%s gml:id=\"%s.%s\">\n", layerName, lp->name, shape.values[featureIdIndex]);
+        } else
           msIO_fprintf(stream, "      <%s>\n", layerName);
                     
         /* write the feature geometry and bounding box */
         if(!(geometryList && geometryList->numgeometries == 1 && strcasecmp(geometryList->geometries[0].name, "none") == 0)) {
-
 #ifdef USE_PROJ
           if(msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), "FGO", MS_TRUE)) { /* use the map projection first*/
             gmlWriteBounds(stream, outputformat, &(shape.bounds), msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), "FGO", MS_TRUE), "        ");
@@ -1672,21 +1486,21 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
 #endif
         }
 
-        /* write the item/values */
+        /* write any item/values */
         for(k=0; k<itemList->numitems; k++) {
           item = &(itemList->items[k]);  
           if(msItemInGroups(item->name, groupList) == MS_FALSE) 
             msGMLWriteItem(stream, item, shape.values[k], namespace_prefix, "        ");
         }
 
-        /* write the constants */
+        /* write any constants */
         for(k=0; k<constantList->numconstants; k++) {
           constant = &(constantList->constants[k]);  
           if(msItemInGroups(constant->name, groupList) == MS_FALSE) 
             msGMLWriteConstant(stream, constant, namespace_prefix, "        ");
         }
 
-        /* write the groups */
+        /* write any groups */
         for(k=0; k<groupList->numgroups; k++)
           msGMLWriteGroup(stream, &(groupList->groups[k]), &shape, itemList, constantList, namespace_prefix, "        ");
 
