@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: maphttp.c 5395 2006-04-17 19:06:49Z dan $
+ * $Id: maphttp.c 6384 2007-07-24 16:35:35Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Utility functions to access files via HTTP (requires libcurl)
@@ -15,8 +15,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  * 
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  * 
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -24,79 +24,7 @@
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
- * DEALINGS IN THE SOFTWARE.
- **********************************************************************
- * $Log$
- * Revision 1.22  2006/04/17 19:06:49  dan
- * Set User-Agent in HTTP headers of client WMS/WFS connections (bug 1749)
- *
- * Revision 1.21  2005/07/15 13:37:47  frank
- * Acquire TLOCK_OWS around whole func in msHTTPCleanup().
- *
- * Revision 1.20  2005/07/14 21:43:26  hobu
- * Fix bug 1417 msHTTPInit() not ever called
- *
- * Revision 1.19  2005/05/13 17:23:34  dan
- * First pass at properly handling XML exceptions from CONNECTIONTYPE WMS
- * layers. Still needs some work. (bug 1246)
- *
- * Revision 1.18  2005/02/18 03:06:45  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.17  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.16  2004/03/29 14:41:56  dan
- * Use CURL's internal timer instead of custom gettimeofday() calls for
- * timing WMS/WFS requests
- *
- * Revision 1.15  2004/03/18 23:11:12  dan
- * Added detailed reporting (using msDebug) of layer rendering times
- *
- * Revision 1.14  2004/03/11 22:45:39  dan
- * Added pszPostContentType in httpRequestObj instead of using hardcoded
- * text/html mime type for all post requests.
- *
- * Revision 1.13  2003/09/19 21:54:19  assefa
- * Add support fot the Post request.
- *
- * Revision 1.12  2003/06/12 20:35:16  dan
- * Fixed test on result of curl_easy_init() in msHTTPExecuteRequests()
- *
- * Revision 1.11  2003/04/23 21:32:25  dan
- * More fixes to return value and error reporting in msHTTPExecuteRequests()
- *
- * Revision 1.10  2003/04/23 15:06:56  dan
- * Added better error reporting for timeout situations
- *
- * Revision 1.9  2003/03/26 20:24:38  dan
- * Do not call msDebug() unless debug flag is turned on
- *
- * Revision 1.8  2003/01/10 19:23:22  dan
- * Do not produce a fatal error if WMS layer download fails, just like in 3.6
- *
- * Revision 1.7  2003/01/08 22:19:35  assefa
- * Patch for windows with problems related to the select function.
- *
- * Revision 1.6  2003/01/07 23:47:00  assefa
- * Correct Compilation error.
- *
- * Revision 1.5  2002/12/19 06:30:59  dan
- * Enable caching WMS/WFS request using tmp filename built from URL
- *
- * Revision 1.4  2002/12/17 21:33:54  dan
- * Enable following redirections with libcurl (requires libcurl 7.10.1+)
- *
- * Revision 1.3  2002/12/16 21:32:59  dan
- * Made CURLOPT_NOSIGNAL setting optional for now to work with libcurl 7.9.6+
- *
- * Revision 1.2  2002/12/16 20:34:59  dan
- * Flush libwww and use libcurl instead for HTTP requests in WMS/WFS client
- *
- * Revision 1.1  2002/10/09 02:29:03  dan
- * Initial implementation of WFS client support.
- *
- **********************************************************************/
+ ****************************************************************************/
 
 /* For now this code is enabled only when WMS/WFS client is enabled.
  * This should be changed to a test on the presence of libcurl which
@@ -104,12 +32,12 @@
  */
 #if defined(USE_WMS_LYR) || defined(USE_WFS_LYR)
 
-#include "map.h"
+#include "mapserver.h"
 #include "maperror.h"
 #include "mapows.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: maphttp.c 5395 2006-04-17 19:06:49Z dan $")
+MS_CVSID("$Id: maphttp.c 6384 2007-07-24 16:35:35Z dmorissette $")
 
 #include <time.h>
 #ifndef _WIN32
@@ -736,6 +664,8 @@ int msHTTPGetFile(const char *pszGetUrl, const char *pszOutputFile,
         *pnHTTPStatus = pasReqInfo[0].nStatus;
         if (pasReqInfo[0].debug)
             msDebug("HTTP request failed for %s.\n", pszGetUrl);
+        msHTTPFreeRequestObj(pasReqInfo, 2);
+        free(pasReqInfo);
         return MS_FAILURE;
     }
 
