@@ -37,7 +37,7 @@
 #include <io.h>
 #endif
 
-MS_CVSID("$Id: mapimagemap.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: mapimagemap.c 7425 2008-03-03 20:05:24Z dmorissette $")
 
 #define MYDEBUG 0
 #define DEBUG_IF if (MYDEBUG > 2) 
@@ -2029,7 +2029,7 @@ DEBUG_IF printf("FLEN %d<BR>\n", (int)strlen(img->img.imagemap));
               {
                   snprintf(workbuffer, sizeof(workbuffer), "%s", img->img.imagemap+iIndice );
                   workbuffer[nSize-1] = '\0';
-                  msIO_fprintf(stream, workbuffer);
+                  msIO_fwrite(workbuffer, strlen(workbuffer), 1, stream);
                   iIndice +=nSize-1;
               }
               if (iIndice < size)
@@ -2039,7 +2039,7 @@ DEBUG_IF printf("FLEN %d<BR>\n", (int)strlen(img->img.imagemap));
               }
           }
           else
-            msIO_fprintf(stream, img->img.imagemap);
+              msIO_fwrite(img->img.imagemap, size, 1, stream);
 	  if( strcasecmp("OFF",msGetOutputFormatOption( format, "SKIPENDTAG", "OFF" )) == 0){
 		  if (dxf == 2)
 			  msIO_fprintf(stream, "END");
