@@ -14,8 +14,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -24,62 +24,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.16  2006/08/22 23:07:52  hobu
- * cast to a const char* the first argument to strncasecmp
- *
- * Revision 1.15  2006/08/09 14:27:00  frank
- * fixed data length handling in strip content header
- *
- * Revision 1.14  2006/07/13 20:18:12  frank
- * added msIO_stripStdoutBufferContentType
- *
- * Revision 1.13  2006/07/06 21:35:29  frank
- * fixed statement order to be valid C
- *
- * Revision 1.12  2006/07/05 05:54:53  frank
- * implement per-thread io contexts
- *
- * Revision 1.11  2006/06/19 15:13:37  frank
- * add io context labelling, avoid depending on function pointer compares
- *
- * Revision 1.10  2006/05/22 19:20:59  frank
- * added some of the RFC 16 entry points
- *
- * Revision 1.9  2006/05/08 20:28:43  frank
- * force stdin into binary mode when reading from stdin on win32 (bug 1768)
- *
- * Revision 1.8  2005/07/25 20:56:00  frank
- * Undef msIO_fread() too.
- *
- * Revision 1.7  2005/07/22 17:26:11  frank
- * bug 1259: fixed POST support in fastcgi mode
- *
- * Revision 1.6  2004/11/04 21:13:02  frank
- * ensure we include io.h on win32
- *
- * Revision 1.5  2004/11/04 21:06:09  frank
- * centralize 'stdout binary mode setting' for win32, add for gdal output
- *
- * Revision 1.4  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.3  2004/10/08 19:11:39  frank
- * ensure msIO_vfprintf is undefed
- *
- * Revision 1.2  2004/10/05 04:23:10  sdlime
- * Added include for stdarg.h to mapio.c.
- *
- * Revision 1.1  2004/10/01 19:03:35  frank
- * New
- *
- */
+ ****************************************************************************/
 
 #include <stdarg.h>
 
-#include "map.h"
+#include "mapserver.h"
 #include "mapthread.h"
 
 #ifdef _WIN32
@@ -87,7 +36,7 @@
 #include <io.h>
 #endif
 
-MS_CVSID("$Id: mapio.c 5628 2006-08-22 23:07:52Z hobu $")
+MS_CVSID("$Id: mapio.c 6507 2007-08-08 17:39:41Z dmorissette $")
 
 static int is_msIO_initialized = MS_FALSE;
 
@@ -311,7 +260,10 @@ int msIO_printf( const char *format, ... )
     va_end( args );
 
     if( return_val < 0 || return_val >= sizeof(workBuf) )
+    {
+        msSetError(MS_MISCERR, "Possible buffer overrun.", "msIO_printf()");
         return -1;
+    }
 
     context = msIO_getHandler( stdout );
     if( context == NULL )
@@ -342,7 +294,10 @@ int msIO_fprintf( FILE *fp, const char *format, ... )
     va_end( args );
 
     if( return_val < 0 || return_val >= sizeof(workBuf) )
+    {
+        msSetError(MS_MISCERR, "Possible buffer overrun.", "msIO_fprintf()");
         return -1;
+    }
 
     context = msIO_getHandler( fp );
     if( context == NULL )
@@ -369,7 +324,10 @@ int msIO_vfprintf( FILE *fp, const char *format, va_list ap )
 #endif
 
     if( return_val < 0 || return_val >= sizeof(workBuf) )
+    {
+        msSetError(MS_MISCERR, "Possible buffer overrun.", "msIO_vfprintf()");
         return -1;
+    }
 
     context = msIO_getHandler( fp );
     if( context == NULL )
