@@ -24,74 +24,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.112  2006/08/22 13:56:58  hobu
- * ssh!  comment out "tile;" which was unused and
- * causing compiler to complain
- *
- * Revision 1.111  2006/04/27 04:05:17  sdlime
- * Initial support for relative coordinates. (bug 1547)
- *
- * Revision 1.110  2006/03/28 17:31:30  assefa
- * When setting the time filter, do not override existing Filter paramter
- * if possible (Bug 1261).
- *
- * Revision 1.109  2006/01/31 17:06:10  assefa
- * Add connectiontype initialization logic when the layer's virtual
- * table is initialized (Bug 1615)
- *
- * Revision 1.108  2005/10/29 02:03:43  jani
- * MS RFC 8: Pluggable External Feature Layer Providers (bug 1477).
- *
- * Revision 1.107  2005/10/28 01:09:41  jani
- * MS RFC 3: Layer vtable architecture (bug 1477)
- *
- * Revision 1.106  2005/07/16 19:07:38  jerryp
- * Bug 1420: PostGIS connector no longer needs two layer close functions.
- *
- * Revision 1.105  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.104  2005/04/25 06:41:56  sdlime
- * Applied Bill's newest gradient patch, more concise in the mapfile and potential to use via MapScript.
- *
- * Revision 1.103  2005/04/15 17:10:36  sdlime
- * Applied Bill Benko's patch for bug 1305, gradient support.
- *
- * Revision 1.102  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.101  2005/01/12 21:11:21  frank
- * removed LABELS_ROTATE_WITH_MAP, rotate labels if angle!=0 or labelangleitem
- *
- * Revision 1.100  2005/01/11 00:24:59  frank
- * added msLayerLabelsRotateWithMap
- *
- * Revision 1.99  2004/11/16 19:18:43  assefa
- * Make sure that the timestring is complete for pg layers (Bug 837)
- *
- * Revision 1.98  2004/11/15 21:11:23  dan
- * Moved the layer->getExtent() logic down to msLayerGetExtent() (bug 1051)
- *
- * Revision 1.97  2004/11/15 20:35:02  dan
- * Added msLayerIsOpen() to all vector layer types (bug 1051)
- *
- * Revision 1.96  2004/10/21 10:54:17  assefa
- * Add postgis date_trunc support.
- *
- * Revision 1.95  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "maptime.h"
 #include "mapogcfilter.h"
 
 #include <assert.h>
-MS_CVSID("$Id: maplayer.c 5621 2006-08-22 13:56:58Z hobu $")
+MS_CVSID("$Id: maplayer.c 6571 2007-08-13 18:19:58Z hobu $")
 
 
 static int populateVirtualTable(layerVTableObj *vtable);
@@ -320,7 +260,7 @@ static int string2list(char **list, int *listsize, char *string)
   int i;
 
   for(i=0; i<(*listsize); i++)
-    if(strcmp(list[i], string) == 0) { 
+    if(strcasecmp(list[i], string) == 0) { 
         /* printf("string2list (duplicate): %s %d\n", string, i); */
       return(i);
     }
@@ -388,7 +328,7 @@ int msLayerWhichItemsNew(layerObj *layer, int classify, int annotate, char *meta
 
   /* allocate space for the various item lists */
   if(classify && layer->filter.type == MS_EXPRESSION) { 
-    numchars = countChars(layer->filter.string, '[');
+    numchars = msCountChars(layer->filter.string, '[');
     if(numchars > 0) {
       layer->filter.items = (char **)calloc(numchars, sizeof(char *)); /* should be more than enough space */
       if(!(layer->filter.items)) {
@@ -412,20 +352,18 @@ int msLayerWhichItemsNew(layerObj *layer, int classify, int annotate, char *meta
 /*
 ** This function builds a list of items necessary to draw or query a particular layer by
 ** examining the contents of the various xxxxitem parameters and expressions. That list is
-** then used to set the iteminfo variable. SDE is kinda special, we always need to retrieve
-** the shape column and a virtual record number column. Most sources should not have to 
-** modify this function.
+** then used to set the iteminfo variable. 
 */
 int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadata) 
 {
-  int i, j, rv;
+  int i, j, k, rv;
   int nt=0, ne=0;
 
-  if ( ! layer->vtable) {
-      rv =  msInitializeVirtualTable(layer);
-      if (rv != MS_SUCCESS) {
-            return rv;
-      }
+  if (!layer->vtable) {
+    rv =  msInitializeVirtualTable(layer);
+    if (rv != MS_SUCCESS) {
+      return rv;
+    }
   }
 
   /*  */
@@ -447,87 +385,95 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     layer->numitems = 0;
   }
 
+  /*
+  ** layer level counts
+  */
   if(classify) {
     if(layer->classitem) nt++;
     if(layer->filteritem) nt++;
 
-    for(i=0; i<layer->numclasses; i++) {
-      for(j=0; j<layer->class[i].numstyles; j++) {
-        if(layer->class[i].styles[j].angleitem) nt++;
-	if(layer->class[i].styles[j].sizeitem) nt++;
-	if(layer->class[i].styles[j].rangeitem) nt++;
-      }
-    }
-
     ne = 0;
     if(layer->filter.type == MS_EXPRESSION) {
-      ne = countChars(layer->filter.string, '[');
+      ne = msCountChars(layer->filter.string, '[');
       if(ne > 0) {
-	layer->filter.items = (char **)calloc(ne, sizeof(char *)); /* should be more than enough space */
-	if(!(layer->filter.items)) {
-	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-	  return(MS_FAILURE);
-	}
-	layer->filter.indexes = (int *)malloc(ne*sizeof(int));
-	if(!(layer->filter.indexes)) {
-	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-	  return(MS_FAILURE);
-	}
-	layer->filter.numitems = 0;
-	nt += ne;
+        layer->filter.items = (char **) calloc(ne, sizeof(char *)); /* should be more than enough space */
+        if(!(layer->filter.items)) {
+          msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
+          return(MS_FAILURE);
+        }
+        layer->filter.indexes = (int *) malloc(ne*sizeof(int));
+        if(!(layer->filter.indexes)) {
+          msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
+          return(MS_FAILURE);
+        }
+        layer->filter.numitems = 0;
+        nt += ne;
       }
     }
   }
 
   if(annotate) {
     if(layer->labelitem) nt++;
-    if(layer->labelsizeitem) nt++;
-    if(layer->labelangleitem) nt++;
   }
 
+  /*
+  ** class level counts
+  */
   for(i=0; i<layer->numclasses; i++) {
-    ne = 0;
-    if(classify && layer->class[i].expression.type == MS_EXPRESSION) { 
-      ne = countChars(layer->class[i].expression.string, '[');
-      if(ne > 0) {
-	layer->class[i].expression.items = (char **)calloc(ne, sizeof(char *)); /* should be more than enough space */
-	if(!(layer->class[i].expression.items)) {
-	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-	  return(MS_FAILURE);
-	}
-	layer->class[i].expression.indexes = (int *)malloc(ne*sizeof(int));
-	if(!(layer->class[i].expression.indexes)) {
-	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-	  return(MS_FAILURE);
-	}
-	layer->class[i].expression.numitems = 0;
-	nt += ne;
+    
+    if(classify) {
+      for(j=0; j<layer->class[i]->numstyles; j++) {
+        if(layer->class[i]->styles[j]->rangeitem) nt++;
+        nt += layer->class[i]->styles[j]->numbindings;
+      }
+
+      ne = 0;
+      if(layer->class[i]->expression.type == MS_EXPRESSION) { 
+        ne = msCountChars(layer->class[i]->expression.string, '[');
+        if(ne > 0) {
+          layer->class[i]->expression.items = (char **) calloc(ne, sizeof(char *)); /* should be more than enough space */
+          if(!(layer->class[i]->expression.items)) {
+            msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
+            return(MS_FAILURE);
+          }
+          layer->class[i]->expression.indexes = (int *) malloc(ne*sizeof(int));
+          if(!(layer->class[i]->expression.indexes)) {
+            msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
+            return(MS_FAILURE);
+          }
+          layer->class[i]->expression.numitems = 0;
+          nt += ne;
+        }
       }
     }
 
-    ne = 0;
-    if(annotate && layer->class[i].text.type == MS_EXPRESSION) { 
-      ne = countChars(layer->class[i].text.string, '[');
-      if(ne > 0) {
-	layer->class[i].text.items = (char **)calloc(ne, sizeof(char *)); /* should be more than enough space */
-	if(!(layer->class[i].text.items)) {
-	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-	  return(MS_FAILURE);
-	}
-	layer->class[i].text.indexes = (int *)malloc(ne*sizeof(int));
-	if(!(layer->class[i].text.indexes)) {
-	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-	  return(MS_FAILURE);
-	}
-	layer->class[i].text.numitems = 0;
-	nt += ne;
+    if(annotate) {
+      nt += layer->class[i]->label.numbindings;
+
+      ne = 0;
+      if(layer->class[i]->text.type == MS_EXPRESSION) { 
+        ne = msCountChars(layer->class[i]->text.string, '[');
+        if(ne > 0) {
+          layer->class[i]->text.items = (char **) calloc(ne, sizeof(char *)); /* should be more than enough space */
+          if(!(layer->class[i]->text.items)) {
+            msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
+            return(MS_FAILURE);
+          }
+          layer->class[i]->text.indexes = (int *) malloc(ne*sizeof(int));
+          if(!(layer->class[i]->text.indexes)) {
+            msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
+            return(MS_FAILURE);
+          }
+          layer->class[i]->text.numitems = 0;
+          nt += ne;
+        }
       }
     }
   }
 
   rv = layer->vtable->LayerCreateItems(layer, nt);
   if (rv != MS_SUCCESS) {
-      return rv;
+    return rv;
   }
 
   if(nt > 0) {
@@ -536,24 +482,24 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
       if(layer->filteritem) layer->filteritemindex = string2list(layer->items, &(layer->numitems), layer->filteritem);
 
       for(i=0; i<layer->numclasses; i++) {
-	for(j=0; j<layer->class[i].numstyles; j++) {
-	  if(layer->class[i].styles[j].angleitem) layer->class[i].styles[j].angleitemindex = string2list(layer->items, &(layer->numitems), layer->class[i].styles[j].angleitem);
-	  if(layer->class[i].styles[j].sizeitem) layer->class[i].styles[j].sizeitemindex = string2list(layer->items, &(layer->numitems), layer->class[i].styles[j].sizeitem); 
-	  if(layer->class[i].styles[j].rangeitem) layer->class[i].styles[j].rangeitemindex = string2list(layer->items, &(layer->numitems), layer->class[i].styles[j].rangeitem); 
-	}
+        if(layer->class[i]->expression.type == MS_EXPRESSION) expression2list(layer->items, &(layer->numitems), &(layer->class[i]->expression));
+        for(j=0; j<layer->class[i]->numstyles; j++) {
+          if(layer->class[i]->styles[j]->rangeitem) layer->class[i]->styles[j]->rangeitemindex = string2list(layer->items, &(layer->numitems), layer->class[i]->styles[j]->rangeitem); 
+	  for(k=0; k<MS_STYLE_BINDING_LENGTH; k++)
+	    if(layer->class[i]->styles[j]->bindings[k].item) layer->class[i]->styles[j]->bindings[k].index = string2list(layer->items, &(layer->numitems), layer->class[i]->styles[j]->bindings[k].item);
+        }
       }
 
       if(layer->filter.type == MS_EXPRESSION) expression2list(layer->items, &(layer->numitems), &(layer->filter));      
     }
+
     if(annotate) {
       if(layer->labelitem) layer->labelitemindex = string2list(layer->items, &(layer->numitems), layer->labelitem);
-      if(layer->labelsizeitem) layer->labelsizeitemindex = string2list(layer->items, &(layer->numitems), layer->labelsizeitem);
-      if(layer->labelangleitem) layer->labelangleitemindex = string2list(layer->items, &(layer->numitems), layer->labelangleitem);
-    }  
-
-    for(i=0; i<layer->numclasses; i++) {
-      if(classify && layer->class[i].expression.type == MS_EXPRESSION) expression2list(layer->items, &(layer->numitems), &(layer->class[i].expression));
-      if(annotate && layer->class[i].text.type == MS_EXPRESSION) expression2list(layer->items, &(layer->numitems), &(layer->class[i].text));
+      for(i=0; i<layer->numclasses; i++) {
+        if(layer->class[i]->text.type == MS_EXPRESSION) expression2list(layer->items, &(layer->numitems), &(layer->class[i]->text));
+        for(k=0; k<MS_LABEL_BINDING_LENGTH; k++)
+          if(layer->class[i]->label.bindings[k].item) layer->class[i]->label.bindings[k].index = string2list(layer->items, &(layer->numitems), layer->class[i]->label.bindings[k].item);
+      }
     }
   }
 
@@ -563,7 +509,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     int j;
     int bFound = 0;
       
-    tokens = split(metadata, ',', &n);
+    tokens = msStringSplit(metadata, ',', &n);
     if(tokens) {
       for(i=0; i<n; i++) {
         bFound = 0;
@@ -766,9 +712,9 @@ makeTimeFilter(layerObj *lp,
                the time. If not just free it */
             if (lp->filter.type == MS_EXPRESSION)
             {
-                pszBuffer = strcatalloc(pszBuffer, "((");
-                pszBuffer = strcatalloc(pszBuffer, lp->filter.string);
-                pszBuffer = strcatalloc(pszBuffer, ") and ");
+                pszBuffer = msStringConcatenate(pszBuffer, "((");
+                pszBuffer = msStringConcatenate(pszBuffer, lp->filter.string);
+                pszBuffer = msStringConcatenate(pszBuffer, ") and ");
             }
             else
             {
@@ -776,37 +722,37 @@ makeTimeFilter(layerObj *lp,
             }
         }
         
-        pszBuffer = strcatalloc(pszBuffer, "(");
+        pszBuffer = msStringConcatenate(pszBuffer, "(");
         if (addtimebacktics)
-          pszBuffer = strcatalloc(pszBuffer,  "`");
+          pszBuffer = msStringConcatenate(pszBuffer,  "`");
 
         if (addtimebacktics)
-           pszBuffer = strcatalloc(pszBuffer, "[");
-        pszBuffer = strcatalloc(pszBuffer, (char *)timefield);
+           pszBuffer = msStringConcatenate(pszBuffer, "[");
+        pszBuffer = msStringConcatenate(pszBuffer, (char *)timefield);
         if (addtimebacktics)
-          pszBuffer = strcatalloc(pszBuffer, "]");
+          pszBuffer = msStringConcatenate(pszBuffer, "]");
         if (addtimebacktics)
-          pszBuffer = strcatalloc(pszBuffer,  "`");
+          pszBuffer = msStringConcatenate(pszBuffer,  "`");
 
          
-        pszBuffer = strcatalloc(pszBuffer, " = ");
+        pszBuffer = msStringConcatenate(pszBuffer, " = ");
         if (addtimebacktics)
-          pszBuffer = strcatalloc(pszBuffer,  "`");
+          pszBuffer = msStringConcatenate(pszBuffer,  "`");
         else
-          pszBuffer = strcatalloc(pszBuffer,  "'");
+          pszBuffer = msStringConcatenate(pszBuffer,  "'");
 
-        pszBuffer = strcatalloc(pszBuffer, (char *)timestring);
+        pszBuffer = msStringConcatenate(pszBuffer, (char *)timestring);
         if (addtimebacktics)
-          pszBuffer = strcatalloc(pszBuffer,  "`");
+          pszBuffer = msStringConcatenate(pszBuffer,  "`");
         else
-          pszBuffer = strcatalloc(pszBuffer,  "'");
+          pszBuffer = msStringConcatenate(pszBuffer,  "'");
 
-        pszBuffer = strcatalloc(pszBuffer, ")");
+        pszBuffer = msStringConcatenate(pszBuffer, ")");
         
         /* if there was a filter, It was concatenate with an And ans should be closed*/
         if(&lp->filter && lp->filter.type == MS_EXPRESSION)
         {
-            pszBuffer = strcatalloc(pszBuffer, ")");
+            pszBuffer = msStringConcatenate(pszBuffer, ")");
         }
 
         loadExpressionString(&lp->filter, pszBuffer);
@@ -817,7 +763,7 @@ makeTimeFilter(layerObj *lp,
         return MS_TRUE;
     }
     
-    atimes = split (timestring, ',', &numtimes);
+    atimes = msStringSplit(timestring, ',', &numtimes);
     if (atimes == NULL || numtimes < 1)
       return MS_FALSE;
 
@@ -825,9 +771,9 @@ makeTimeFilter(layerObj *lp,
     {
         if (&lp->filter && lp->filter.type == MS_EXPRESSION)
         {
-            pszBuffer = strcatalloc(pszBuffer, "((");
-            pszBuffer = strcatalloc(pszBuffer, lp->filter.string);
-            pszBuffer = strcatalloc(pszBuffer, ") and ");
+            pszBuffer = msStringConcatenate(pszBuffer, "((");
+            pszBuffer = msStringConcatenate(pszBuffer, lp->filter.string);
+            pszBuffer = msStringConcatenate(pszBuffer, ") and ");
             /*this flag is used to indicate that the buffer contains only the 
               existing filter. It is set to 0 when time filter parts are
               added to the buffer */
@@ -837,114 +783,114 @@ makeTimeFilter(layerObj *lp,
           freeExpression(&lp->filter);
 
         /* check to see if we have ranges by parsing the first entry */
-        tokens = split(atimes[0],  '/', &ntmp);
+        tokens = msStringSplit(atimes[0],  '/', &ntmp);
         if (ntmp == 2) /* ranges */
         {
             msFreeCharArray(tokens, ntmp);
             for (i=0; i<numtimes; i++)
             {
-                 tokens = split(atimes[i],  '/', &ntmp);
+                 tokens = msStringSplit(atimes[i],  '/', &ntmp);
                  if (ntmp == 2)
                  {
                      if (pszBuffer && strlen(pszBuffer) > 0 && bOnlyExistingFilter == 0)
-                       pszBuffer = strcatalloc(pszBuffer, " OR ");
+                       pszBuffer = msStringConcatenate(pszBuffer, " OR ");
                      else
-                       pszBuffer = strcatalloc(pszBuffer, "(");
+                       pszBuffer = msStringConcatenate(pszBuffer, "(");
 
                      bOnlyExistingFilter = 0;
 
-                     pszBuffer = strcatalloc(pszBuffer, "(");
+                     pszBuffer = msStringConcatenate(pszBuffer, "(");
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer,  "`");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "`");
 
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer, "[");
-                     pszBuffer = strcatalloc(pszBuffer, (char *)timefield);
+                       pszBuffer = msStringConcatenate(pszBuffer, "[");
+                     pszBuffer = msStringConcatenate(pszBuffer, (char *)timefield);
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer, "]");
+                       pszBuffer = msStringConcatenate(pszBuffer, "]");
                      
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer,  "`");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "`");
 
-                     pszBuffer = strcatalloc(pszBuffer, " >= ");
+                     pszBuffer = msStringConcatenate(pszBuffer, " >= ");
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer,  "`");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "`");
                      else
-                       pszBuffer = strcatalloc(pszBuffer,  "'");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "'");
 
-                     pszBuffer = strcatalloc(pszBuffer, tokens[0]);
+                     pszBuffer = msStringConcatenate(pszBuffer, tokens[0]);
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer,  "`");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "`");
                      else
-                       pszBuffer = strcatalloc(pszBuffer,  "'");
-                     pszBuffer = strcatalloc(pszBuffer, " AND ");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "'");
+                     pszBuffer = msStringConcatenate(pszBuffer, " AND ");
 
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer,  "`");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "`");
 
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer, "[");
-                     pszBuffer = strcatalloc(pszBuffer, (char *)timefield);
+                       pszBuffer = msStringConcatenate(pszBuffer, "[");
+                     pszBuffer = msStringConcatenate(pszBuffer, (char *)timefield);
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer, "]");
+                       pszBuffer = msStringConcatenate(pszBuffer, "]");
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer,  "`");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "`");
 
-                     pszBuffer = strcatalloc(pszBuffer, " <= ");
+                     pszBuffer = msStringConcatenate(pszBuffer, " <= ");
                      
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer,  "`");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "`");
                      else
-                       pszBuffer = strcatalloc(pszBuffer,  "'");
-                     pszBuffer = strcatalloc(pszBuffer, tokens[1]);
+                       pszBuffer = msStringConcatenate(pszBuffer,  "'");
+                     pszBuffer = msStringConcatenate(pszBuffer, tokens[1]);
                      if (addtimebacktics)
-                       pszBuffer = strcatalloc(pszBuffer,  "`");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "`");
                      else
-                       pszBuffer = strcatalloc(pszBuffer,  "'");
-                     pszBuffer = strcatalloc(pszBuffer, ")");
+                       pszBuffer = msStringConcatenate(pszBuffer,  "'");
+                     pszBuffer = msStringConcatenate(pszBuffer, ")");
                  }
                  
                   msFreeCharArray(tokens, ntmp);
             }
             if (pszBuffer && strlen(pszBuffer) > 0 && bOnlyExistingFilter == 0)
-              pszBuffer = strcatalloc(pszBuffer, ")");
+              pszBuffer = msStringConcatenate(pszBuffer, ")");
         }
         else if (ntmp == 1) /* multiple times */
         {
             msFreeCharArray(tokens, ntmp);
-            pszBuffer = strcatalloc(pszBuffer, "(");
+            pszBuffer = msStringConcatenate(pszBuffer, "(");
             for (i=0; i<numtimes; i++)
             {
                 if (i > 0)
-                  pszBuffer = strcatalloc(pszBuffer, " OR ");
+                  pszBuffer = msStringConcatenate(pszBuffer, " OR ");
 
-                pszBuffer = strcatalloc(pszBuffer, "(");
+                pszBuffer = msStringConcatenate(pszBuffer, "(");
                 if (addtimebacktics)
-                  pszBuffer = strcatalloc(pszBuffer, "`");
+                  pszBuffer = msStringConcatenate(pszBuffer, "`");
                   
                 if (addtimebacktics)
-                  pszBuffer = strcatalloc(pszBuffer, "[");
-                pszBuffer = strcatalloc(pszBuffer, (char *)timefield);
+                  pszBuffer = msStringConcatenate(pszBuffer, "[");
+                pszBuffer = msStringConcatenate(pszBuffer, (char *)timefield);
                 if (addtimebacktics)
-                  pszBuffer = strcatalloc(pszBuffer, "]");
+                  pszBuffer = msStringConcatenate(pszBuffer, "]");
 
                 if (addtimebacktics)
-                  pszBuffer = strcatalloc(pszBuffer, "`");
+                  pszBuffer = msStringConcatenate(pszBuffer, "`");
 
-                pszBuffer = strcatalloc(pszBuffer, " = ");
+                pszBuffer = msStringConcatenate(pszBuffer, " = ");
                   
                 if (addtimebacktics)
-                  pszBuffer = strcatalloc(pszBuffer, "`");
+                  pszBuffer = msStringConcatenate(pszBuffer, "`");
                 else
-                  pszBuffer = strcatalloc(pszBuffer,  "'");
-                pszBuffer = strcatalloc(pszBuffer, atimes[i]);
+                  pszBuffer = msStringConcatenate(pszBuffer,  "'");
+                pszBuffer = msStringConcatenate(pszBuffer, atimes[i]);
                 if (addtimebacktics)
-                  pszBuffer = strcatalloc(pszBuffer,  "`");
+                  pszBuffer = msStringConcatenate(pszBuffer,  "`");
                 else
-                  pszBuffer = strcatalloc(pszBuffer,  "'");
-                pszBuffer = strcatalloc(pszBuffer, ")");
+                  pszBuffer = msStringConcatenate(pszBuffer,  "'");
+                pszBuffer = msStringConcatenate(pszBuffer, ")");
             } 
-            pszBuffer = strcatalloc(pszBuffer, ")");
+            pszBuffer = msStringConcatenate(pszBuffer, ")");
         }
         else
         {
@@ -958,7 +904,7 @@ makeTimeFilter(layerObj *lp,
         if (pszBuffer && strlen(pszBuffer) > 0)
         {
             if(&lp->filter && lp->filter.type == MS_EXPRESSION)
-              pszBuffer = strcatalloc(pszBuffer, ")");
+              pszBuffer = msStringConcatenate(pszBuffer, ")");
             /*
             if(lp->filteritem) 
               free(lp->filteritem);
