@@ -28,7 +28,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: maplegend.c 6615 2007-08-17 14:00:22Z dmorissette $")
+MS_CVSID("$Id: maplegend.c 6907 2007-09-25 12:53:12Z tbonfort $")
 
 #define PSF .8
 #define VMARGIN 5 /* margin at top and bottom of legend graphic */
@@ -267,7 +267,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
 
     if(!scale_independent && map->scaledenom > 0) {
       if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
-      if((lp->maxscaledenom > 0) && (map->scaledenom <= lp->maxscaledenom)) continue;
+      if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
  
     for(j=0;j<lp->numclasses;j++) {
@@ -277,7 +277,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
       if(!scale_independent && map->scaledenom > 0) {  /* verify class scale here */
 	if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom > lp->class[j]->maxscaledenom))
 	  continue;
-	if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom <= lp->class[j]->maxscaledenom))
+	if((lp->class[j]->minscaledenom > 0) && (map->scaledenom <= lp->class[j]->minscaledenom))
 	  continue;
       }
 
@@ -342,7 +342,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
     if(!scale_independent && map->scaledenom > 0) {
       if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom))
 	continue;
-      if((lp->maxscaledenom > 0) && (map->scaledenom <= lp->maxscaledenom))
+      if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom))
 	continue;
 
       /* Should we also consider lp->symbolscale? I don't think so. Showing the "standard" size makes the most sense. */
@@ -358,7 +358,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
       if(!scale_independent && map->scaledenom > 0) {  /* verify class scale here */
         if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom > lp->class[j]->maxscaledenom))
           continue;
-        if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom <= lp->class[j]->maxscaledenom))
+        if((lp->class[j]->minscaledenom > 0) && (map->scaledenom <= lp->class[j]->minscaledenom))
           continue;
       }
  
