@@ -24,39 +24,9 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.21  2006/05/26 21:46:27  tamas
- * Moving layerObj.sameconnection and msCheckConnection() internal to the MYGIS data provider.
- *
- * Revision 1.20  2005/10/28 01:09:41  jani
- * MS RFC 3: Layer vtable architecture (bug 1477)
- *
- * Revision 1.19  2005/06/14 16:03:33  dan
- * Updated copyright date to 2005
- *
- * Revision 1.18  2005/04/21 15:09:27  julien
- * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
- *
- * Revision 1.17  2005/04/14 15:17:14  julien
- * Bug 1244: Remove Z and M from point by default to gain performance.
- *
- * Revision 1.16  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.15  2004/12/07 16:56:31  dan
- * Fixed build error introduced by adding msLayerIsOpen() (bug 1116)
- *
- * Revision 1.14  2004/11/15 20:35:02  dan
- * Added msLayerIsOpen() to all vector layer types (bug 1051)
- *
- * Revision 1.13  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
 #include <assert.h>
 #include <sys/types.h>
@@ -65,7 +35,7 @@
 #include <netinet/in.h>
 #endif
 
-MS_CVSID("$Id: mapmygis.c 5464 2006-05-26 21:46:27Z tamas $")
+MS_CVSID("$Id: mapmygis.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 #ifndef FLT_MAX
 #define FLT_MAX 25000000.0
@@ -341,7 +311,7 @@ layerObj* msMYGISCheckConnection(layerObj * layer) {
 
   /* TODO: there is an issue with layer order since it's possible that layers to be rendered out of order */
   for (i=0;i<layer->index;i++) { 	/* check all layers previous to this one */
-    lp = &(layer->map->layers[i]);
+    lp = &(GET_LAYER(layer->map, i));
 
     if (lp == layer) continue;
 
@@ -630,13 +600,7 @@ static int prep_DB(char	*geom_table,char  *geom_column,layerObj *layer, MYSQL_RE
 /* } */
 /* if (layer->labelitem) */
 /* sprintf(attribselect,"%s, feature.%s", attribselect, layer->labelitem); */
-/* if (layer->labelangleitem) */
-/* sprintf(attribselect,"%s, feature.%s", attribselect, layer->labelangleitem); */
-/* if (layer->labelsizeitem) */
-/* sprintf(attribselect,"%s, feature.%s", attribselect, layer->labelsizeitem); */
-/* if (layer->labelsizeitem) */
-/* sprintf(attribselect,"%s, feature.%s", attribselect, layer->labelsizeitem); */
-	sprintf(attribselect,", %s%s, %s%s, %s%s", layer->labelitem ? "feature." : "", layer->labelitem ? layer->labelitem: "''", layer->labelangleitem ? "feature." : "", layer->labelangleitem ? layer->labelangleitem: "''", layer->labelsizeitem ? "feature." : "", layer->labelsizeitem ? layer->labelsizeitem: "''");
+	sprintf(attribselect,", %s%s", layer->labelitem ? "feature." : "", layer->labelitem ? layer->labelitem: "''");
 	}
 	if (wkbdata){
 		layerinfo->attriboffset = 3;
@@ -1405,6 +1369,7 @@ int msMYGISLayerGetShapeRandom(layerObj *layer, shapeObj *shape, long *record)
 						force_to_polygons(row, layerinfo->query_result, shape, record);
 					break;
 				case MS_LAYER_ANNOTATION:
+				case MS_LAYER_CHART:
 				case MS_LAYER_QUERY:
 					result = wkbdata ?
 						wkb_dont_force(wkb, shape) :
@@ -1441,20 +1406,13 @@ int msMYGISLayerGetShapeRandom(layerObj *layer, shapeObj *shape, long *record)
 						    if (layer->labelitem && strlen(row[layerinfo->attriboffset+0]) > 0){
 						    	shape->values[layer->labelitemindex]=strdup(row[layerinfo->attriboffset+0]);
 						    }
-						    if (layer->labelangleitem && strlen(row[layerinfo->attriboffset+1]) > 0){
-						    	shape->values[layer->labelangleitemindex]=strdup(row[layerinfo->attriboffset+1]);
-						    }
-						    if (layer->labelsizeitem && strlen(row[layerinfo->attriboffset+2]) > 0){
-						    	shape->values[layer->labelsizeitemindex]=strdup(row[layerinfo->attriboffset+2]);
-						    }
 					} else {
 
 
 
 
 /* sprintf(tmpstr,"select attribute, value from shape_attr where shape_attr.shape='%d'", shape->index); */
-					sprintf(tmpstr,"SELECT %s, %s, %s FROM %s feature WHERE feature.id='%li'", layer->labelitem ? layer->labelitem: "''", layer->labelangleitem ? layer->labelangleitem: "''", layer->labelsizeitem ? layer->labelsizeitem: "''", layerinfo->feature ? layerinfo->feature : "feature", shape->index);
-/* sprintf(tmpstr,"SELECT %s, %s, %s FROM %s WHERE feature.id='%d' AND feature.GID=geometry.GID GROUP BY feature.id", layer->labelitem ? layer->labelitem: "''", layer->labelangleitem ? layer->labelangleitem: "''", layer->labelsizeitem ? layer->labelsizeitem: "''", layerinfo->table ? layerinfo->table : "feature", shape->index); */
+					sprintf(tmpstr,"SELECT %s FROM %s feature WHERE feature.id='%li'", layer->labelitem ? layer->labelitem: "''", layerinfo->feature ? layerinfo->feature : "feature", shape->index);
 
 					   if (layerinfo->query2_result != NULL) /* query leftover  */
 					   {
@@ -1484,16 +1442,10 @@ if (MYDEBUG)					printf("%s<BR>\n", tmpstr);
 								printf("attr_nullfetch(%s-%d/%d)<BR>\n",tmpstr,t,numrows2);
 /* return(MS_DONE); */
 							}
-/* printf("%s,%s,%s/%s,%s<BR>\n", layer->labelitem, layer->labelsizeitem, layer->labelangleitem, row_attr[0], row_attr[1]); */
+/* printf("%s/%s,%s<BR>\n", layer->labelitem, row_attr[0], row_attr[1]); */
 						    if (layer->labelitem && strlen(row_attr[0]) > 0){
 						    	shape->values[layer->labelitemindex]=strdup(row_attr[0]);
 						    }
-						    if (layer->labelangleitem && strlen(row_attr[1]) > 0){
-						    	shape->values[layer->labelangleitemindex]=strdup(row_attr[1]);
-						    }
-						    if (layer->labelsizeitem && strlen(row_attr[2]) > 0){
-						    	shape->values[layer->labelsizeitemindex]=strdup(row_attr[2]);
-						    }
 						}
 					   }
 
