@@ -34,7 +34,7 @@
 #  include "cpl_conv.h"
 #endif
 
-MS_CVSID("$Id: mapobject.c 6585 2007-08-15 03:30:43Z hobu $")
+MS_CVSID("$Id: mapobject.c 7249 2008-01-07 15:43:27Z tamas $")
 
 void freeWeb(webObj *web);
 void freeScalebar(scalebarObj *scalebar);
@@ -81,8 +81,7 @@ void msFreeMap(mapObj *map) {
   /*printf("msFreeMap(): maybe freeing map at %p count=%d.\n",map, map->refcount);*/
   if ( MS_REFCNT_DECR_IS_NOT_ZERO(map) ) { return; }
   if (map->debug >= MS_DEBUGLEVEL_VV)
-     msDebug("msFreeMap(): freeing map at %p.",map);
-  /*printf("msFreeMap(): freeing map at %p.\n",map);*/
+     msDebug("msFreeMap(): freeing map at %p.\n",map);
   
   msCloseConnections(map);
 
