@@ -14,8 +14,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -24,128 +24,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.36  2006/07/11 01:09:42  frank
- * msCloneOutputFormat() needs to propagate inmapfile flag (bug 1823).
- *
- * Revision 1.35  2005/11/10 15:41:16  frank
- * Added missing function header.
- *
- * Revision 1.34  2005/04/07 17:23:16  assefa
- * Remove #ifdef USE_SVG. It was added during development.
- *
- * Revision 1.33  2005/03/08 19:53:25  frank
- * adjusted png24 mimetype per bug 1231
- *
- * Revision 1.32  2005/02/08 21:36:23  frank
- * Return actual error code.
- *
- * Revision 1.31  2005/02/03 00:06:57  assefa
- * Add SVG function prototypes and related calls.
- *
- * Revision 1.30  2004/11/18 20:55:38  frank
- * In msOutputFormatValidate() we now ensure that GD/JPEG does not have
- * alpha enabled, or transparent turned on.   Bug 1073.
- *
- * Revision 1.29  2004/11/18 19:27:16  frank
- * Fixed formatting of debug message.
- *
- * Revision 1.28  2004/11/17 23:53:07  assefa
- * Advertize only gd and gdal formats for wms capabilities (Bug 455).
- *
- * Revision 1.27  2004/11/05 23:04:50  assefa
- * Change a bit test in function msGetOutputFormatMimeListGD.
- *
- * Revision 1.26  2004/11/05 16:29:40  assefa
- * add function msGetOutputFormatMimeListGD.
- *
- * Revision 1.25  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.24  2004/10/20 20:46:51  frank
- * msOutputFormatValidate() now also fixes stuff up including setting
- * imagemode to MS_RENDER_WITH_RAWDATA if appropriate, and setting
- * imagemode to MS_IMAGEMODE_RGBA from RGB if transparent enabled.
- * See Bugs 724 and 425/977.
- *
- * Revision 1.23  2004/04/23 16:44:54  frank
- * msRemoteOutputFormat() now frees output formats if refcount==0
- *
- * Revision 1.22  2004/03/08 04:28:46  sean
- * New functions to support mapscript interface to outputformatlist
- *
- * Revision 1.21  2004/03/05 05:57:04  frank
- * support multi-band rawmode output
- *
- * Revision 1.20  2003/07/08 12:36:11  frank
- * don't crash if format==NULL
- *
- * Revision 1.19  2003/07/03 03:01:58  assefa
- * Add support for saving the output formats.
- *
- * Revision 1.18  2003/03/14 13:12:11  attila
- * Introduce MySQL generic support, enabled with --with-mygis when configuring
- *
- * Revision 1.17  2003/02/20 19:49:14  frank
- * fixed implicit setting of JPEG quality from IMAGEQUALITY keyword
- *
- * Revision 1.16  2003/01/17 15:47:48  frank
- * ensure that msApplyDefaultOutputFormats() does not modify map->imagetype
- *
- * Revision 1.15  2003/01/11 23:25:48  frank
- * Bug 249: msApplyOutputFormat() will switch imagemode from RGB to RGBA if
- * transparency is requested.
- * Added msOutputFormatValidate() function.
- *
- * Revision 1.14  2003/01/11 17:32:27  frank
- * When GD2 isn't available create a PC256 mode JPEG driver by default
- *
- * Revision 1.13  2002/12/21 22:04:54  frank
- * I regressed the changes Julien made, and reimplemented the same effect
- * with the logic in msApplyDefaultOutputFormats().  Should be much safer now.
- *
- * Revision 1.11  2002/12/17 23:02:19  dan
- * I assigned the wrong value to map->imagetype.  Duh!
- *
- * Revision 1.10  2002/12/17 22:52:43  dan
- * Update value of map->imagetype in msSelectOutputFormat()
- *
- * Revision 1.9  2002/12/17 22:04:58  dan
- * Added PNG24 to the list of default output formats with GD2
- *
- * Revision 1.8  2002/11/19 14:38:39  frank
- * fixed bug in msApplyOutputFormat with transparent override
- *
- * Revision 1.7  2002/11/19 05:47:37  frank
- * cleanup memory leak
- *
- * Revision 1.6  2002/10/05 16:32:40  zak
- * updated e-mail address
- *
- * Revision 1.5  2002/10/04 21:15:59  assefa
- * Add PDF support using the new output architecture. Work done
- * by Zak James (zak-ms@aiya.dhs.org).
- *
- * Revision 1.4  2002/06/21 18:29:45  frank
- * Added support for GD/PC256 as a pseudo-driver
- *
- * Revision 1.3  2002/06/13 19:56:15  frank
- * don't automatically emit Imagine format support
- *
- * Revision 1.2  2002/06/11 20:45:22  frank
- * add renderer copy to output format clone function
- *
- * Revision 1.1  2002/06/11 13:49:52  frank
- * New
- *
- */
+ *****************************************************************************/
 
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapoutput.c 5560 2006-07-11 01:10:34Z frank $")
+MS_CVSID("$Id: mapoutput.c 6455 2007-08-03 14:18:54Z warmerdam $")
 
 static outputFormatObj *msAllocOutputFormat( mapObj *map, const char *name, 
                                              const char *driver );
@@ -297,11 +181,7 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
     {
         format = msAllocOutputFormat( map, "jpeg", driver );
         format->mimetype = strdup("image/jpeg");
-#if GD2_VERS > 1
         format->imagemode = MS_IMAGEMODE_RGB;
-#else
-        format->imagemode = MS_IMAGEMODE_PC256;
-#endif
         format->extension = strdup("jpg");
         format->renderer = MS_RENDER_WITH_GD;
     }
@@ -316,6 +196,28 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
         format->renderer = MS_RENDER_WITH_GD;
     }
 #endif
+
+#if defined(USE_AGG) && defined(USE_GD_PNG)
+    if( strcasecmp(driver,"AGG/PNG") == 0 )
+    {
+        format = msAllocOutputFormat( map, "aggpng24", driver );
+        format->mimetype = strdup("image/png; mode=24bit");
+        format->imagemode = MS_IMAGEMODE_RGB;
+        format->extension = strdup("png");
+        format->renderer = MS_RENDER_WITH_AGG;
+    }
+#endif /* USE_GD_PNG */
+
+#if defined(USE_AGG) && defined(USE_GD_JPEG)
+    if( strcasecmp(driver,"AGG/JPEG") == 0 )
+    {
+        format = msAllocOutputFormat( map, "aggjpeg", driver );
+        format->mimetype = strdup("image/jpeg");
+        format->imagemode = MS_IMAGEMODE_RGB;
+        format->extension = strdup("jpg");
+        format->renderer = MS_RENDER_WITH_AGG;
+    }
+#endif
 #ifdef USE_MING_FLASH
     if( strcasecmp(driver,"swf") == 0 )
     {
@@ -404,6 +306,12 @@ void msApplyDefaultOutputFormats( mapObj *map )
     if( msSelectOutputFormat( map, "wbmp" ) == NULL )
         msCreateDefaultOutputFormat( map, "GD/WBMP" );
 
+    if( msSelectOutputFormat( map, "aggpng24" ) == NULL )
+        msCreateDefaultOutputFormat( map, "AGG/PNG" );
+
+    if( msSelectOutputFormat( map, "aggjpeg" ) == NULL )
+        msCreateDefaultOutputFormat( map, "AGG/JPEG" );
+
     if( msSelectOutputFormat( map, "swf" ) == NULL )
         msCreateDefaultOutputFormat( map, "swf" );
 
@@ -918,7 +826,7 @@ void msGetOutputFormatMimeListGD( mapObj *map, char **mime_list, int max_mime )
 /*                  msGetOutputFormatMimeListRaster()                   */
 /************************************************************************/
 
-void msGetOutputFormatMimeListRaster( mapObj *map, char **mime_list, int max_mime )
+void msGetOutputFormatMimeListWMS( mapObj *map, char **mime_list, int max_mime )
 
 {
     int mime_count = 0, i;
@@ -939,7 +847,8 @@ void msGetOutputFormatMimeListRaster( mapObj *map, char **mime_list, int max_mim
 
         if( j == mime_count && map->outputformatlist[i]->driver &&
             (strncasecmp(map->outputformatlist[i]->driver, "GD/", 3)==0 ||
-             strncasecmp(map->outputformatlist[i]->driver, "GDAL/", 5)==0))
+             strncasecmp(map->outputformatlist[i]->driver, "GDAL/", 5)==0 ||
+             strcasecmp(map->outputformatlist[i]->driver, "svg")==0)) 
             mime_list[mime_count++] = map->outputformatlist[i]->mimetype;
     }
 
