@@ -35,7 +35,7 @@
 #endif
 #include "mapserver.h"
 
-MS_CVSID("$Id: mappdf.c 6693 2007-08-23 12:21:12Z dmorissette $")
+MS_CVSID("$Id: mappdf.c 7299 2008-01-23 15:14:10Z Assefa $")
 
 /* -------------------------------------------------------------------- */
 /*      prototypes.                                                     */
@@ -242,6 +242,7 @@ void drawPolylinePDF(PDF *pdf, shapeObj *p, colorObj  *c, double width)
     if (width)
     {
         PDF_setlinejoin(pdf,1);
+         PDF_setlinecap(pdf,1); 
         PDF_setlinewidth(pdf,(float)width);
     }
 
@@ -1164,7 +1165,7 @@ int msSaveImagePDF(imageObj *image, char *filename)
       /send the active buffer to disk
       /-----------------------------------------------*/
   
-      fwrite(pdfBuffer, sizeof(char), size, stream);
+      msIO_fwrite(pdfBuffer, sizeof(char), size, stream);
       if(filename != NULL && strlen(filename) > 0)
          fclose(stream);
       /* free(pdfBuffer); */
