@@ -26,7 +26,7 @@
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
-/* $Id: mappostgis.c 6477 2007-08-06 15:18:07Z unicoletti $ */
+/* $Id: mappostgis.c 7623 2008-05-27 14:58:47Z pramsey $ */
 #include <assert.h>
 #include "mapserver.h"
 #include "maptime.h"
@@ -48,7 +48,7 @@
 #include <string.h>
 #include <ctype.h> /* tolower() */
 
-MS_CVSID("$Id: mappostgis.c 6477 2007-08-06 15:18:07Z unicoletti $")
+MS_CVSID("$Id: mappostgis.c 7623 2008-05-27 14:58:47Z pramsey $")
 
 typedef struct ms_POSTGIS_layer_info_t
 {
@@ -165,6 +165,65 @@ static void msPOSTGISCloseConnection(void *conn_handle)
     PQfinish((PGconn*) conn_handle);
 }
 
+/* takes a connection and ensures that it is in a valid transactional state */
+/* performing PQreset() and/or ROLLBACK on it as necessary */
+int msPOSTGISSanitizeConnection(PGconn *conn)
+{
+    int conn_bad = 0;
+    PGTransactionStatusType trans_status;
+    
+    if (PQstatus(conn) == CONNECTION_BAD)
+    {
+        msDebug("Warning: resetting bad database connection due to PQstatus(conn) == CONNECTION_BAD in msPOSTGISSanitizeConnection()\n");
+        conn_bad = 1;
+    }
+    else if (PQtransactionStatus(conn) == PQTRANS_UNKNOWN)
+    {
+	msDebug("Warning: resetting bad database connection due to PQtransactionStatus(conn) == PQTRANS_UNKNOWN in msPOSTGISSanitizeConnection()\n");
+        conn_bad = 1;
+    }
+
+    /* if connection is in bad, PQreset() it */
+    if (conn_bad)
+    {
+        PQreset(conn);
+        if (PQstatus(conn) == CONNECTION_BAD)
+        {
+            msSetError(MS_QUERYERR, "Database connection status is CONNECTION_BAD even after attempt to PQreset() it: %s", "msPOSTGISSanitizeConnection()", PQerrorMessage(conn));
+            return MS_FAILURE;
+        }
+        else if (PQtransactionStatus(conn) == PQTRANS_UNKNOWN) 
+        {
+            msSetError(MS_QUERYERR, "Database connection transaction status is PQTRANS_UNKNOWN even after attempt to PQreset() it: %s", "msPOSTGISSanitizeConnection()", PQerrorMessage(conn)); 
+            return MS_FAILURE;
+        }
+    }
+
+    if (PQtransactionStatus(conn) == PQTRANS_ACTIVE) /* no connection should have an active async call */
+    {
+        msSetError(MS_QUERYERR, "Refusing to sanitize a database connection with a pending asynchronous query (transaction status of PQTRANS_ACTIVE).", "msPOSTGISSanitizeConnection()");
+	return MS_FAILURE;
+    }
+
+    trans_status = PQtransactionStatus(conn);
+    if (trans_status == PQTRANS_INTRANS || trans_status == PQTRANS_INERROR) /* idle, in a transaction block or a failed transaction block */
+    {
+        PGresult *rb_res = PQexec(conn, "ROLLBACK");
+        if (!rb_res || PQresultStatus(rb_res) != PGRES_COMMAND_OK) {
+            msSetError(MS_QUERYERR, "Error executing POSTGIS ROLLBACK statement: %s", "msPOSTGISSanitizeConnection()", PQerrorMessage(conn));
+
+            if(rb_res) {
+                PQclear(rb_res);
+            }
+
+            return MS_FAILURE;
+        }
+        PQclear(rb_res);
+    }
+ 
+    return MS_SUCCESS;
+}
+
 /*static int gBYTE_ORDER = 0;*/
 
 /* open up a connection to the postgresql database using the connection string in layer->connection */
@@ -173,8 +232,6 @@ int msPOSTGISLayerOpen(layerObj *layer)
 {
     msPOSTGISLayerInfo  *layerinfo;
     int                 order_test = 1;
-    char                *index, *maskeddata;
-    int                 i, count;
 
     if(layer->debug) {
         msDebug("msPOSTGISLayerOpen called datastatement: %s\n", layer->data);
@@ -212,32 +269,38 @@ int msPOSTGISLayerOpen(layerObj *layer)
             msDebug("MSPOSTGISLayerOpen -- shared connection not available.\n");
         }
 
+        if (!layer->connection) {
+          msSetError(MS_MISCERR, "Missing CONNECTION keyword.", "msPOSTGISLayerOpen()");
+          return MS_FAILURE;
+        }
+
         /* Decrypt any encrypted token in connection and attempt to connect */
         conn_decrypted = msDecryptStringTokens(layer->map, layer->connection);
         if (conn_decrypted == NULL) {
           return(MS_FAILURE);  /* An error should already have been produced */
         }
    
-        layerinfo->conn = PQconnectdb(layer->connection);
+        layerinfo->conn = PQconnectdb(conn_decrypted);
 
         msFree(conn_decrypted);
         conn_decrypted = NULL;
 
         if(!layerinfo->conn || PQstatus(layerinfo->conn) == CONNECTION_BAD) {
-            msDebug("FAILURE!!!");
-            maskeddata = (char *)malloc(strlen(layer->connection) + 1);
-            strcpy(maskeddata, layer->connection);
+            char *index, *maskeddata;
+            if (layer->debug)
+                msDebug("msPOSTGISLayerOpen() FAILURE!!!\n");
+
+            maskeddata = strdup(layer->connection);
             index = strstr(maskeddata, "password=");
             if(index != NULL) {
               index = (char *)(index + 9);
-              count = (int)(strstr(index, " ") - index);
-              for(i = 0; i < count; i++) {
-                  strncpy(index, "*", (int)1);
+              while(*index != '\0' && *index != ' ') {
+                  *index = '*';
                   index++;
               }
             }
             
-            msSetError(MS_QUERYERR, "couldnt make connection to DB with connect string '%s'.\n\nError reported was '%s'.\n\n\nThis error occured when trying to make a connection to the specified postgresql server.  \n\nMost commonly this is caused by \n(1) incorrect connection string \n(2) you didnt specify a 'user=...' in your connection string \n(3) the postmaster (postgresql server) isnt running \n(4) you are not allowing TCP/IP connection to the postmaster \n(5) your postmaster is not running on the correct port - if its not on 5432 you must specify a 'port=...' \n (6) the security on your system does not allow the webserver (usually user 'nobody') to make socket connections to the postmaster \n(7) you forgot to specify a 'host=...' if the postmaster is on a different machine\n(8) you made a typo \n  ", "msPOSTGISLayerOpen()", maskeddata,PQerrorMessage(layerinfo->conn));
+            msSetError(MS_QUERYERR, "Couldn't make connection to DB with connect string '%s'.\n\nError reported was '%s'.\n\n\nThis error occured when trying to make a connection to the specified postgresql server.  \n\nMost commonly this is caused by \n(1) incorrect connection string \n(2) you didnt specify a 'user=...' in your connection string \n(3) the postmaster (postgresql server) isnt running \n(4) you are not allowing TCP/IP connection to the postmaster \n(5) your postmaster is not running on the correct port - if its not on 5432 you must specify a 'port=...' \n (6) the security on your system does not allow the webserver (usually user 'nobody') to make socket connections to the postmaster \n(7) you forgot to specify a 'host=...' if the postmaster is on a different machine\n(8) you made a typo \n  ", "msPOSTGISLayerOpen()", maskeddata, PQerrorMessage(layerinfo->conn));
 
             free(maskeddata);
             free(layerinfo);
@@ -330,7 +393,7 @@ int msPOSTGISLayerInitItemInfo(layerObj *layer)
 static int prepare_database(const char *geom_table, const char *geom_column, layerObj *layer, PGresult **sql_results, rectObj rect, char **query_string, const char *urid_name, const char *user_srid)
 {
     msPOSTGISLayerInfo *layerinfo;
-    PGresult    *result = 0;
+    PGresult    *result = 0, *beg_res = 0;
     char        *temp = 0;
     char        *columns_wanted = 0;
     char        *data_source = 0;
@@ -341,11 +404,19 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
     size_t      length;
     char        *pos_from, *pos_ftab, *pos_space, *pos_paren;
 
-    char        *tmp2 = 0;
-    char        *error_message = 0;
-    char        *postgresql_error;
-
     layerinfo =  getPostGISLayerInfo(layer);
+    
+    beg_res = PQexec(layerinfo->conn, "BEGIN");
+    if (!beg_res || PQresultStatus(beg_res) != PGRES_COMMAND_OK)
+    {
+        msSetError(MS_QUERYERR, "Error executing PostgreSQL BEGIN statement: %s", "prepare_database()", PQerrorMessage(layerinfo->conn));
+        if(beg_res) {
+            PQclear(beg_res);
+        }
+        msPOSTGISSanitizeConnection(layerinfo->conn);
+        return MS_FAILURE;
+    }
+    PQclear(beg_res);
 
     /* Set the urid name */
     layerinfo->urid_name = (char *) malloc(strlen(urid_name) + 1);
@@ -480,28 +551,6 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
     free(f_table_name);
     free(columns_wanted);
 
-    /* start transaction required by cursor */
-
-    result = PQexec(layerinfo->conn, "BEGIN");
-    if(!result || PQresultStatus(result) != PGRES_COMMAND_OK) {
-        msSetError(MS_QUERYERR, "Error executing POSTGIS BEGIN statement.", "prepare_database()");
-
-        if(result) {
-            PQclear(result);
-        }
-		if(layerinfo->query_result) {
-			PQclear(layerinfo->query_result);
-		}
-        layerinfo->query_result = NULL;
-        PQreset(layerinfo->conn);
-
-        free(query_string_0_6);
-
-        return MS_FAILURE;     /* totally screwed */
-    }
-
-    PQclear(result);
-
     /* set enable_seqscan=off not required (already done) */
 
     if(layer->debug) {
@@ -509,65 +558,25 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
     }
     result = PQexec(layerinfo->conn, query_string_0_6);
 
-    if(result && PQresultStatus(result) == PGRES_COMMAND_OK)
-    {
-        *sql_results = result;
-
-        *query_string = (char *) malloc(strlen(query_string_0_6) + 1);
-        strcpy(*query_string, query_string_0_6);
-
-        free(query_string_0_6);
-
-        return MS_SUCCESS;
-    }
-
-    /* Save PostgreSQL Error Message */
-    postgresql_error = PQerrorMessage(layerinfo->conn);
-    error_message = (char *) malloc(strlen(postgresql_error) + 1);
-    strcpy(error_message, postgresql_error);
-
-    /* okay, that command didnt work.  Its probably a 0.5 database */
-    /* We have to everything again, after performing a rollback. */
-
-    if(result) {
-        PQclear(result);
-    }
-    result = PQexec(layerinfo->conn, "rollback" );
-    if(result) {
-        PQclear(result);
-    }
-    result = PQexec(layerinfo->conn, "begin" );
-
     if(!result || PQresultStatus(result) != PGRES_COMMAND_OK)
     {
-        msSetError(MS_QUERYERR, "Couldnt recover from a bad query: \n'%s'\n", "prepare_database()", query_string_0_6);
-
-        if(result) {
-            PQclear(result);
-        }
-        layerinfo->query_result = NULL;
-        PQreset(layerinfo->conn);
-
-        free(error_message);
+        msSetError(MS_QUERYERR, "Error declaring cursor: %s\nWith query string: %s\n", "prepare_database()", PQerrorMessage(layerinfo->conn), query_string_0_6);
         free(query_string_0_6);
-
-        return MS_FAILURE;     /* totally screwed */
+        if (result)
+        {
+	    PQclear(result);
+        }
+        return MS_FAILURE;
     }
+        
+    *sql_results = result;
 
-    PQclear(result);
-    layerinfo->query_result = NULL;
-    PQreset(layerinfo->conn);
+    *query_string = (char *) malloc(strlen(query_string_0_6) + 1);
+    strcpy(*query_string, query_string_0_6);
 
-    /* TODO Rename tmp2 to something meaningful */
-    tmp2 = (char *) malloc(149 + strlen(query_string_0_6) + strlen(error_message) + 1);
-    sprintf(tmp2, "Error executing POSTGIS DECLARE (the actual query) statement: '%s' \n\nPostgresql reports the error as '%s'\n\nMore Help:\n\n", query_string_0_6, error_message);
-    msSetError(MS_QUERYERR, DATA_ERROR_MESSAGE, "prepare_database()", tmp2, "check your .map file");
-
-    free(tmp2);
-    free(error_message);
     free(query_string_0_6);
 
-    return MS_FAILURE;     /* totally screwed */
+    return MS_SUCCESS;
 }
 
 
@@ -637,7 +646,7 @@ int msPOSTGISLayerWhichShapes(layerObj *layer, rectObj rect)
           PQclear(layerinfo->query_result);
         }
         layerinfo->query_result = NULL;
-        PQreset(layerinfo->conn);
+        msPOSTGISSanitizeConnection(layerinfo->conn);
 
         return MS_FAILURE;
     }
@@ -651,7 +660,8 @@ int msPOSTGISLayerWhichShapes(layerObj *layer, rectObj rect)
 int msPOSTGISLayerClose(layerObj *layer)
 {
     msPOSTGISLayerInfo  *layerinfo;
-
+    PGresult *rollb_res;
+    
     layerinfo = getPostGISLayerInfo(layer);
 
     if(layer->debug) {
@@ -683,9 +693,28 @@ int msPOSTGISLayerClose(layerObj *layer)
             query_result = PQexec(layerinfo->conn, cmd_buffer );
             if(query_result) {
                 PQclear(query_result);
-            }
+            } else {
+              if (msPOSTGISSanitizeConnection(layerinfo->conn) != MS_SUCCESS)
+	      {
+                return MS_FAILURE;
+	      }
+	    }
 
             layerinfo->cursor_name[0] = '\0';
+
+            rollb_res = PQexec(layerinfo->conn, "ROLLBACK");
+            if (!rollb_res || PQresultStatus(rollb_res) != PGRES_COMMAND_OK) {
+                msSetError(MS_QUERYERR, "Error executing PostgreSQL ROLLBACK statement: %s", "msPOSTGISLayerClose()", PQerrorMessage(layerinfo->conn));
+        
+                if(rollb_res) {
+                    PQclear(rollb_res);
+                }
+                msPOSTGISSanitizeConnection(layerinfo->conn);
+        
+                return MS_FAILURE;
+            }
+            PQclear(rollb_res);
+
         }
 
         msConnPoolRelease(layer, layerinfo->conn);
@@ -738,7 +767,8 @@ static int  force_to_points(char *wkb, shapeObj *shape)
     shape->type = MS_SHAPE_NULL;  /* nothing in it */
 
     memcpy(&ngeoms, &wkb[5], 4);
-    offset = 9;  /* were the first geometry is */
+    offset = 9;  /* where the first geometry is */
+ 
     for(t=0; t < ngeoms; t++) {
         memcpy(&type, &wkb[offset + 1], 4);  /* type of this geometry */
 
@@ -792,7 +822,7 @@ static int  force_to_points(char *wkb, shapeObj *shape)
             }
         }
     }
-
+    
     return MS_SUCCESS;
 }
 
@@ -1129,10 +1159,10 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
     size_t  length;
     char    *temp;
 
-    PGresult            *query_result;
+    PGresult            *query_result, *beg_res, *rollb_res, *close_res;
     msPOSTGISLayerInfo  *layerinfo;
     char                *wkb;
-    int                 result, t, size;
+    int                 result, t, size, num_tuples;
     char                *temp1, *temp2;
 
     if(layer->debug) {
@@ -1147,6 +1177,19 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
 
         return MS_FAILURE;
     }
+    
+    beg_res = PQexec(layerinfo->conn, "BEGIN");
+    if (!beg_res || PQresultStatus(beg_res) != PGRES_COMMAND_OK)
+    {
+        msSetError(MS_QUERYERR, "Error executing PostgreSQL BEGIN statement: %s", "msPOSTGISLayerGetShape()", PQerrorMessage(layerinfo->conn));
+        if(beg_res)
+	{
+            PQclear(beg_res);
+        }
+        msPOSTGISSanitizeConnection(layerinfo->conn);
+        return MS_FAILURE;
+    }
+    PQclear(beg_res);
 
     if (msPOSTGISLayerParseData(layer, &geom_column_name, &table_name, &urid_name, &user_srid, layer->debug) != MS_SUCCESS) {
 		return MS_FAILURE;
@@ -1196,21 +1239,6 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
     free(geom_column_name);
     free(table_name);
 
-    query_result = PQexec(layerinfo->conn, "BEGIN");
-    if(!query_result || PQresultStatus(query_result) != PGRES_COMMAND_OK) {
-        msSetError(MS_QUERYERR, "Error executing POSTGIS  BEGIN   statement.", "msPOSTGISLayerGetShape()");
-
-        if(query_result) {
-            PQclear(query_result);
-        }
-        PQreset(layerinfo->conn);
-
-        free(query_str);
-
-        return MS_FAILURE;
-    }
-    PQclear(query_result);
-
     query_result = PQexec(layerinfo->conn, query_str);
 
     if(!query_result || PQresultStatus(query_result) != PGRES_COMMAND_OK) {
@@ -1219,7 +1247,7 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
         if(query_result) {
           PQclear(query_result);
         }
-        PQreset(layerinfo->conn);
+        msPOSTGISSanitizeConnection(layerinfo->conn);
 
         free(query_str);
 
@@ -1235,7 +1263,7 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
         if(query_result) {
           PQclear(query_result);
         }
-        PQreset(layerinfo->conn);
+        msPOSTGISSanitizeConnection(layerinfo->conn);
 
         free(query_str);
 
@@ -1248,7 +1276,8 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
     /* query has been done, so we can retreive the results */
     shape->type = MS_SHAPE_NULL;
 
-    if(0 < PQntuples(query_result)) {
+    num_tuples = PQntuples(query_result);
+    if(0 < num_tuples) {
         /* only need to get one shape */
         /* retreive an item */
         wkb = (char *) PQgetvalue(query_result, 0, layer->numitems);  
@@ -1302,62 +1331,43 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
             shape->numvalues = layer->numitems;
 
             find_bounds(shape);
-
-	        PQclear(query_result);
-
-            query_result = PQexec(layerinfo->conn, "CLOSE mycursor2");
-            if(query_result) {
-                PQclear(query_result);
-            }
-
-            query_result = PQexec(layerinfo->conn, "ROLLBACK");
-            if(!query_result || PQresultStatus(query_result) != PGRES_COMMAND_OK) {
-                msSetError(MS_QUERYERR, "Error executing POSTGIS  BEGIN   statement.", "msPOSTGISLayerGetShape()");
-
-                if(query_result) {
-                    PQclear(query_result);
-                }
-                PQreset(layerinfo->conn);
-
-                msFreeShape(shape);
-
-                return MS_FAILURE;
-            }
-
-            PQclear(query_result);
-
-            return MS_SUCCESS;
         }
-
-        PQclear(query_result);
-    } else {
-        PQclear(query_result);
-
-        query_result = PQexec(layerinfo->conn, "CLOSE mycursor2");
-        if(query_result) {
-            PQclear(query_result);
-        }
-
-        query_result = PQexec(layerinfo->conn, "ROLLBACK");
-        if(!query_result || PQresultStatus(query_result) != PGRES_COMMAND_OK) {
-            msSetError(MS_QUERYERR, "Error executing POSTGIS  BEGIN   statement.", "msPOSTGISLayerGetShape()");
-
-            if(query_result) {
-                PQclear(query_result);
-            }
-            PQreset(layerinfo->conn);
-
-            return MS_FAILURE;
-        }
-
-        PQclear(query_result);
-
-        return MS_DONE;
     }
 
-    msFreeShape(shape);
+    PQclear(query_result);
+
+    close_res = PQexec(layerinfo->conn, "CLOSE mycursor2");
+    if (!close_res || PQresultStatus(close_res) != PGRES_COMMAND_OK)
+    {
+        msSetError(MS_QUERYERR, "Error executing PostgreSQL CLOSE statement.", "msPOSTGISLayerGetShape()");
+        if (num_tuples > 0 && shape->type != MS_SHAPE_NULL)
+        {
+            msFreeShape(shape);
+        }
+        if (close_res)
+        {
+            PQclear(close_res);
+        }
+        msPOSTGISSanitizeConnection(layerinfo->conn);
+        return MS_FAILURE;
+    }
+    PQclear(close_res);
+
+    rollb_res = PQexec(layerinfo->conn, "ROLLBACK");
+    if (!rollb_res || PQresultStatus(rollb_res) != PGRES_COMMAND_OK)
+    {
+        msSetError(MS_QUERYERR, "Error executing PostgreSQL ROLLBACK statement: %s", "msPOSTGISLayerClose()", PQerrorMessage(layerinfo->conn));
+        if(rollb_res)
+	{
+            PQclear(rollb_res);
+        }
+        msPOSTGISSanitizeConnection(layerinfo->conn);
+        return MS_FAILURE;
+    }
+    PQclear(rollb_res);
+
+    return (shape->type == MS_SHAPE_NULL) ? MS_FAILURE : ( (num_tuples > 0) ? MS_SUCCESS : MS_DONE );
 
-    return MS_FAILURE;
 }
 
 
@@ -1428,7 +1438,9 @@ int msPOSTGISLayerGetItems(layerObj *layer)
 
         if(query_result) {
             PQclear(query_result);
-        }
+        } else {
+            msPOSTGISSanitizeConnection(layerinfo->conn);
+	}
 
         free(sql);
         free(geom_column_name);
@@ -1606,6 +1618,7 @@ int msPOSTGISLayerRetrievePGVersion(layerObj *layer, int debug, int *major, int
           msDebug("msPOSTGISLayerRetrievePGVersion: No results returned.\n");
         }
         free(tmp2);
+        msPOSTGISSanitizeConnection(layerinfo->conn);
         return(MS_FAILURE);
 
     }
@@ -1753,6 +1766,7 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
       msSetError(MS_QUERYERR, tmp2, "msPOSTGISLayerRetrievePK()");
       free(tmp2);
       free(sql);
+      msPOSTGISSanitizeConnection(layerinfo->conn);
       return(MS_FAILURE);
 
     }
