@@ -24,18 +24,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.1  2006/05/12 18:32:41  pramsey
- * Added support for postgresql joins (Bug 1570).
- *
- */
+ ****************************************************************************/
 
 
-/* $Id: mappostgresql.c 5433 2006-05-12 18:32:41Z pramsey $ */
+/* $Id: mappostgresql.c 6428 2007-07-31 19:58:45Z dmorissette $ */
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
 #include "maptime.h"
 
 #ifndef FLT_MAX
@@ -55,7 +49,7 @@
 #include <string.h>
 #include <ctype.h> /* tolower() */
 
-MS_CVSID("$Id: mappostgresql.c 5433 2006-05-12 18:32:41Z pramsey $")
+MS_CVSID("$Id: mappostgresql.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 
 typedef struct {
