@@ -24,125 +24,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.76  2006/09/01 18:03:19  umberto
- * Removed USE_GEOS defines, bug #1890
- *
- * Revision 1.75  2006/08/17 04:32:16  sdlime
- * Disable path following labels unless GD 2.0.29 or greater is available.
- *
- * Revision 1.74  2006/06/27 13:58:19  frank
- * Place shapeObj->geometry assignment into #ifdef USE_GEOS.
- *
- * Revision 1.73  2006/06/27 06:32:09  sdlime
- * Fixed msCopyShape to initialize the geometry property of the destination shape to NULL.
- *
- * Revision 1.72  2006/05/18 22:20:37  sdlime
- * Fixed offseting code percentage of 1.0 maps to width-1 or height-1.
- *
- * Revision 1.71  2006/04/28 03:13:02  sdlime
- * Fixed a few issues with relative coordinates. Added support for all nine relative positions. (bug 1547)
- *
- * Revision 1.70  2006/04/27 04:05:17  sdlime
- * Initial support for relative coordinates. (bug 1547)
- *
- * Revision 1.69  2006/04/26 13:42:53  frank
- * temporarily block out MS_PERCENTAGES use till it is defined.
- *
- * Revision 1.68  2006/04/26 03:25:47  sdlime
- * Applied most recent patch for curved labels. (bug 1620)
- *
- * Revision 1.67  2006/03/22 23:31:20  sdlime
- * Applied latest patch for curved labels. (bug 1620)
- *
- * Revision 1.66  2006/03/02 06:43:51  sdlime
- * Applied latest patch for curved labels. (bug 1620)
- *
- * Revision 1.65  2006/02/24 05:53:49  sdlime
- * Applied another round of patches for bug 1620.
- *
- * Revision 1.64  2006/02/18 21:14:09  hobu
- * INFINITY is already defined on in the math headers on osx.
- * Don't redefine it if it is already there.
- *
- * Revision 1.63  2006/02/18 20:59:13  sdlime
- * Initial code for curved labels. (bug 1620)
- *
- * Revision 1.62  2006/02/17 03:05:23  sdlime
- * Slightly more efficient version (no modulus operator) of the routine to test if a ring is an outer ring. (bug 1648)
- *
- * Revision 1.61  2006/02/16 07:51:31  sdlime
- * Fixed a flaw in routine that computes outer ring list. In certain cases it could miss an outer ring with holes in certain places. (bug 1648)
- *
- * Revision 1.60  2005/11/01 05:35:50  frank
- * added preliminary implementation of OGR based WKT translation, still untested
- *
- * Revision 1.59  2005/10/31 04:58:14  sdlime
- * Added a check in msFreeShape() to make sure the incoming shapeObj is not NULL.
- *
- * Revision 1.58  2005/10/30 05:05:07  sdlime
- * Initial support for WKT via GEOS. The reader is only integrated via the map file reader, with MapScript, CGI and URL support following ASAP. (bug 1466)
- *
- * Revision 1.57  2005/10/20 19:37:03  frank
- * added msAddPointToLine
- *
- * Revision 1.56  2005/10/18 03:20:44  frank
- * fixed use of memmove in msShapeDeleteLine()
- *
- * Revision 1.55  2005/10/18 03:10:45  frank
- * added msShapeDeleteLine
- *
- * Revision 1.54  2005/07/27 18:21:58  frank
- * bug 1432: optimized msAddLine() to use realloc()
- *
- * Revision 1.53  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.52  2005/05/19 05:57:08  sdlime
- * Added GEOS geometry clean up code to msFreeShape...
- *
- * Revision 1.51  2005/04/21 15:09:28  julien
- * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
- *
- * Revision 1.50  2005/04/14 15:17:14  julien
- * Bug 1244: Remove Z and M from point by default to gain performance.
- *
- * Revision 1.49  2005/03/25 05:42:58  frank
- * added msAddLineDirectly(), msAddLine() uses memcpy()
- *
- * Revision 1.48  2005/03/24 22:27:20  frank
- * optimized msTransformShapeToPixel - avoid division
- *
- * Revision 1.47  2005/03/24 17:50:04  frank
- * optimized msClipPoly{gon/line}Rect for all-inside case
- *
- * Revision 1.46  2005/02/22 07:40:27  sdlime
- * A bunch of updates to GEOS integration. Can move many primatives between MapServer and GEOS, still need to do collections (e.g. multi-point/line/polygon). Added buffer method to mapscript (mapscript/shape.i).
- *
- * Revision 1.45  2005/02/18 03:06:46  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.44  2005/02/13 22:16:06  dan
- * Use double as second arg to pow() (bug 1235)
- *
- * Revision 1.43  2004/12/14 21:30:43  sdlime
- * Moved functions to build lists of inner and outer rings to mapprimitive.c from mapgml.c. They are needed to covert between MapServer polygons and GEOS gemometries (bug 771).
- *
- * Revision 1.42  2004/11/12 20:23:16  frank
- * include z and m in formatted pointObj, fmt is const!
- *
- * Revision 1.41  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "mapprimitive.h"
 #include <assert.h>
+#include <locale.h>
 
-MS_CVSID("$Id: mapprimitive.c 5724 2006-09-01 18:03:19Z umberto $")
+MS_CVSID("$Id: mapprimitive.c 6777 2007-09-03 03:54:47Z sdlime $")
 
 typedef enum {CLIP_LEFT, CLIP_MIDDLE, CLIP_RIGHT} CLIP_STATE;
 
@@ -792,6 +681,9 @@ void msClipPolygonRect(shapeObj *shape, rectObj rect)
 void msOffsetPointRelativeTo(pointObj *point, layerObj *layer)
 {
   double x=0, y=0;
+  if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
+	return;
+  
 
   if(layer->transform == MS_TRUE) return; /* nothing to do */
 
@@ -852,6 +744,9 @@ void msOffsetShapeRelativeTo(shapeObj *shape, layerObj *layer)
   double x=0, y=0;
 
   if(layer->transform == MS_TRUE) return; /* nothing to do */
+  if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
+	return;
+  
 
   if(layer->units == MS_PERCENTAGES) {
     for (i=0; i<shape->numlines; i++) {
@@ -1115,23 +1010,6 @@ static int get_centroid(shapeObj *p, pointObj *lp, double *miny, double *maxy)
 }
 #endif
 
-static void get_bbox(shapeObj *poly, double *minx, double *miny, double *maxx, double *maxy) {
-  int i, j;
-
-  *minx = *maxx = poly->line[0].point[0].x;
-  *miny = *maxy = poly->line[0].point[0].y;
-  for(i=0; i<poly->numlines; i++) {
-    for(j=1; j<poly->line[i].numpoints; j++) {
-      *minx = MS_MIN(*minx, poly->line[i].point[j].x);
-      *maxx = MS_MAX(*maxx, poly->line[i].point[j].x);
-      *miny = MS_MIN(*miny, poly->line[i].point[j].y);
-      *maxy = MS_MAX(*maxy, poly->line[i].point[j].y);
-    }
-  }
-
-  return;
-}
-
 #define NUM_SCANLINES 5
 
 /*
@@ -1148,7 +1026,11 @@ int msPolygonLabelPoint(shapeObj *p, pointObj *lp, int min_dimension)
   double len, max_len=0;
   double skip, minx, maxx, maxy, miny;
 
-  get_bbox(p, &minx, &miny, &maxx, &maxy);
+  msComputeBounds(p);
+  minx = p->bounds.minx;
+  miny = p->bounds.miny;
+  maxx = p->bounds.maxx;
+  maxy = p->bounds.maxy;
 
   if(min_dimension != -1)
     if(MS_MIN(maxx-minx,maxy-miny) < min_dimension) return(MS_FAILURE);
@@ -1336,6 +1218,8 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
   double line_length, max_line_length, segment_length, total_length, distance_along_segment;
   double fwd_line_length, rev_line_length, text_length, text_start_length;
   int segment_index, line_index;
+
+  int numchars;
   
   int i,j,k, inc, final_j;
   double direction;
@@ -1353,45 +1237,53 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
   char *font = NULL;
 
   /* Line smoothing kernel */
-  double kernel[] = {0.1,0.2,2,0.2,0.1}; /*{1.5, 2, 15, 2, 1.5};*/
+  double kernel[] = {0.1, 0.2, 2, 0.2, 0.1}; /* {1.5, 2, 15, 2, 1.5}; */
   double kernel_normal = 2.6; /* Must be sum of kernel elements */
+  int kernel_size = 5;
 
   double letterspacing = 1.25;
 
   offsets = NULL;
-  
-  /* Assume success */
+
+  /* assume success */  
   *status = MS_SUCCESS;
 
 #ifndef GD_HAS_FTEX_XSHOW
   goto FAILURE; /* we don't have a current enough version of GD, fall back to ANGLE AUTO */
 #else
-  /* Skip the label and use the normal algorithm if it has fewer than 2 characters */
-  if ( strlen(string) < 2 ) {
+
+  /* If the labelObj has an encodiing set then we expect UTF-8 as input, otherwise
+   * we treat the string as a regular array of chars 
+   */
+  if (label->encoding)
+      numchars = msGetNumUTF8Chars(string);
+  else
+      numchars = strlen(string);
+
+  /* skip the label and use the normal algorithm if it has fewer than 2 characters */
+  if(numchars < 2)
     goto FAILURE;
-  }
-  
+
   segment_index = line_index = 0;
   total_length = max_line_length = 0.0;
   
-  /* Determine longest line */
-  segment_lengths = (double**)malloc(sizeof(double*) * p->numlines);
-  for ( i = 0; i < p->numlines; i++ ) {
+  /* determine longest line */
+  segment_lengths = (double **) malloc(sizeof(double *) * p->numlines);
+  for(i=0; i<p->numlines; i++) {
     
-    segment_lengths[i] = (double*)malloc(sizeof(double) * p->line[i].numpoints);    
+    segment_lengths[i] = (double*) malloc(sizeof(double) * p->line[i].numpoints);    
     line_length = 0;
     
-    for ( j = 1; j < p->line[i].numpoints; j++ ) {
+    for (j=1; j<p->line[i].numpoints; j++) {
       segment_length = sqrt( pow((p->line[i].point[j].x - p->line[i].point[j-1].x), 2.0) +
                              pow((p->line[i].point[j].y - p->line[i].point[j-1].y), 2.0) );
       line_length += segment_length;
-      segment_lengths[i][j-1] = segment_length;
-      
+      segment_lengths[i][j-1] = segment_length;      
     }
 
     total_length += line_length;
     
-    if ( line_length > max_line_length ) {
+    if(line_length > max_line_length) {
       line_index = i;
       max_line_length = line_length;
     }
@@ -1399,25 +1291,21 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
 
   i = line_index;
   
-  if ( ((min_length != -1) && (total_length < min_length)) ) {
-    /* Too short */ 
+  if (((min_length != -1) && (total_length < min_length))) { /* too short */
     *status = MS_FAILURE;
     goto FAILURE;
   }
 
-  if ( p->line[i].numpoints < 2 ) {
-    /* Degenerate */
+  if(p->line[i].numpoints < 2) { /* degenerate */
     *status = MS_FAILURE;
     goto FAILURE;
   }
 
-  if ( p->line[i].numpoints == 2 ) {
-    /* We can just use the regular algorithm to save some cycles */
+  if(p->line[i].numpoints == 2) /* use the regular angled text algorithm */
     goto FAILURE;
-  }
   
-  /* Determine the total length of the text */
-  if ( msGetLabelSizeEx(string, label, &bbox, fontset, scalefactor, MS_FALSE, &offsets) == MS_FAILURE ) {
+  /* determine the total length of the text */
+  if (msGetLabelSizeEx(string, label, &bbox, fontset, scalefactor, MS_FALSE, &offsets) == MS_FAILURE) {
     *status = MS_FAILURE;
     goto FAILURE;
   }
@@ -1438,40 +1326,40 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
   
   text_length = letterspacing * (bbox.maxx - bbox.minx);
 
-  /* If the text length is way longer than the line, skip adding the
-     label if it isn't forced (long extrapolated labels tend to be
-     ugly) */
+  /* 
+  ** if the text length is way longer than the line, skip adding the
+  ** label if it isn't forced (long extrapolated labels tend to be ugly) 
+  */
   if ( text_length > 1.5 * max_line_length && label->force == MS_FALSE ) {
     *status = MS_FAILURE;
     goto FAILURE;
   }
-  
-  /* Allocate the labelpath */
-  labelpath = (labelPathObj*)malloc(sizeof(labelPathObj));
-  labelpath->path.numpoints = strlen(string);
-  labelpath->path.point = (pointObj*)calloc(labelpath->path.numpoints,sizeof(pointObj));
-  labelpath->angles = (double*)malloc(sizeof(double) * (labelpath->path.numpoints));
+
+  /* allocate the labelpath */
+  labelpath = (labelPathObj *) malloc(sizeof(labelPathObj));
+  labelpath->path.numpoints = numchars;
+  labelpath->path.point = (pointObj *) calloc(labelpath->path.numpoints, sizeof(pointObj));
+  labelpath->angles = (double *) malloc(sizeof(double) * (labelpath->path.numpoints));
   msInitShape(&(labelpath->bounds));
 
-  /* The bounds will have two points for each character plus an endpoint:
-     the UL corners of each bbox will be tied together and the LL corners
-     will be tied together. */
-  bounds.numpoints = 2*strlen(string) + 1;
-  bounds.point = (pointObj*)malloc(sizeof(pointObj) * bounds.numpoints);
-                                   
-  /* The points start at (max_line_length - text_length) / 2 in order to be centred */
+  /* 
+  ** The bounds will have two points for each character plus an endpoint:
+  ** the UL corners of each bbox will be tied together and the LL corners
+  ** will be tied together.
+  */
+  bounds.numpoints = 2*numchars + 1;
+  bounds.point = (pointObj *) malloc(sizeof(pointObj) * bounds.numpoints);
+
+  /* the points start at (max_line_length - text_length) / 2 in order to be centred */
   text_start_length = (max_line_length - text_length) / 2.0;
-  
-  /* The text is longer than the line: extrapolate the first and last segments */
-  if ( text_start_length < 0.0 ) {
-    
+
+  /* the text is longer than the line: extrapolate the first and last segments */
+  if(text_start_length < 0.0) {    
     j = 0;
     final_j = p->line[i].numpoints - 1;
     fwd_line_length = rev_line_length = 0;
-    
   } else {
-
-    /* Proceed until we've traversed text_start_length in distance */
+    /* proceed until we've traversed text_start_length in distance */
     fwd_line_length = 0;
     j = 0;
     while ( fwd_line_length < text_start_length )
@@ -1479,50 +1367,46 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
 
     j--;
 
-    /* Determine the last segment */ 
+    /* determine the last segment */ 
     rev_line_length = 0;
     final_j = p->line[i].numpoints - 1;
-    while ( rev_line_length < text_start_length ) {
+    while(rev_line_length < text_start_length) {
       rev_line_length += segment_lengths[i][final_j - 1];
       final_j--;
     }
     final_j++;
-
   }
 
-  if ( final_j == 0 )
+  if(final_j == 0)
     final_j = 1;
   
-  /* Determine if the line is mostly left to right or right to left */
+  /* determine if the line is mostly left to right or right to left, 
+     see bug 1620 discussion by Steve Woodbridge */
   direction = 0;
   k = j;
-  while ( k < final_j ) {
+  while (k < final_j) {
     direction += p->line[i].point[k+1].x - p->line[i].point[k].x;
     k++;
   }
-  
-  if ( direction > 0 ) {
 
-    /* j is already correct */
-    inc = 1;
+  if(direction > 0) {    
+    inc = 1; /* j is already correct */
 
-    /* Length of the segment containing the starting point */
+    /* length of the segment containing the starting point */
     segment_length = segment_lengths[i][j];
-    /* Determine how far along the segment we need to go */
-    t = 1 - (fwd_line_length - text_start_length) / segment_length;
-    
-  } else {
 
+    /* determine how far along the segment we need to go */
+    t = 1 - (fwd_line_length - text_start_length) / segment_length;
+  } else {
     j = final_j;
     inc = -1;
 
-    /* Length of the segment containing the starting point */
+    /* length of the segment containing the starting point */
     segment_length = segment_lengths[i][j-1];
-    t = 1 - (rev_line_length - text_start_length) / segment_length;
-    
+    t = 1 - (rev_line_length - text_start_length) / segment_length;    
   }
     
-  distance_along_segment = t * segment_length; /* Starting point */
+  distance_along_segment = t * segment_length; /* starting point */
   
   theta = 0;
   k = 0;
@@ -1534,77 +1418,70 @@ labelPathObj* msPolylineLabelPath(shapeObj *p, int min_length, fontSetObj *fonts
     x = t * (p->line[i].point[j+inc].x - p->line[i].point[j].x) + p->line[i].point[j].x;
     y = t * (p->line[i].point[j+inc].y - p->line[i].point[j].y) + p->line[i].point[j].y;
 
-    /* Average this label point with its neighbors according to the
-       smoothing kernel */
-    
-    if ( k == 0 ) {
-      labelpath->path.point[k].x += (kernel[0] + kernel[1]) * x;      
-      labelpath->path.point[k].y += (kernel[0] + kernel[1]) * y;      
-      
-    } else if ( k == 1 ) {
+    /* 
+    ** This used to be a series of if-then-else's, but that fails for short (length < 4)
+    ** labels. There may be small speed-ups possible here. (bug 1921)
+    **
+    ** average this label point with its neighbors according to the smoothing kernel
+    */
+    if ( k == 0) {
+      labelpath->path.point[k].x += (kernel[0] + kernel[1]) * x;
+      labelpath->path.point[k].y += (kernel[0] + kernel[1]) * y;
+    } 
+    if ( k == 1) {
       labelpath->path.point[k].x += kernel[0] * x;
       labelpath->path.point[k].y += kernel[0] * y;
-      
-    } else if ( k == labelpath->path.numpoints - 2 ) {
+    } 
+    if ( k == labelpath->path.numpoints - 2) {
       labelpath->path.point[k].x += kernel[4] * x;
-      labelpath->path.point[k].y += kernel[4] * y;
-      
-    } else if ( k == labelpath->path.numpoints - 1 ) {
+      labelpath->path.point[k].y += kernel[4] * y;      
+    } 
+    if ( k == labelpath->path.numpoints - 1) {
       labelpath->path.point[k].x += (kernel[3] + kernel[4]) * x;
       labelpath->path.point[k].y += (kernel[3] + kernel[4]) * y;
-      }
+    }
       
-    for (m = 0; m < 5; m++) {
-      if ( m + k - 2 < 0 || m + k - 2 > labelpath->path.numpoints - 1 )
+    for(m = 0; m < kernel_size; m++) {
+      if(m + k - 2 < 0 || m + k - 2 > labelpath->path.numpoints - 1)
         continue;
-      
       labelpath->path.point[k+m-2].x += kernel[m]*x;
       labelpath->path.point[k+m-2].y += kernel[m]*y;
     }
     
     w = letterspacing*offsets[k];
     
-    /* Add the character's width to the distance along the line */
+    /* add the character's width to the distance along the line */
     distance_along_segment += w;
 
-    /* If we still have segments left and we've past the current
-       segment, move to the next one */
-    
-    if ( inc == 1 && j < p->line[i].numpoints - 2 ) {
-      
+    /* if we still have segments left and we've past the current segment, move to the next one */    
+    if(inc == 1 && j < p->line[i].numpoints - 2) {      
+
       while ( j < p->line[i].numpoints - 2 && distance_along_segment > segment_lengths[i][j] ) {
-        distance_along_segment -= segment_lengths[i][j];
-        
-        /* Move to next segment */
-        j += inc;
+        distance_along_segment -= segment_lengths[i][j];                
+        j += inc; /* move to next segment */
       }
 
       segment_length = segment_lengths[i][j];
-      
-      
-    } else if ( inc == -1 && j > 1 ) {
+            
+    } else if( inc == -1 && j > 1 ) {
 
       while ( j > 1 && distance_along_segment > segment_lengths[i][j-1] ) {
         distance_along_segment -= segment_lengths[i][j-1];
-        
-        /* Move to next segment */
-        j += inc;
+        j += inc; /* move to next segment */
       }
       
       segment_length = segment_lengths[i][j-1];
-      
     }
     
     /* Recalculate interpolation parameter */
     t = distance_along_segment / segment_length;
     
     k++;
-    
   }
   
-  /* Pre-calc the character's centre y value.  Used for rotation adjustment.  */
+  /* pre-calc the character's centre y value.  Used for rotation adjustment. */
   cy = -size / 2.0;
-    
+
   labelpath->path.point[0].x /= kernel_normal;
   labelpath->path.point[0].y /= kernel_normal;
   
