@@ -28,7 +28,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapquery.c 6600 2007-08-16 14:48:27Z dmorissette $")
+MS_CVSID("$Id: mapquery.c 7559 2008-05-05 14:46:11Z pramsey $")
 
 /*
 ** msIsLayerQueryable()  returns MS_TRUE/MS_FALSE
@@ -266,6 +266,10 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
 
   lp = (GET_LAYER(map, qlayer));
 
+  /* conditions may have changed since this layer last drawn, so set 
+     layer->project true to recheck projection needs (Bug #673) */ 
+  lp->project = MS_TRUE; 
+
   /* free any previous search results, do now in case one of the following tests fails */
   if(lp->resultcache) {
     if(lp->resultcache->results) free(lp->resultcache->results);
@@ -421,6 +425,10 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
   for(l=start; l>=stop; l--) {
     lp = (GET_LAYER(map, l));
 
+    /* conditions may have changed since this layer last drawn, so set 
+       layer->project true to recheck projection needs (Bug #673) */ 
+    lp->project = MS_TRUE; 
+
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
       if(lp->resultcache->results) free(lp->resultcache->results);
@@ -478,13 +486,13 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
 
       shape.classindex = msShapeGetClass(lp, &shape, map->scaledenom);
       if(!(lp->template) && ((shape.classindex == -1) || (lp->class[shape.classindex]->status == MS_OFF))) { /* not a valid shape */
-	msFreeShape(&shape);
-	continue;
+	      msFreeShape(&shape);
+	      continue;
       }
 
       if(!(lp->template) && !(lp->class[shape.classindex]->template)) { /* no valid template */
-	msFreeShape(&shape);
-	continue;
+        msFreeShape(&shape);
+	      continue;
       }
 
 #ifdef USE_PROJ
@@ -495,11 +503,11 @@ int msQueryByRect(mapObj *map, int qlayer, rectObj rect)
 #endif
 
       if(msRectContained(&shape.bounds, &rect) == MS_TRUE) { /* if the whole shape is in, don't intersect */	
-	status = MS_TRUE;
+	      status = MS_TRUE;
       } else {
 	switch(shape.type) { /* make sure shape actually intersects the rect (ADD FUNCTIONS SPECIFIC TO RECTOBJ) */
 	case MS_SHAPE_POINT:
-	  status = msIntersectMultipointPolygon(&shape.line[0], &searchshape);
+	  status = msIntersectMultipointPolygon(&shape, &searchshape);
 	  break;
 	case MS_SHAPE_LINE:
 	  status = msIntersectPolylinePolygon(&shape, &searchshape);
@@ -579,6 +587,10 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
     return(MS_FAILURE);
   }
 
+  /* conditions may have changed since this layer last drawn, so set 
+     layer->project true to recheck projection needs (Bug #673) */ 
+  slp->project = MS_TRUE; 
+
   if(qlayer < 0 || qlayer >= map->numlayers)
     start = map->numlayers-1;
   else
@@ -592,6 +604,10 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
     
     lp = (GET_LAYER(map, l));
 
+    /* conditions may have changed since this layer last drawn, so set 
+       layer->project true to recheck projection needs (Bug #673) */ 
+    lp->project = MS_TRUE; 
+
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
       if(lp->resultcache->results) free(lp->resultcache->results);
@@ -716,7 +732,7 @@ int msQueryByFeatures(mapObj *map, int qlayer, int slayer)
 	  switch(shape.type) { /* make sure shape actually intersects the selectshape */
 	  case MS_SHAPE_POINT:
 	    if(tolerance == 0) /* just test for intersection */
-	      status = msIntersectMultipointPolygon(&shape.line[0], &selectshape);
+	      status = msIntersectMultipointPolygon(&shape, &selectshape);
 	    else { /* check distance, distance=0 means they intersect */
 	      distance = msDistanceShapeToShape(&selectshape, &shape);
 	      if(distance < tolerance) status = MS_TRUE;
@@ -805,6 +821,10 @@ int msQueryByPoint(mapObj *map, int qlayer, int mode, pointObj p, double buffer)
   for(l=start; l>=stop; l--) {
     lp = (GET_LAYER(map, l));    
 
+    /* conditions may have changed since this layer last drawn, so set 
+       layer->project true to recheck projection needs (Bug #673) */ 
+    lp->project = MS_TRUE; 
+
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
       if(lp->resultcache->results) free(lp->resultcache->results);
@@ -965,6 +985,10 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
   for(l=start; l>=stop; l--) { /* each layer */
     lp = (GET_LAYER(map, l));
 
+    /* conditions may have changed since this layer last drawn, so set 
+       layer->project true to recheck projection needs (Bug #673) */ 
+    lp->project = MS_TRUE; 
+
     /* free any previous search results, do it now in case one of the next few tests fail */
     if(lp->resultcache) {
       if(lp->resultcache->results) free(lp->resultcache->results);
@@ -1064,7 +1088,7 @@ int msQueryByShape(mapObj *map, int qlayer, shapeObj *selectshape)
       switch(shape.type) { /* make sure shape actually intersects the shape */
       case MS_SHAPE_POINT:
         if(tolerance == 0) /* just test for intersection */
-	  status = msIntersectMultipointPolygon(&shape.line[0], selectshape);
+	  status = msIntersectMultipointPolygon(&shape, selectshape);
 	else { /* check distance, distance=0 means they intersect */
 	  distance = msDistanceShapeToShape(selectshape, &shape);
 	  if(distance < tolerance) status = MS_TRUE;
@@ -1157,6 +1181,10 @@ int msQueryByOperator(mapObj *map, int qlayer, shapeObj *selectshape,
     for(l=start; l>=stop; l--) { /* each layer */
       lp = (GET_LAYER(map, l));
 
+      /* conditions may have changed since this layer last drawn, so set 
+         layer->project true to recheck projection needs (Bug #673) */ 
+      lp->project = MS_TRUE; 
+
       /* free any previous search results, do it now in case one of the next few tests fail */
       if(lp->resultcache) {
         if(lp->resultcache->results) free(lp->resultcache->results);
@@ -1348,8 +1376,9 @@ int msGetQueryResultBounds(mapObj *map, rectObj *bounds)
 {
   int i, found=0;
   rectObj tmpBounds;
-  
+
   for(i=0; i<map->numlayers; i++) {
+
     layerObj *lp;
     lp = (GET_LAYER(map, i));
 
