@@ -16,8 +16,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -26,101 +26,15 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.137  2006/09/24 02:42:12  frank
- * handle upsidedown images through resample logic. (bug 1904)
- *
- * Revision 1.136  2006/04/18 17:20:37  frank
- * Support large (>2GB) raster files relative to SHAPEPATH. (bug 1748)
- *
- * Revision 1.135  2006/02/18 21:14:55  frank
- * Fixed regex test.
- *
- * Revision 1.134  2005/12/15 16:37:06  frank
- * Last fix was for bug 1562
- *
- * Revision 1.133  2005/12/15 16:35:42  frank
- * Dont require valid projections before calling mapresample.c code
- *
- * Revision 1.132  2005/12/08 19:06:30  hobu
- * switch off SDE raster support
- *
- * Revision 1.131  2005/10/26 17:50:54  frank
- * Avoid type punning warnings in readGEOTiff().
- *
- * Revision 1.130  2005/10/25 16:16:02  assefa
- * Copy the filteritem and filter into the temporary layer
- * created when handling tileindex rasters. (Bug 1506).
- *
- * Revision 1.129  2005/09/21 01:18:33  frank
- * use mapresample.c if RESAMPLE is set
- *
- * Revision 1.128  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.127  2005/05/27 15:00:12  dan
- * New regex wrappers to solve issues with previous version (bug 1354)
- *
- * Revision 1.126  2005/03/08 18:24:50  frank
- * lock parser for evaluating parser expressions
- *
- * Revision 1.125  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.124  2005/01/28 06:16:54  sdlime
- * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
- *
- * Revision 1.123  2004/11/15 18:55:49  frank
- * added "experimental" drawSDE support.
- *
- * Revision 1.122  2004/11/05 03:08:23  frank
- * Cleanup layer level leaks under various error conditions.
- * http://mapserver.gis.umn.edu/bugs/show_bug.cgi?id=713
- *
- * Revision 1.121  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.120  2004/07/29 18:16:44  sdlime
- * Fixed an error with eppl7 code where row/col access to the GD pixels was reversed.
- *
- * Revision 1.119  2004/07/23 12:57:18  frank
- * improved header purpose text
- *
- * Revision 1.118  2004/06/15 16:08:52  dan
- * Fixed problem with tiled raster layers if there is no tile in the current
- * view (bug 729).
- *
- * Revision 1.117  2004/06/01 14:33:43  frank
- * Fixed bug 698 - filename from tileindex freed before use.
- *
- * Revision 1.116  2004/05/28 18:34:34  frank
- * removed drawERD() ... now handled by GDAL
- *
- * Revision 1.115  2004/05/28 05:12:34  sdlime
- * Fixed (I believe) bug 660, a memory leak in msDrawRasterLow()...
- *
- * Revision 1.114  2004/04/16 20:19:39  dan
- * Added try_addimage_if_notfound to msGetSymbolIndex() (bug 612)
- *
- * Revision 1.113  2004/04/06 18:49:37  frank
- * add function comment blocks to split things up a bit
- *
- * Revision 1.112  2004/04/06 06:44:59  sdlime
- * Working version of layer-based tiling for raster data. The layer does not have to be a shapefile.
- *
- * Revision 1.111  2004/03/31 14:44:48  frank
- * Added my standard headers so I can see the log messages.
- *
- */
+ ****************************************************************************/
 
 #include <assert.h>
-#include "map.h"
+#include "mapserver.h"
+#include "mapfile.h"
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapraster.c 5761 2006-09-24 02:42:12Z frank $")
+MS_CVSID("$Id: mapraster.c 6481 2007-08-06 18:16:54Z warmerdam $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -173,50 +87,50 @@ int msGetClass(layerObj *layer, colorObj *color)
   int status;
   int expresult;
   
-  if((layer->numclasses == 1) && !(layer->class[0].expression.string)) /* no need to do lookup */
+  if((layer->numclasses == 1) && !(layer->class[0]->expression.string)) /* no need to do lookup */
     return(0);
 
   if(!color) return(-1);
 
   for(i=0; i<layer->numclasses; i++) {
-    if (layer->class[i].expression.string == NULL) /* Empty expression - always matches */
+    if (layer->class[i]->expression.string == NULL) /* Empty expression - always matches */
       return(i);
-    switch(layer->class[i].expression.type) {
+    switch(layer->class[i]->expression.type) {
     case(MS_STRING):
       sprintf(tmpstr2, "%d %d %d", color->red, color->green, color->blue);
-      if(strcmp(layer->class[i].expression.string, tmpstr2) == 0) return(i); /* matched */
+      if(strcmp(layer->class[i]->expression.string, tmpstr2) == 0) return(i); /* matched */
       sprintf(tmpstr2, "%d", color->pen);
-      if(strcmp(layer->class[i].expression.string, tmpstr2) == 0) return(i); /* matched */
+      if(strcmp(layer->class[i]->expression.string, tmpstr2) == 0) return(i); /* matched */
       break;
     case(MS_REGEX):
-      if(!layer->class[i].expression.compiled) {
-	if(ms_regcomp(&(layer->class[i].expression.regex), layer->class[i].expression.string, MS_REG_EXTENDED|MS_REG_NOSUB) != 0) { /* compile the expression  */
+      if(!layer->class[i]->expression.compiled) {
+	if(ms_regcomp(&(layer->class[i]->expression.regex), layer->class[i]->expression.string, MS_REG_EXTENDED|MS_REG_NOSUB) != 0) { /* compile the expression  */
 	  msSetError(MS_REGEXERR, "Invalid regular expression.", "msGetClass()");
 	  return(-1);
 	}
-	layer->class[i].expression.compiled = MS_TRUE;
+	layer->class[i]->expression.compiled = MS_TRUE;
       }
 
       sprintf(tmpstr2, "%d %d %d", color->red, color->green, color->blue);
-      if(ms_regexec(&(layer->class[i].expression.regex), tmpstr2, 0, NULL, 0) == 0) return(i); /* got a match */
+      if(ms_regexec(&(layer->class[i]->expression.regex), tmpstr2, 0, NULL, 0) == 0) return(i); /* got a match */
       sprintf(tmpstr2, "%d", color->pen);
-      if(ms_regexec(&(layer->class[i].expression.regex), tmpstr2, 0, NULL, 0) == 0) return(i); /* got a match */
+      if(ms_regexec(&(layer->class[i]->expression.regex), tmpstr2, 0, NULL, 0) == 0) return(i); /* got a match */
       break;
     case(MS_EXPRESSION):
-      tmpstr1 = strdup(layer->class[i].expression.string);
+      tmpstr1 = strdup(layer->class[i]->expression.string);
 
       sprintf(tmpstr2, "%d", color->red);
-      tmpstr1 = gsub(tmpstr1, "[red]", tmpstr2);
+      tmpstr1 = msReplaceSubstring(tmpstr1, "[red]", tmpstr2);
       sprintf(tmpstr2, "%d", color->green);
-      tmpstr1 = gsub(tmpstr1, "[green]", tmpstr2);
+      tmpstr1 = msReplaceSubstring(tmpstr1, "[green]", tmpstr2);
       sprintf(tmpstr2, "%d", color->blue);
-      tmpstr1 = gsub(tmpstr1, "[blue]", tmpstr2);
+      tmpstr1 = msReplaceSubstring(tmpstr1, "[blue]", tmpstr2);
 
       sprintf(tmpstr2, "%d", color->pen);
-      tmpstr1 = gsub(tmpstr1, "[pixel]", tmpstr2);
+      tmpstr1 = msReplaceSubstring(tmpstr1, "[pixel]", tmpstr2);
 
       msAcquireLock( TLOCK_PARSER );
-      msyystate = 4; msyystring = tmpstr1;
+      msyystate = MS_TOKENIZE_EXPRESSION; msyystring = tmpstr1;
       status = msyyparse();
       expresult = msyyresult;
       msReleaseLock( TLOCK_PARSER );
@@ -245,40 +159,40 @@ int msGetClass_Float(layerObj *layer, float fValue)
     char tmpstr2[100];
     int status, expresult;
   
-    if((layer->numclasses == 1) && !(layer->class[0].expression.string)) /* no need to do lookup */
+    if((layer->numclasses == 1) && !(layer->class[0]->expression.string)) /* no need to do lookup */
         return(0);
 
     for(i=0; i<layer->numclasses; i++) {
-        if (layer->class[i].expression.string == NULL) /* Empty expression - always matches */
+        if (layer->class[i]->expression.string == NULL) /* Empty expression - always matches */
             return(i);
 
-        switch(layer->class[i].expression.type) {
+        switch(layer->class[i]->expression.type) {
           case(MS_STRING):
             sprintf(tmpstr2, "%18g", fValue );
-            if(strcmp(layer->class[i].expression.string, tmpstr2) == 0) return(i); /* matched */
+            if(strcmp(layer->class[i]->expression.string, tmpstr2) == 0) return(i); /* matched */
             break;
 
           case(MS_REGEX):
-            if(!layer->class[i].expression.compiled) {
-                if(ms_regcomp(&(layer->class[i].expression.regex), layer->class[i].expression.string, MS_REG_EXTENDED|MS_REG_NOSUB) != 0) { /* compile the expression  */
+            if(!layer->class[i]->expression.compiled) {
+                if(ms_regcomp(&(layer->class[i]->expression.regex), layer->class[i]->expression.string, MS_REG_EXTENDED|MS_REG_NOSUB) != 0) { /* compile the expression  */
                     msSetError(MS_REGEXERR, "Invalid regular expression.", "msGetClass()");
                     return(-1);
                 }
-                layer->class[i].expression.compiled = MS_TRUE;
+                layer->class[i]->expression.compiled = MS_TRUE;
             }
 
             sprintf(tmpstr2, "%18g", fValue );
-            if(ms_regexec(&(layer->class[i].expression.regex), tmpstr2, 0, NULL, 0) == 0) return(i); /* got a match */
+            if(ms_regexec(&(layer->class[i]->expression.regex), tmpstr2, 0, NULL, 0) == 0) return(i); /* got a match */
             break;
 
           case(MS_EXPRESSION):
-            tmpstr1 = strdup(layer->class[i].expression.string);
+            tmpstr1 = strdup(layer->class[i]->expression.string);
 
             sprintf(tmpstr2, "%18g", fValue);
-            tmpstr1 = gsub(tmpstr1, "[pixel]", tmpstr2);
+            tmpstr1 = msReplaceSubstring(tmpstr1, "[pixel]", tmpstr2);
 
             msAcquireLock( TLOCK_PARSER );
-            msyystate = 4; msyystring = tmpstr1;
+            msyystate = MS_TOKENIZE_EXPRESSION; msyystring = tmpstr1;
             status = msyyparse();
             expresult = msyyresult;
             msReleaseLock( TLOCK_PARSER );
@@ -409,6 +323,13 @@ int msAddColorGD(mapObj *map, gdImagePtr img, int cmt, int r, int g, int b)
   return op; /* Return newly allocated color */  
 }
 
+#ifdef USE_AGG
+int msAddColorAGG(mapObj *map, gdImagePtr img, int cmt, int r, int g, int b)
+{
+	 return msAddColorGD( map, img, cmt, r, g, b );
+}
+#endif
+
 /************************************************************************/
 /*                           readWorldFile()                            */
 /*                                                                      */
@@ -497,7 +418,7 @@ static int readGEOTiff(TIFF *tif, double *ulx, double *uly, double *cx, double *
   tiepos=0; cellpos=0;
   for (i=0; i<entries; i++) {
     fread(&tdir,sizeof(tdir),1,f);
-    if (swap) TIFFSwabShort(&tdir.tdir_tag);
+    if (swap) TIFFSwabShort((unsigned short *) &tdir.tdir_tag);
     if (tdir.tdir_tag==0x8482) {  /* geoTIFF tie point */
       tiepos=tdir.tdir_offset;
       if (swap) TIFFSwabLong(&tiepos);
@@ -615,10 +536,10 @@ static int drawTIFF(mapObj *map, layerObj *layer, gdImagePtr img, char *filename
 	  if(c == -1) /* doesn't belong to any class, so handle like offsite */
 	    cmap[i] = -1;
 	  else {
-            RESOLVE_PEN_GD(img, layer->class[c].styles[0].color);
-	    if(MS_VALID_COLOR(layer->class[c].styles[0].color)) 
-	      cmap[i] = msAddColorGD(map,img, 0, layer->class[c].styles[0].color.red, layer->class[c].styles[0].color.green, layer->class[c].styles[0].color.blue); /* use class color */
-	    else if(MS_TRANSPARENT_COLOR(layer->class[c].styles[0].color)) 
+            RESOLVE_PEN_GD(img, layer->class[c]->styles[0]->color);
+	    if(MS_VALID_COLOR(layer->class[c]->styles[0]->color)) 
+	      cmap[i] = msAddColorGD(map,img, 0, layer->class[c]->styles[0]->color.red, layer->class[c]->styles[0]->color.green, layer->class[c]->styles[0]->color.blue); /* use class color */
+	    else if(MS_TRANSPARENT_COLOR(layer->class[c]->styles[0]->color)) 
 	      cmap[i] = -1; /* make transparent */
 	    else
               cmap[i] = msAddColorGD(map,img, 0, pixel.red, pixel.green, pixel.blue); /* use raster color */
@@ -833,10 +754,10 @@ static int drawPNG(mapObj *map, layerObj *layer, gdImagePtr img, char *filename)
 	if(c == -1) /* doesn't belong to any class, so handle like offsite */
 	  cmap[i] = -1;
 	else {
-          RESOLVE_PEN_GD(img, layer->class[c].styles[0].color);
-          if(MS_VALID_COLOR(layer->class[c].styles[0].color)) 
-	    cmap[i] = msAddColorGD(map,img, 0, layer->class[c].styles[0].color.red, layer->class[c].styles[0].color.green, layer->class[c].styles[0].color.blue); /* use class color */
-	  else if(MS_TRANSPARENT_COLOR(layer->class[c].styles[0].color)) 
+          RESOLVE_PEN_GD(img, layer->class[c]->styles[0]->color);
+          if(MS_VALID_COLOR(layer->class[c]->styles[0]->color)) 
+	    cmap[i] = msAddColorGD(map,img, 0, layer->class[c]->styles[0]->color.red, layer->class[c]->styles[0]->color.green, layer->class[c]->styles[0]->color.blue); /* use class color */
+	  else if(MS_TRANSPARENT_COLOR(layer->class[c]->styles[0]->color)) 
 	    cmap[i] = -1; /* make transparent */
 	  else
             cmap[i] = msAddColorGD(map,img, 0, pixel.red, pixel.green, pixel.blue); /* use raster color	   */
@@ -954,10 +875,10 @@ static int drawGIF(mapObj *map, layerObj *layer, gdImagePtr img, char *filename)
         if(c == -1) /* doesn't belong to any class, so handle like offsite */
 	  cmap[i] = -1;
 	else {
-          RESOLVE_PEN_GD(img, layer->class[c].styles[0].color);
-          if(MS_VALID_COLOR(layer->class[c].styles[0].color)) 
-	    cmap[i] = msAddColorGD(map,img, 0, layer->class[c].styles[0].color.red, layer->class[c].styles[0].color.green, layer->class[c].styles[0].color.blue); /* use class color */
-	  else if(MS_TRANSPARENT_COLOR(layer->class[c].styles[0].color)) 
+          RESOLVE_PEN_GD(img, layer->class[c]->styles[0]->color);
+          if(MS_VALID_COLOR(layer->class[c]->styles[0]->color)) 
+	    cmap[i] = msAddColorGD(map,img, 0, layer->class[c]->styles[0]->color.red, layer->class[c]->styles[0]->color.green, layer->class[c]->styles[0]->color.blue); /* use class color */
+	  else if(MS_TRANSPARENT_COLOR(layer->class[c]->styles[0]->color)) 
 	    cmap[i] = -1; /* make transparent */
 	  else
             cmap[i] = msAddColorGD(map,img, 0, pixel.red, pixel.green, pixel.blue); /* use raster color	   */
@@ -1201,10 +1122,10 @@ static int drawEPP(mapObj *map, layerObj *layer, gdImagePtr img, char *filename)
 	  if(c == -1) 
 	    cmap[i] = -1;
 	  else {
-            RESOLVE_PEN_GD(img, layer->class[c].styles[0].color);
-	    if(MS_VALID_COLOR(layer->class[c].styles[0].color)) 
-	      cmap[i] = msAddColorGD(map,img, 0, layer->class[c].styles[0].color.red, layer->class[c].styles[0].color.green, layer->class[c].styles[0].color.blue); /* use class color */
-	    else if(MS_TRANSPARENT_COLOR(layer->class[c].styles[0].color)) 
+            RESOLVE_PEN_GD(img, layer->class[c]->styles[0]->color);
+	    if(MS_VALID_COLOR(layer->class[c]->styles[0]->color)) 
+	      cmap[i] = msAddColorGD(map,img, 0, layer->class[c]->styles[0]->color.red, layer->class[c]->styles[0]->color.green, layer->class[c]->styles[0]->color.blue); /* use class color */
+	    else if(MS_TRANSPARENT_COLOR(layer->class[c]->styles[0]->color)) 
 	      cmap[i] = -1; /* make transparent */
 	    else {              
 	      clrget(&clr,i,&color);
@@ -1304,13 +1225,13 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
     return(0);
   }
 
-  if(map->scale > 0) {
-    if((layer->maxscale > 0) && (map->scale > layer->maxscale)) {
-      if(layer->debug == MS_TRUE) msDebug( "msDrawRasterLayerLow(%s): skipping, map scale %.2g > MAXSCALE=%g\n", layer->name, map->scale, layer->maxscale );
+  if(map->scaledenom > 0) {
+    if((layer->maxscaledenom > 0) && (map->scaledenom > layer->maxscaledenom)) {
+      if(layer->debug == MS_TRUE) msDebug( "msDrawRasterLayerLow(%s): skipping, map scale %.2g > MAXSCALEDENOM=%g\n", layer->name, map->scaledenom, layer->maxscaledenom );
       return(0);
     }
-    if((layer->minscale > 0) && (map->scale <= layer->minscale)) {
-      if(layer->debug == MS_TRUE) msDebug( "msDrawRasterLayerLow(%s): skipping, map scale %.2g < MINSCALE=%g\n", layer->name, map->scale, layer->minscale );
+    if((layer->minscaledenom > 0) && (map->scaledenom <= layer->minscaledenom)) {
+      if(layer->debug == MS_TRUE) msDebug( "msDrawRasterLayerLow(%s): skipping, map scale %.2g < MINSCALEDENOM=%g\n", layer->name, map->scaledenom, layer->minscaledenom );
       return(0);
     }
   }
@@ -1318,6 +1239,10 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
   force_gdal = MS_FALSE;
   if(MS_RENDERER_GD(image->format))
     img = image->img.gd;
+#ifdef USE_AGG
+  else if(MS_RENDERER_AGG(image->format))
+    img = image->img.gd;
+#endif
   else {
     img = NULL;
     force_gdal = MS_TRUE;
@@ -1395,9 +1320,11 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
           tlp->filter.type = layer->filter.type;
       }
 
-    } else
-      tlp = &(layer->map->layers[tilelayerindex]);
-      
+    } else {
+    	if ( msCheckParentPointer(layer->map,"map")==MS_FAILURE )
+			return MS_FAILURE;
+	tlp = (GET_LAYER(layer->map, tilelayerindex));
+    }
     status = msLayerOpen(tlp);
     if(status != MS_SUCCESS)
     {
