@@ -29,7 +29,7 @@
 #include <assert.h>
 #include "mapresample.h"
 
-MS_CVSID("$Id: mapresample.c 6881 2007-09-10 20:21:02Z warmerdam $")
+MS_CVSID("$Id: mapresample.c 7131 2007-11-30 22:13:28Z warmerdam $")
 
 #ifndef MAX
 #  define MIN(a,b)      ((a<b) ? a : b)
@@ -597,21 +597,21 @@ msBilinearRasterResampler( imageObj *psSrcImage, colorObj offsite,
                         psDstImage->img.raw_16bit[
                             nDstX + nDstY * psDstImage->width
                             + band*psDstImage->width*psDstImage->height] 
-                            = (short) padfPixelSum[0];
+                            = (short) padfPixelSum[band];
                     }
                     else if( psSrcImage->format->imagemode == MS_IMAGEMODE_FLOAT32)
                     {
                         psDstImage->img.raw_float[
                             nDstX + nDstY * psDstImage->width
                             + band*psDstImage->width*psDstImage->height] 
-                            = (float) padfPixelSum[0];
+                            = (float) padfPixelSum[band];
                     }
                     else if( psSrcImage->format->imagemode == MS_IMAGEMODE_BYTE )
                     {
                         psDstImage->img.raw_byte[
                             nDstX + nDstY * psDstImage->width
                             + band*psDstImage->width*psDstImage->height] 
-                            = (unsigned char) padfPixelSum[0];
+                            = (unsigned char) padfPixelSum[band];
                     }
                 }
             }
@@ -863,21 +863,21 @@ msAverageRasterResampler( imageObj *psSrcImage, colorObj offsite,
                         psDstImage->img.raw_16bit[
                             nDstX + nDstY * psDstImage->width
                             + band*psDstImage->width*psDstImage->height] 
-                            = (short) padfPixelSum[0];
+                            = (short) padfPixelSum[band];
                     }
                     else if( psSrcImage->format->imagemode == MS_IMAGEMODE_FLOAT32)
                     {
                         psDstImage->img.raw_float[
                             nDstX + nDstY * psDstImage->width
                             + band*psDstImage->width*psDstImage->height] 
-                            = (float) padfPixelSum[0];
+                            = (float) padfPixelSum[band];
                     }
                     else if( psSrcImage->format->imagemode == MS_IMAGEMODE_BYTE )
                     {
                         psDstImage->img.raw_byte[
                             nDstX + nDstY * psDstImage->width
                             + band*psDstImage->width*psDstImage->height] 
-                            = (unsigned char) padfPixelSum[0];
+                            = (unsigned char) padfPixelSum[band];
                     }
                 }
             }
