@@ -24,41 +24,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.52  2006/08/26 22:04:13  novak
- * Enable support for TrueType fonts (bug 1882)
- *
- * Revision 1.51  2006/03/27 05:48:03  sdlime
- * Fixed symbol initialization error with embedded scalebars and legends. (bug 1725)
- *
- * Revision 1.50  2006/02/18 20:59:13  sdlime
- * Initial code for curved labels. (bug 1620)
- *
- * Revision 1.49  2005/09/27 15:27:18  sean
- * Fixed typo to prevent extra scalebar layer creation (bug 1480)
- *
- * Revision 1.48  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.47  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.46  2005/01/11 00:24:07  frank
- * added labelObj arg to msAddLabel()
- *
- * Revision 1.45  2004/11/05 19:22:19  frank
- * ensure state is initialized in msDrawScalebar() (all cases)
- *
- * Revision 1.44  2004/10/21 04:30:56  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapscale.c 5677 2006-08-26 22:04:13Z novak $")
+MS_CVSID("$Id: mapscale.c 6567 2007-08-13 08:20:26Z tbonfort $")
 
 #define VMARGIN 3 /* buffer around the scalebar */
 #define HMARGIN 3
@@ -66,7 +36,7 @@ MS_CVSID("$Id: mapscale.c 5677 2006-08-26 22:04:13Z novak $")
 #define VSLOP 5 /* makes things fit a bit better vertically */
 
 /*
-** Match this with with unit enumerations is map.h
+** Match this with with unit enumerations is mapserver.h
 */
 static char *unitText[5]={"in", "ft", "mi", "m", "km"};
 double inchesPerUnit[6]={1, 12, 63360.0, 39.3701, 39370.1, 4374754};
@@ -123,7 +93,7 @@ int msCalculateScale(rectObj extent, int units, int width, int height, double re
   case(MS_INCHES):  
   case(MS_FEET):
     center_y = (extent.miny+extent.maxy)/2.0;
-    md = width/(resolution*msInchesPerUnit(units, center_y)); /* was (width-1) */
+    md = (width-1)/(resolution*msInchesPerUnit(units, center_y)); /* remember, we use a pixel-center to pixel-center extent, hence the width-1 */
     gd = extent.maxx - extent.minx;
     *scale = gd/md;
     break;
@@ -181,7 +151,8 @@ imageObj *msDrawScalebar(mapObj *map)
   double i, msx;
   int j;
   int isx, sx, sy, ox, oy, state, dsx;
-  pointObj p;
+  pointObj p,p2;
+  rectObj r;
   gdFontPtr fontPtr = NULL;
   imageObj      *image = NULL;
   outputFormatObj *format = NULL;
@@ -253,7 +224,7 @@ imageObj *msDrawScalebar(mapObj *map)
   if(!fontPtr) return(NULL);
 
   map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
-  status = msCalculateScale(map->extent, map->units, map->width, map->height, map->resolution, &map->scale);
+  status = msCalculateScale(map->extent, map->units, map->width, map->height, map->resolution, &map->scaledenom);
   if(status != MS_SUCCESS)
   { 	
     if( iFreeGDFont ) free( fontPtr );
@@ -301,6 +272,7 @@ imageObj *msDrawScalebar(mapObj *map)
 
   msImageInitGD( image, &(map->scalebar.imagecolor));
   
+
   if (map->outputformat->imagemode == MS_IMAGEMODE_RGB || map->outputformat->imagemode == MS_IMAGEMODE_RGBA) gdImageAlphaBlending(image->img.gd, 1);
 
   ox = MS_NINT((map->scalebar.width - sx)/2.0 + fontPtr->w/2.0); /* center the computed scalebar */
@@ -328,8 +300,10 @@ imageObj *msDrawScalebar(mapObj *map)
       map->scalebar.label.position = MS_CC;
       p.x = ox + j*isx; /* + MS_NINT(fontPtr->w/2); */
       p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontPtr->h);
-      /* TODO */
-      msDrawLabel(image, p, label, &(map->scalebar.label), &(map->fontset), 1.0);
+      if(msGetLabelSize(label,&(map->scalebar.label), &r, 
+              &(map->fontset), 1, MS_FALSE) == -1) return(NULL);
+      p2 = get_metrics(&p, MS_CC, r, 0,0, 0, 0, NULL);
+      msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
       state = -state;
     }
@@ -339,8 +313,11 @@ imageObj *msDrawScalebar(mapObj *map)
     map->scalebar.label.position = MS_CR;
     p.x = ox; /* + MS_NINT(fontPtr->w/2); */
     p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontPtr->h);
-    /* TODO */
-    msDrawLabel(image, p, label, &(map->scalebar.label), &(map->fontset), 1.0);
+    if(msGetLabelSize(label,&(map->scalebar.label), &r, 
+            &(map->fontset), 1, MS_FALSE) == -1) return(NULL);
+    p2 = get_metrics(&p, MS_CR, r, 0,0, 0, 0, NULL);
+    msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
+
     break;
 
   case(1):
@@ -356,8 +333,10 @@ imageObj *msDrawScalebar(mapObj *map)
       map->scalebar.label.position = MS_CC;
       p.x = ox + j*isx; /* + MS_NINT(fontPtr->w/2); */
       p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontPtr->h);
-      /* TODO */
-      msDrawLabel(image, p, label, &(map->scalebar.label), &(map->fontset), 1.0);
+      if(msGetLabelSize(label,&(map->scalebar.label), &r, 
+              &(map->fontset), 1, MS_FALSE) == -1) return(NULL);
+      p2 = get_metrics(&p, MS_CC, r, 0,0, 0, 0, NULL);
+      msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
       state = -state;
     }
@@ -370,8 +349,11 @@ imageObj *msDrawScalebar(mapObj *map)
     map->scalebar.label.position = MS_CR;
     p.x = ox; /* + MS_NINT(fontPtr->w/2); */
     p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontPtr->h);
-    /* TODO */
-    msDrawLabel(image, p, label, &(map->scalebar.label), &(map->fontset), 1.0);
+    if(msGetLabelSize(label,&(map->scalebar.label), &r, 
+            &(map->fontset), 1, MS_FALSE) == -1) return(NULL);
+    p2 = get_metrics(&p, MS_CR, r, 0,0, 0, 0, NULL);
+    msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
+
     break;
   default:
     msSetError(MS_MISCERR, "Unsupported scalebar style.", "msDrawScalebar()");
@@ -384,7 +366,6 @@ imageObj *msDrawScalebar(mapObj *map)
   msClearScalebarPenValues( &(map->scalebar));
 
   if( iFreeGDFont ) free( fontPtr );
-
   return(image);
 
 }
@@ -397,93 +378,101 @@ int msEmbedScalebar(mapObj *map, gdImagePtr img)
 
   s = msGetSymbolIndex(&(map->symbolset), "scalebar", MS_FALSE);
   if(s == -1) {
+    if (msGrowSymbolSet(&map->symbolset) == NULL)
+        return -1;
     s = map->symbolset.numsymbols;
     map->symbolset.numsymbols++;
-    initSymbol(&(map->symbolset.symbol[s]));
+    initSymbol(map->symbolset.symbol[s]);
   } else {
-    if(map->symbolset.symbol[s].img) 
-      gdImageDestroy(map->symbolset.symbol[s].img);
+    if(map->symbolset.symbol[s]->img) 
+      gdImageDestroy(map->symbolset.symbol[s]->img);
   }
   
   image = msDrawScalebar(map);
-  map->symbolset.symbol[s].img =  image->img.gd; /* TODO  */
-  if(!map->symbolset.symbol[s].img) return(-1); /* something went wrong creating scalebar */
+  map->symbolset.symbol[s]->img =  image->img.gd; /* TODO  */
+  if(!map->symbolset.symbol[s]->img) return(-1); /* something went wrong creating scalebar */
 
-  map->symbolset.symbol[s].type = MS_SYMBOL_PIXMAP; /* intialize a few things */
-  map->symbolset.symbol[s].name = strdup("scalebar");  
-  map->symbolset.symbol[s].sizex = map->symbolset.symbol[s].img->sx;
-  map->symbolset.symbol[s].sizey = map->symbolset.symbol[s].img->sy;
+  map->symbolset.symbol[s]->type = MS_SYMBOL_PIXMAP; /* intialize a few things */
+  map->symbolset.symbol[s]->name = strdup("scalebar");  
+  map->symbolset.symbol[s]->sizex = map->symbolset.symbol[s]->img->sx;
+  map->symbolset.symbol[s]->sizey = map->symbolset.symbol[s]->img->sy;
 
   /* in 8 bit, mark 0 as being transparent.  In 24bit hopefully already
      have transparency enabled ... */
   if(map->scalebar.transparent == MS_ON 
      && !gdImageTrueColor(image->img.gd ) )
-    gdImageColorTransparent(map->symbolset.symbol[s].img, 0);
+    gdImageColorTransparent(map->symbolset.symbol[s]->img, 0);
 
   switch(map->scalebar.position) {
   case(MS_LL):
-    point.x = MS_NINT(map->symbolset.symbol[s].img->sx/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.x = MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
+    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_LR):
-    point.x = map->width - MS_NINT(map->symbolset.symbol[s].img->sx/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.x = map->width - MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
+    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_LC):
     point.x = MS_NINT(map->width/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_UR):
-    point.x = map->width - MS_NINT(map->symbolset.symbol[s].img->sx/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.x = map->width - MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
+    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_UL):
-    point.x = MS_NINT(map->symbolset.symbol[s].img->sx/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.x = MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
+    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   case(MS_UC):
     point.x = MS_NINT(map->width/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s].img->sy/2.0);
+    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
     break;
   }
 
   l = msGetLayerIndex(map, "__embed__scalebar");
   if(l == -1) {
+    if (msGrowMapLayers(map) == NULL)
+        return(-1);
     l = map->numlayers;
     map->numlayers++;
+    if(initLayer((GET_LAYER(map, l)), map) == -1) return(-1);
+    GET_LAYER(map, l)->name = strdup("__embed__scalebar");
+    GET_LAYER(map, l)->type = MS_LAYER_ANNOTATION;
 
-    if(initLayer(&(map->layers[l]), map) == -1) return(-1);
-    map->layers[l].name = strdup("__embed__scalebar");
-    map->layers[l].type = MS_LAYER_ANNOTATION;
+    if (msGrowLayerClasses( GET_LAYER(map, l) ) == NULL)
+        return(-1);
 
-    if(initClass(&(map->layers[l].class[0])) == -1) return(-1);
-    map->layers[l].numclasses = 1; /* so we make sure to free it */
+    if(initClass(GET_LAYER(map, l)->class[0]) == -1) return(-1);
+    GET_LAYER(map, l)->numclasses = 1; /* so we make sure to free it */
     
     /* update the layer order list with the layer's index. */
     map->layerorder[l] = l;
   }
 
   /* to resolve bug 490 */
-  map->layers[l].transparency = MS_GD_ALPHA;
+  GET_LAYER(map, l)->opacity = MS_GD_ALPHA;
   
-  map->layers[l].status = MS_ON;
+  GET_LAYER(map, l)->status = MS_ON;
 
-  map->layers[l].class[0].numstyles = 1;
-  map->layers[l].class[0].styles[0].symbol = s;
-  map->layers[l].class[0].styles[0].color.pen = -1;
-  map->layers[l].class[0].label.force = MS_TRUE;
-  map->layers[l].class[0].label.size = MS_MEDIUM; /* must set a size to have a valid label definition */
+/* TODO: Change this when we get rid of MS_MAXSTYLES */
+  if (msMaybeAllocateStyle(GET_LAYER(map, l)->class[0], 0)==MS_FAILURE) return MS_FAILURE;
+  GET_LAYER(map, l)->class[0]->styles[0]->symbol = s;
+  GET_LAYER(map, l)->class[0]->styles[0]->color.pen = -1;
+  GET_LAYER(map, l)->class[0]->label.force = MS_TRUE;
+  GET_LAYER(map, l)->class[0]->label.size = MS_MEDIUM; /* must set a size to have a valid label definition */
+  GET_LAYER(map, l)->class[0]->label.priority = MS_MAX_LABEL_PRIORITY;
 
   if(map->scalebar.postlabelcache) /* add it directly to the image //TODO */
   {
-    msDrawMarkerSymbolGD(&map->symbolset, img, &point, &(map->layers[l].class[0].styles[0]), 1.0);
+    msDrawMarkerSymbolGD(&map->symbolset, img, &point, GET_LAYER(map, l)->class[0]->styles[0], 1.0);
   }
   else
     msAddLabel(map, l, 0, -1, -1, &point, NULL, " ", 1.0, NULL);
 
   /* Mark layer as deleted so that it doesn't interfere with html legends or */
   /* with saving maps */
-  map->layers[l].status = MS_DELETE;
+  GET_LAYER(map, l)->status = MS_DELETE;
 
   /* Free image (but keep the GD image which is in the symbol cache now) */
   image->img.gd = NULL;
