@@ -1,4 +1,5 @@
 using System;
+using OSGeo.MapServer;
 
 /**
  * <p>Title: Mapscript shape dump example.</p>
@@ -30,7 +31,7 @@ class ShapeDump {
 	  Console.WriteLine("bounds (" + s_obj.bounds.minx + ", " +  s_obj.bounds.miny + ") (" + s_obj.bounds.maxx + ", " + s_obj.bounds.maxy + ")" );
 	  for(int j=0; j<s_obj.numlines; j++) 
 	  {
-		lineObj l_obj = s_obj.get(i);
+		lineObj l_obj = s_obj.get(j);
 		Console.WriteLine("Part " + j + " has " + l_obj.numpoints + " points.");
                     
 		for(int k=0; k < l_obj.numpoints; k++) {
