@@ -1,4 +1,4 @@
-// $Id: MapThread.java 5643 2006-08-25 07:15:45Z umberto $
+// $Id: MapThread.java 6897 2007-09-19 13:25:02Z tbonfort $
 //
 // See README_THREADTEST.TXT for usage details.
 //
@@ -68,7 +68,7 @@ public class MapThread extends Thread {
 				//bufferLayer.setType(mapscriptConstants.MS_LAYER_POLYGON);
 				bufferLayer.setProjection("init=epsg:4326");
 				bufferLayer.setType(MS_LAYER_TYPE.MS_LAYER_POINT);
-				bufferLayer.setTransparency(50);
+				bufferLayer.setOpacity(50);
 				classObj clazz=new classObj(bufferLayer);
 				clazz.setName("Buffer class");
 				styleObj style=new styleObj(clazz);
