@@ -4,132 +4,35 @@
  *
  * This file was originally based on the SWIG interface file mapscript.i
  *
- * $Id: mapscript_i.c 5722 2006-09-01 02:30:15Z sdlime $
+ * $Id: mapscript_i.c 6601 2007-08-16 15:34:18Z dmorissette $
  *
- * $Log$
- * Revision 1.102  2006/09/01 02:30:15  sdlime
- * Dan beat me to the bug 1428 fix. I took a bit futher by removing msLayerGetFilterString() from layerobject.c and refer to that in the mapscript getFilter/getFilterString methods.
+ **********************************************************************
+ * Copyright (c) 2000, 2007, Daniel Morissette, DM Solutions Group
  *
- * Revision 1.101  2006/08/31 20:48:47  dan
- * Fixed MapScript getExpressionString() that was failing on expressions
- * longer that 256 chars (SWIG) and 512 chars (PHP). Also moved all that
- * code to a msGetExpressionString() in mapfile.c (bug 1428)
- *
- * Revision 1.100  2006/08/29 01:56:53  sdlime
- * Fixed buffer overflow with POSTs and huge numbers of name/value pairs. Reduced MAX_PARAMS (now MS_MAX_CGI_PARAMS) from 10,000 to 100.
- *
- * Revision 1.99  2006/08/22 15:55:03  assefa
- * Adding geos functions to php mapscript (Bug 1327)
- *
- * Revision 1.98  2006/05/29 19:02:01  assefa
- * Update PHP mapscript to support addition of MapScript WxS Services
- * (RFC 16, Bug 1790)
- *
- * Revision 1.97  2006/05/17 16:04:55  assefa
- * Add geos functions union, difference and intersection (Bug 1778)
- *
- * Revision 1.96  2006/01/20 15:03:35  assefa
- * Add containsshape function using uderlying GEOS function (Bug 1623).
- *
- * Revision 1.95  2005/10/13 16:00:36  assefa
- * Add support for whichshape and nextshape (Bug 1491)
- * Adsd support for config setting/getting at the map level (Bug 1487).
- *
- * Revision 1.94  2005/09/08 19:24:50  assefa
- * Expose GEOS operations through PHP Mapscript (Bug 1327).
- * Initially only functions buffer and convexhull on a shape object
- * are available.
- *
- * Revision 1.93  2005/06/06 05:48:20  dan
- * Added $layerObj->removeClass() (was already in SWIG MapScript)
- * Added $layerObj->removeClass() to PHP MapScript (was already in SWIG
- * MapScript, bug 1373)
- *
- * Revision 1.92  2005/04/21 15:09:29  julien
- * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
- *
- * Revision 1.91  2005/04/14 15:17:15  julien
- * Bug 1244: Remove Z and M from point by default to gain performance.
- *
- * Revision 1.90  2004/11/12 18:43:01  assefa
- * rectObj memebers are initialized ot -1 (Bug 788).
- *
- * Revision 1.89  2004/11/02 21:03:36  assefa
- * Add a 2nd optional argument to LoadMapContext function (Bug 1023).
- *
- * Revision 1.88  2004/10/28 18:16:17  dan
- * Fixed WMS GetLegendGraphic which was returning an exception (GD error)
- * when requested layer was out of scale (bug 1006)
- *
- * Revision 1.87  2004/10/26 17:42:48  dan
- * Temporarily force layer status to MS_ON in layer->query*() methods and
- * restore status before returning (bug 925)
- *
- * Revision 1.86  2004/10/09 18:22:41  sean
- * towards resolving bug 339, have implemented a mutex acquiring wrapper for
- * the loadExpressionString function.  the new msLoadExpressionString should be
- * used everywhere outside of the mapfile loading phase, and the previous
- * loadExpressionString function should be used within the mapfile loading
- * phase.
- *
- * Revision 1.85  2004/10/08 22:40:07  dan
- * Moved mapscript's prepareImage() logic into msPrepareImage() which is
- * also going to be used by msDrawMap(). (bug 945)
- *
- * Revision 1.84  2004/08/12 17:18:26  assefa
- * Check if string is null in classObj_getExpressionString.
- *
- * Revision 1.83  2004/07/28 22:03:50  dan
- * Added layer->getFilter() to PHP MapScript (bug 787)
- *
- * Revision 1.82  2004/07/28 21:45:10  assefa
- * Function msImageCreate have an additional argument (map object).
- *
- * Revision 1.81  2004/07/22 19:44:10  assefa
- * Function names have cganhed to use the ms prefix.
- *
- * Revision 1.80  2004/06/23 20:17:57  dan
- * Updated current methods for changes to the hashTableObj (bug 737)
- *
- * Revision 1.79  2004/05/31 15:35:03  dan
- * Added setRotation() (bug 702) and fixed layer->drawQuery() (bug 695)
- *
- * Revision 1.78  2004/04/16 20:19:39  dan
- * Added try_addimage_if_notfound to msGetSymbolIndex() (bug 612)
- *
- * Revision 1.77  2004/02/16 19:19:20  dan
- * Free expression in setExpression() if expression is null or empty.
- *
- * Revision 1.76  2004/01/30 17:01:12  assefa
- * Add function deletestyle on a class object.
- *
- * Revision 1.75  2004/01/13 23:52:39  assefa
- * Add functions to move styles up and down.
- * Add function to clone style.
- *
- * Revision 1.74  2004/01/12 19:56:18  assefa
- * Add moveclassup and moveclassdown on a layer object.
- * Add clone function for the class object.
- * Add a 2nd optional argument for function ms_newclassobj to be able
- * to pass a class as argument.
- *
- * Revision 1.73  2004/01/05 21:27:14  assefa
- * applySLDURL and applySLD on a layer object can now take an optional
- * argument which is the name of the NamedLayer to use to style the layer.
- *
- * ...
- *
- * Revision 1.3  2000/03/11 21:53:27  daniel
- * Ported extension to MapServer version 3.3.008
- *
- */
-
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * DEALINGS IN THE SOFTWARE.
+ **********************************************************************/ 
 
 #include "php_mapscript.h"
 
 /* grab mapserver declarations to wrap
  */
-#include "../../map.h"
+#include "../../mapserver.h"
 #include "../../maperror.h"
 #include "../../mapprimitive.h"
 #include "../../mapsymbol.h"
@@ -168,7 +71,7 @@ int mapObj_setRotation(mapObj* self, double rotation_angle ) {
 
 layerObj *mapObj_getLayer(mapObj* self, int i) {
     if(i >= 0 && i < self->numlayers)	
-      return &(self->layers[i]); /* returns an EXISTING layer */
+      return (self->layers[i]); /* returns an EXISTING layer */
     else
       return NULL;
   }
@@ -179,7 +82,7 @@ layerObj *mapObj_getLayerByName(mapObj* self, char *name) {
     i = msGetLayerIndex(self, name);
 
     if(i != -1)
-      return &(self->layers[i]); /* returns an EXISTING layer */
+      return (self->layers[i]); /* returns an EXISTING layer */
     else
       return NULL;
   }
@@ -197,8 +100,8 @@ int mapObj_getSymbolByName(mapObj* self, char *name) {
 void mapObj_prepareQuery(mapObj* self) {
     int status;
 
-    status = msCalculateScale(self->extent, self->units, self->width, self->height, self->resolution, &self->scale);
-    if(status != MS_SUCCESS) self->scale = -1; // degenerate extents ok here
+    status = msCalculateScale(self->extent, self->units, self->width, self->height, self->resolution, &self->scaledenom);
+    if(status != MS_SUCCESS) self->scaledenom = -1; // degenerate extents ok here
   }
 
 imageObj *mapObj_prepareImage(mapObj* self) {
@@ -206,11 +109,11 @@ imageObj *mapObj_prepareImage(mapObj* self) {
   }
 
 imageObj *mapObj_draw(mapObj* self) {
-    return msDrawMap(self);
+    return msDrawMap(self, MS_FALSE);
   }
 
 imageObj *mapObj_drawQuery(mapObj* self) {
-    return msDrawQueryMap(self);
+    return msDrawMap(self, MS_TRUE);
   }
 
 imageObj *mapObj_drawLegend(mapObj* self) {
@@ -240,15 +143,6 @@ int mapObj_drawLabelCache(mapObj* self, imageObj *img) {
     return msDrawLabelCache(img, self);
   }
 
-labelCacheMemberObj *mapObj_nextLabel(mapObj* self) {
-    static int i=0;
-
-    if(i<self->labelcache.numlabels)
-      return &(self->labelcache.labels[i++]);
-    else
-      return NULL;	
-  }
-
 int mapObj_queryByPoint(mapObj* self, pointObj *point, 
                          int mode, double buffer) {
     return msQueryByPoint(self, -1, mode, *point, buffer);
@@ -444,19 +338,19 @@ int mapObj_OWSDispatch(mapObj *self, cgiRequestObj *req )
  * class extensions for layerObj, always within the context of a map
  **********************************************************************/
 layerObj *layerObj_new(mapObj *map) {
-    if(map->numlayers == MS_MAXLAYERS) // no room
+    if(msGrowMapLayers(map) == NULL)
       return(NULL);
 
-    if(initLayer(&(map->layers[map->numlayers]), map) == -1)
+    if(initLayer((map->layers[map->numlayers]), map) == -1)
       return(NULL);
 
-    map->layers[map->numlayers].index = map->numlayers;
-      //Update the layer order list with the layer's index.
+    map->layers[map->numlayers]->index = map->numlayers;
+    //Update the layer order list with the layer's index.
     map->layerorder[map->numlayers] = map->numlayers;
 
     map->numlayers++;
 
-    return &(map->layers[map->numlayers-1]);
+    return (map->layers[map->numlayers-1]);
   }
 
 void layerObj_destroy(layerObj *self) {
@@ -511,7 +405,7 @@ resultCacheMemberObj *layerObj_getResult(layerObj *self, int i) {
 
 classObj *layerObj_getClass(layerObj *self, int i) { // returns an EXISTING class
     if(i >= 0 && i < self->numclasses)
-      return &(self->class[i]); 
+      return (self->class[i]); 
     else
       return(NULL);
   }
@@ -682,22 +576,22 @@ classObj *layerObj_removeClass(layerObj *self, int index)
  * class extensions for classObj, always within the context of a layer
  **********************************************************************/
 classObj *classObj_new(layerObj *layer, classObj *class) {
-    if(layer->numclasses == MS_MAXCLASSES) // no room
+    if(msGrowLayerClasses(layer) == NULL)
       return NULL;
 
-    if(initClass(&(layer->class[layer->numclasses])) == -1)
+    if(initClass((layer->class[layer->numclasses])) == -1)
       return NULL;
 
     if (class){
-      msCopyClass(&(layer->class[layer->numclasses]), class, layer);
-      layer->class[layer->numclasses].layer = layer;
+      msCopyClass((layer->class[layer->numclasses]), class, layer);
+      layer->class[layer->numclasses]->layer = layer;
     }
 
-    layer->class[layer->numclasses].type = layer->type;
+    layer->class[layer->numclasses]->type = layer->type;
 
     layer->numclasses++;
 
-    return &(layer->class[layer->numclasses-1]);
+    return (layer->class[layer->numclasses-1]);
   }
 
 void  classObj_destroy(classObj *self) {
@@ -720,6 +614,10 @@ int classObj_setText(classObj *self, layerObj *layer, char *string) {
     return msLoadExpressionString(&self->text, string);
 }
 
+char *classObj_getTextString(classObj *self) {
+    return msGetExpressionString(&(self->text));
+}
+
 int classObj_drawLegendIcon(classObj *self, mapObj *map, layerObj *layer, int width, int height, gdImagePtr dstImg, int dstX, int dstY) {
   msClearLayerPenValues(layer); // just in case the mapfile has already been processed
     return msDrawLegendIcon(map, layer, self, width, height, dstImg, dstX, dstY);
@@ -1058,7 +956,22 @@ double shapeObj_getlength(shapeObj *self)
     return msGEOSLength(self);
 }
 
- 
+pointObj *shapeObj_getLabelPoint(shapeObj *self)
+{
+    pointObj *point = (pointObj *)calloc(1, sizeof(pointObj));
+    if (point == NULL) {
+        msSetError(MS_MEMERR, "Failed to allocate memory for point", "getLabelPoint()");
+        return NULL;
+    }
+
+    if(self->type == MS_SHAPE_POLYGON && msPolygonLabelPoint(self, point, -1) == MS_SUCCESS)
+        return point;
+
+    free(point);
+    return NULL;
+}
+
+
 /**********************************************************************
  * class extensions for rectObj
  **********************************************************************/
@@ -1250,18 +1163,18 @@ DBFFieldType DBFInfo_getFieldType(DBFInfo *self, int iField) {
  * class extensions for styleObj, always within the context of a class
  **********************************************************************/
 styleObj *styleObj_new(classObj *class, styleObj *style) {
-    if(class->numstyles == MS_MAXSTYLES) // no room
+    if(msGrowClassStyles(class) == NULL)
       return NULL;
 
-    if(initStyle(&(class->styles[class->numstyles])) == -1)
+    if(initStyle(class->styles[class->numstyles]) == -1)
       return NULL;
 
     if (style)
-      msCopyStyle(&(class->styles[class->numstyles]), style);
+      msCopyStyle(class->styles[class->numstyles], style);
         
     class->numstyles++;
 
-    return &(class->styles[class->numstyles-1]);
+    return class->styles[class->numstyles-1];
   }
 
 void  styleObj_destroy(styleObj *self) {
