@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: php_mapscript.h 5622 2006-08-22 15:55:34Z assefa $
+ * $Id: php_mapscript.h 6601 2007-08-16 15:34:18Z dmorissette $
  *
  * Name:     php_mapscript.c
  * Project:  PHP/MapScript extension for MapServer.
@@ -17,8 +17,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  * 
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  * 
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -27,94 +27,9 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
  * DEALINGS IN THE SOFTWARE.
- **********************************************************************
- *
- * $Log$
- * Revision 1.57  2006/08/22 15:55:03  assefa
- * Adding geos functions to php mapscript (Bug 1327)
- *
- * Revision 1.56  2006/05/29 19:02:01  assefa
- * Update PHP mapscript to support addition of MapScript WxS Services
- * (RFC 16, Bug 1790)
- *
- * Revision 1.55  2006/05/17 16:04:55  assefa
- * Add geos functions union, difference and intersection (Bug 1778)
- *
- * Revision 1.54  2006/01/20 15:03:35  assefa
- * Add containsshape function using uderlying GEOS function (Bug 1623).
- *
- * Revision 1.53  2005/10/13 16:00:36  assefa
- * Add support for whichshape and nextshape (Bug 1491)
- * Adsd support for config setting/getting at the map level (Bug 1487).
- *
- * Revision 1.52  2005/09/08 19:24:50  assefa
- * Expose GEOS operations through PHP Mapscript (Bug 1327).
- * Initially only functions buffer and convexhull on a shape object
- * are available.
- *
- * Revision 1.51  2005/06/14 16:03:36  dan
- * Updated copyright date to 2005
- *
- * Revision 1.50  2005/06/06 05:48:20  dan
- * Added $layerObj->removeClass() (was already in SWIG MapScript)
- * Added $layerObj->removeClass() to PHP MapScript (was already in SWIG
- * MapScript, bug 1373)
- *
- * Revision 1.49  2005/05/27 15:00:12  dan
- * New regex wrappers to solve issues with previous version (bug 1354)
- *
- * Revision 1.48  2005/05/25 21:02:06  dan
- * Added a regex wrapper, allowing MapServer to build with PHP compiled
- * with its builtin regex (bug 1354)
- *
- * Revision 1.47  2004/11/02 21:03:36  assefa
- * Add a 2nd optional argument to LoadMapContext function (Bug 1023).
- *
- * Revision 1.46  2004/07/28 22:03:50  dan
- * Added layer->getFilter() to PHP MapScript (bug 787)
- *
- * Revision 1.45  2004/07/26 14:12:37  dan
- * Created hashtable_i.c for new hashTableObj
- *
- * Revision 1.44  2004/05/31 15:35:39  dan
- * Added setRotation() (bug 702)
- *
- * Revision 1.43  2004/01/30 17:01:12  assefa
- * Add function deletestyle on a class object.
- *
- * Revision 1.42  2004/01/13 23:52:38  assefa
- * Add functions to move styles up and down.
- * Add function to clone style.
- *
- * Revision 1.41  2004/01/12 19:56:19  assefa
- * Add moveclassup and moveclassdown on a layer object.
- * Add clone function for the class object.
- * Add a 2nd optional argument for function ms_newclassobj to be able
- * to pass a class as argument.
- *
- * Revision 1.40  2004/01/05 21:27:14  assefa
- * applySLDURL and applySLD on a layer object can now take an optional
- * argument which is the name of the NamedLayer to use to style the layer.
- *
- * ...
- *
- * Revision 1.1  2000/05/09 21:06:11  dan
- * Initial Import
- *
- * Revision 1.4  2000/04/26 16:10:02  daniel
- * Changes to build with ms_3.3.010
- *
- * Revision 1.3  2000/03/15 00:42:24  daniel
- * Finished conversion of all cover functions to real C
- *
- * Revision 1.2  2000/02/07 05:19:38  daniel
- * Added layerObj and classObj
- *
- * Revision 1.1  2000/01/31 08:38:43  daniel
- * First working version - only mapObj implemented with read-only properties
- *
  **********************************************************************/
 
+
 #ifndef _PHP_MAPSCRIPT_H_INCLUDED_
 #define _PHP_MAPSCRIPT_H_INCLUDED_
 
@@ -132,7 +47,7 @@
 #include <unistd.h>
 #endif
 
-#include "map.h"
+#include "mapserver.h"
 #include "mapregex.h"
 #endif /* USE_PHP_REGEX */
 
@@ -262,6 +177,7 @@ void            classObj_destroy(classObj* self);
 int             classObj_setExpression(classObj *self, char *string);
 char            *classObj_getExpressionString(classObj *self);
 int             classObj_setText(classObj *self,layerObj *layer,char *string);
+char           *classObj_getTextString(classObj *self);
 int             classObj_drawLegendIcon(classObj *self,
                                         mapObj *map,
                                         layerObj *layer, 
@@ -336,6 +252,7 @@ int             shapeObj_disjoint(shapeObj *self, shapeObj *shape);
 pointObj        *shapeObj_getcentroid(shapeObj *self);
 double          shapeObj_getarea(shapeObj *self);
 double          shapeObj_getlength(shapeObj *self);
+pointObj        *shapeObj_getLabelPoint(shapeObj *self);
 
 rectObj        *rectObj_new();
 void            rectObj_destroy(rectObj *self);
