@@ -28,7 +28,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapsearch.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: mapsearch.c 7559 2008-05-05 14:46:11Z pramsey $")
 
 #define LASTVERT(v,n)  ((v) == 0 ? n-2 : v-1)
 #define NEXTVERT(v,n)  ((v) == n-2 ? 0 : v+1)
@@ -179,12 +179,15 @@ int msIntersectPointPolygon(pointObj *point, shapeObj *poly) {
   return(status);  
 }
 
-int msIntersectMultipointPolygon(multipointObj *points, shapeObj *poly) {
-  int i;
+int msIntersectMultipointPolygon(shapeObj *multipoint, shapeObj *poly) {
+  int i,j;
   
-  for(i=0; i<points->numpoints; i++) {
-    if(msIntersectPointPolygon(&(points->point[i]), poly) == MS_TRUE)
-      return(MS_TRUE);
+  for(i=0; i<multipoint->numlines; i++ ) {
+    lineObj points = multipoint->line[i];
+    for(j=0; j<points.numpoints; j++) {
+          if(msIntersectPointPolygon(&(points.point[j]), poly) == MS_TRUE)
+        return(MS_TRUE);
+    }
   }
     
   return(MS_FALSE);
