@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapserver.h 6885 2007-09-17 18:30:41Z dmorissette $
+ * $Id: mapserver.h 7646 2008-06-04 19:43:45Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  Primary MapServer include file.
@@ -30,7 +30,7 @@
 #ifndef MAP_H
 #define MAP_H
 
-/* $Id: mapserver.h 6885 2007-09-17 18:30:41Z dmorissette $
+/* $Id: mapserver.h 7646 2008-06-04 19:43:45Z dmorissette $
 **
 ** Main includes. If a particular header was needed by several .c files then
 ** I just put it here. What the hell, it works and it's all right here. -SDL-
@@ -118,7 +118,7 @@ extern "C" {
 
 /* General defines, wrapable */
 
-#define MS_VERSION "5.0.0"
+#define MS_VERSION "5.0.3"
 
 #define MS_TRUE 1 /* logical control variables */
 #define MS_FALSE 0
@@ -794,11 +794,14 @@ typedef struct {
 #ifdef SWIG
   %immutable;
 #endif /* SWIG */
-  int refcount;
   int numsymbols;
   int maxsymbols;
-  symbolObj** symbol;
+#ifdef SWIG
+%mutable;
+#endif /* SWIG */
 #ifndef SWIG
+  int refcount;
+  symbolObj** symbol;
   struct map_obj *map;
   fontSetObj *fontset; /* a pointer to the main mapObj version */
   struct imageCacheObj *imagecache;
@@ -1290,6 +1293,7 @@ int msBuildPluginLibraryPath(char **dest, const char *lib_str, mapObj *map);
 
 MS_DLL_EXPORT int  hex2int(char *hex);
 
+MS_DLL_EXPORT void initJoin(joinObj *join);
 MS_DLL_EXPORT void initSymbol(symbolObj *s);
 MS_DLL_EXPORT int initMap(mapObj *map);
 MS_DLL_EXPORT layerObj *msGrowMapLayers( mapObj *map );
@@ -1415,7 +1419,7 @@ MS_DLL_EXPORT double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, poin
 MS_DLL_EXPORT double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2);
 MS_DLL_EXPORT int msIntersectSegments(pointObj *a, pointObj *b, pointObj *c, pointObj *d);
 MS_DLL_EXPORT int msPointInPolygon(pointObj *p, lineObj *c);
-MS_DLL_EXPORT int msIntersectMultipointPolygon(multipointObj *points, shapeObj *polygon);
+MS_DLL_EXPORT int msIntersectMultipointPolygon(shapeObj *multipoint, shapeObj *polygon);
 MS_DLL_EXPORT int msIntersectPointPolygon(pointObj *p, shapeObj *polygon);
 MS_DLL_EXPORT int msIntersectPolylinePolygon(shapeObj *line, shapeObj *poly);
 MS_DLL_EXPORT int msIntersectPolygons(shapeObj *p1, shapeObj *p2);
