@@ -35,7 +35,7 @@
 #include <assert.h>
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapshape.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: mapshape.c 7568 2008-05-07 04:36:01Z pramsey $")
 
 
 #define ByteCopy( a, b, c )     memcpy( b, a, c )
@@ -1289,7 +1289,7 @@ int msSHPReadBounds( SHPHandle psSHP, int hEntity, rectObj *padBounds)
       return MS_FAILURE;
     } 
     
-    if( psSHP->nShapeType != SHP_POINT ) {
+    if( psSHP->nShapeType != SHP_POINT && psSHP->nShapeType != SHP_POINTZ && psSHP->nShapeType != SHP_POINTM) {
       fseek( psSHP->fpSHP, psSHP->panRecOffset[hEntity]+12, 0 );
       fread( padBounds, sizeof(double)*4, 1, psSHP->fpSHP );
 
@@ -1433,6 +1433,9 @@ int msSHPWhichShapes(shapefileObj *shpfile, rectObj rect, int debug)
   int i;
   rectObj shaperect;
   char *filename;
+  char *sourcename = 0; /* shape file source string from map file */
+  char *s = 0; /* pointer to start of '.shp' in source string */
+
 
   if(shpfile->status) {
     free(shpfile->status);
@@ -1454,14 +1457,22 @@ int msSHPWhichShapes(shapefileObj *shpfile, rectObj rect, int debug)
     for(i=0;i<shpfile->numshapes;i++) 
       msSetBit(shpfile->status, i, 1);
   } else {
-    if((filename = (char *)malloc(strlen(shpfile->source)+strlen(MS_INDEX_EXTENSION)+1)) == NULL) {
+
+    /* deal with case where sourcename is of the form 'file.shp' */
+    sourcename = strdup(shpfile->source);
+    /* TODO: need to handle this case insensitively */
+    s = strstr(sourcename, ".shp");
+    if( s ) *s = '\0';
+
+    if((filename = (char *)malloc(strlen(sourcename)+strlen(MS_INDEX_EXTENSION)+1)) == NULL) {
       msSetError(MS_MEMERR, NULL, "msSHPWhichShapes()");    
       return(MS_FAILURE);
     }
-    sprintf(filename, "%s%s", shpfile->source, MS_INDEX_EXTENSION);
+    sprintf(filename, "%s%s", sourcename, MS_INDEX_EXTENSION);
     
     shpfile->status = msSearchDiskTree(filename, rect, debug);
     free(filename);
+    free(sourcename);
 
     if(shpfile->status) /* index  */
       msFilterTreeSearch(shpfile, shpfile->status, rect);
