@@ -25,44 +25,19 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.35  2006/08/16 14:05:07  sdlime
- * Removed any ambiguity with msCommifyString(). At the moment it only handles North American representaions of numbers (e.g. 2,345.678).
- *
- * Revision 1.34  2006/01/31 17:09:28  sdlime
- * Added function to 'commify' a number stored as a string. (supports bug 1636)
- *
- * Revision 1.33  2005/06/14 16:03:34  dan
- * Updated copyright date to 2005
- *
- * Revision 1.32  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.31  2005/02/04 14:06:46  sdlime
- * Fixed bug 1207, thread safety issue in mapstring.c
- *
- * Revision 1.30  2004/11/19 19:28:13  hobu
- * check for null inputs in strncasecmp and strcasecmp
- *
- * Revision 1.29  2004/11/12 00:12:22  sdlime
- * Fixed bug 1040 so when escaping for HTML we use &#39; instead of &apos; for an apostrophe.
- *
- * Revision 1.28  2004/10/21 19:19:44  assefa
- * Add utility function trimLeft.
- *
- * Revision 1.27  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapstring.c 5599 2006-08-16 14:05:07Z sdlime $")
+MS_CVSID("$Id: mapstring.c 6662 2007-08-20 16:50:43Z hobu $")
 
 #include <ctype.h>
 
+#ifdef USE_ICONV
+#include <iconv.h>
+#include <wchar.h>
+#endif
+
 #ifdef NEED_STRLCAT
 /*
  * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
@@ -195,21 +170,21 @@ int strcasecmp(const char *s1, const char *s2)
 }
 #endif
 
-char *long2string(long value) {
+char *msLongToString(long value) {
   char buffer[256]; /* plenty of space */
 
   sprintf(buffer, "%ld", value);
   return(strdup(buffer));
 }
 
-char *double2string(double value) {
+char *msDoubleToString(double value) {
   char buffer[256]; /* plenty of space */
 
   sprintf(buffer, "%g", value);
   return(strdup(buffer));
 }
 
-char *chop(char *string) {  
+char *msStringChop(char *string) {  
   int n;
 
   n = strlen(string);
@@ -223,7 +198,7 @@ char *chop(char *string) {
  * remove leading white spaces and shif evey thing to the left
  */
 
-char *trimLeft(char *string)
+char *msStringTrimLeft(char *string)
 {
     char *read, *write;
     int i, length;
@@ -259,7 +234,7 @@ char *trimLeft(char *string)
 /* ------------------------------------------------------------------------------- */
 /*       Trims leading blanks from a string                                        */
 /* ------------------------------------------------------------------------------- */
-void trimBlanks(char *string)
+void msStringTrimBlanks(char *string)
 {
    int i,n;
 
@@ -276,7 +251,7 @@ void trimBlanks(char *string)
 /*       Trims end-of-line marker from a string                                    */
 /*       Usefull in conjunction with fgets() calls                                 */
 /* ------------------------------------------------------------------------------- */
-void trimEOL(char *string)
+void msStringTrimEOL(char *string)
 {
   int i;
 
@@ -292,7 +267,7 @@ void trimEOL(char *string)
 /*       Replace all occurances of old with new in str.                            */
 /*       It is assumed that str was dynamically created using malloc.              */
 /* ------------------------------------------------------------------------------- */
-char *gsub(char *str, const char *old, const char *new)
+char *msReplaceSubstring(char *str, const char *old, const char *new)
 {
       size_t str_len, old_len, new_len, tmp_offset;
       char *tmp_ptr;
@@ -353,7 +328,7 @@ char *gsub(char *str, const char *old, const char *new)
 /*
 ** how many times does ch occur in str
 */
-int countChars(char *str, char ch) 
+int msCountChars(char *str, char ch) 
 {
   int i, l, n=0;
 
@@ -367,7 +342,7 @@ int countChars(char *str, char ch)
 /* ------------------------------------------------------------------------------- */
 /*       Strip filename from a full path                                           */
 /* ------------------------------------------------------------------------------- */
-char *stripPath(char *fn)
+char *msStripPath(char *fn)
 {
   char *str;
 
@@ -381,7 +356,7 @@ char *stripPath(char *fn)
 /*
 ** Returns the *path* portion of the filename fn. Memory is allocated using malloc.
 */
-char *getPath(char *fn)
+char *msGetPath(char *fn)
 {
   char *str;
   int i, length;
@@ -529,7 +504,7 @@ char *msTryBuildPath3(char *szReturnPath, const char *abs_path, const char *path
 /*
 ** Splits a string into multiple strings based on ch. Consecutive ch's are ignored.
 */
-char **split(const char *string, char ch, int *num_tokens) 
+char **msStringSplit(const char *string, char ch, int *num_tokens) 
 {
   int i,j,k;
   int length,n;
@@ -798,7 +773,7 @@ int msIsXMLTagValid(const char *string)
 /*
  * Concatenate pszSrc to pszDest and reallocate memory if necessary.
 */
-char *strcatalloc(char *pszDest, char *pszSrc)
+char *msStringConcatenate(char *pszDest, char *pszSrc)
 {
    int nLen;
    
@@ -821,7 +796,7 @@ char *strcatalloc(char *pszDest, char *pszSrc)
          pszDest[nLen] = '\0';
       }
       else {
-         msSetError(MS_MEMERR, "Error while reallocating memory.", "strcatalloc()");
+         msSetError(MS_MEMERR, "Error while reallocating memory.", "msStringConcatenate()");
          return NULL;
       }        
    }
@@ -895,7 +870,7 @@ char *msCommifyString(char *str)
 
   if(!str) return NULL;
 
-  num_decimal_points = countChars(str, decimal_point);
+  num_decimal_points = msCountChars(str, decimal_point);
   if(num_decimal_points > 1) return str;
 
   old_length = strlen(str);
@@ -933,3 +908,333 @@ char *msCommifyString(char *str)
 
   return str;
 }
+
+
+/************************************************************************/
+/*                  case incensitive equivalent of strstr               */
+/************************************************************************/
+const char *msCaseFindSubstring(const char *haystack, const char *needle)
+{   
+    if ( !*needle )   
+    {      
+        return haystack;   
+    }   
+    for ( ; *haystack; ++haystack )   
+    {      
+        if ( toupper(*haystack) == toupper(*needle) )      
+        {         
+            /*          * Matched starting char -- loop through remaining chars.          */
+            const char *h, *n;         
+            for ( h = haystack, n = needle; *h && *n; ++h, ++n )         
+            {            
+                if ( toupper(*h) != toupper(*n) )            
+                {
+                    break;            
+                }         
+            }         
+            if ( !*n ) /* matched all of 'needle' to null termination */         
+            {            
+                return haystack; /* return the start of the match */         
+            }      
+        }   
+    }   
+    return 0;
+}
+
+
+
+/* ------------------------------------------------------------------------------- */
+/*       Replace all occurances of old with new in str.                            */
+/*       It is assumed that str was dynamically created using malloc.              */
+/*       Same function as msReplaceSubstring but this is case incensitive                        */
+/* ------------------------------------------------------------------------------- */
+char *msCaseReplaceSubstring(char *str, const char *old, const char *new)
+{
+      size_t str_len, old_len, new_len, tmp_offset;
+      char *tmp_ptr;
+
+      if(new == NULL)
+          new = "";
+
+      /*
+      ** If old is not found then leave str alone
+      */
+      if( (tmp_ptr = (char *) msCaseFindSubstring(str, old)) == NULL)
+	return(str);
+
+      /*
+      ** Grab some info about incoming strings
+      */
+      str_len = strlen(str);
+      old_len = strlen(old);
+      new_len = strlen(new);
+
+      /*
+      ** Now loop until old is NOT found in new
+      */
+      while( tmp_ptr != NULL ) {
+
+	/*
+	** re-allocate memory for buf assuming 1 replacement of old with new
+        ** don't bother reallocating if old is larger than new)
+	*/
+        if (old_len < new_len) {
+          tmp_offset = tmp_ptr - str;
+          str_len = str_len - old_len + new_len;
+          str = (char *)realloc(str, (str_len + 1)); /* make new space for a copy */
+          tmp_ptr = str + tmp_offset;
+        }
+
+        /*
+        ** Move the trailing part of str to make some room unless old_len == new_len
+        */
+        if (old_len != new_len) {
+            memmove(tmp_ptr+new_len, tmp_ptr+old_len, strlen(tmp_ptr)-old_len+1);
+        }
+
+        /*
+        ** Now copy new over old
+        */
+        memcpy(tmp_ptr, new, new_len);
+
+        /*
+        ** And look for more matches in the rest of the string
+        */
+        tmp_ptr = (char *) msCaseFindSubstring(tmp_ptr + new_len, old);
+      }
+
+      return(str);
+}
+
+/* 
+** Converts a 2 character hexidecimal string to an integer.
+*/
+int msHexToInt(char *hex) {
+  int number;
+
+  number = (hex[0] >= 'A' ? ((hex[0] & 0xdf) - 'A')+10 : (hex[0] - '0'));
+  number *= 16;
+  number += (hex[1] >= 'A' ? ((hex[1] & 0xdf) - 'A')+10 : (hex[1] - '0'));
+   
+  return(number);
+}
+
+
+/*
+** Simple charset converter. Converts string from specified encoding to UTF-8.
+** The return value must be freed by the caller.
+*/
+char *msGetEncodedString(const char *string, const char *encoding)
+{
+#ifdef USE_ICONV
+  iconv_t cd = NULL;
+  const char *inp;
+  char *outp, *out = NULL;
+  size_t len, bufsize, bufleft, status;
+
+  len = strlen(string);
+
+  if (len == 0 || (encoding && strcasecmp(encoding, "UTF-8")==0))
+      return strdup(string);    /* Nothing to do: string already in UTF-8 */
+
+  cd = iconv_open("UTF-8", encoding);
+  if(cd == (iconv_t)-1) {
+    msSetError(MS_IDENTERR, "Encoding not supported by libiconv (%s).", 
+               "msGetEncodedString()", encoding);
+    return NULL;
+  }
+
+  bufsize = len * 6 + 1; /* Each UTF-8 char can be up to 6 bytes */
+  inp = string;
+  out = (char*) malloc(bufsize);
+  if(out == NULL){
+    msSetError(MS_MEMERR, NULL, "msGetEncodedString()");
+    iconv_close(cd);
+    return NULL;
+  }
+  strcpy(out, string);
+  outp = out;
+
+  bufleft = bufsize;
+  status = -1;
+
+  while (len > 0){
+    status = iconv(cd, (char**)&inp, &len, &outp, &bufleft);
+    if(status == -1){
+      msFree(out);
+      iconv_close(cd);
+      return strdup(string);
+    }
+  }
+  out[bufsize - bufleft] = '\0';
+  
+  iconv_close(cd);
+
+  return out;
+#else
+  msSetError(MS_MISCERR, "Not implemeted since Iconv is not enabled.", "msGetEncodedString()");
+  return NULL;
+#endif
+}
+
+
+char* msConvertWideStringToUTF8 (const wchar_t* string, const char* encoding) {
+#ifdef USE_ICONV
+
+    int bconvFailed = MS_TRUE;
+    char* output = NULL;
+    iconv_t cd = NULL;
+    size_t nStr;
+    size_t nInSize;
+    size_t nOutSize;
+    size_t nConv;
+    size_t nBufferSize;
+
+    char* pszUTF8 = NULL;
+    const wchar_t* pwszWide = NULL;
+
+    if (string != NULL)
+    {   
+        nStr = wcslen (string);
+        nBufferSize = ((nStr * 6) + 1);
+        output = (char*) malloc (nBufferSize);
+        if (output == NULL) {
+            msSetError(MS_MEMERR, NULL, "msGetEncodedString()");
+            return NULL;
+        }
+        if (nStr == 0) {
+            /* return an empty 8 byte string */
+            output[0] = '\0';
+            return output;
+        }
+
+        cd = iconv_open("UTF-8", encoding);
+        
+        nOutSize = nBufferSize;
+        if ((iconv_t)-1 != cd)
+        {  
+           nInSize = sizeof (wchar_t)*nStr;
+           pszUTF8 = output;
+           pwszWide = string;
+           nConv = iconv(cd, (char **)&pwszWide, &nInSize, &pszUTF8, &nOutSize);
+           if ((size_t)-1 != nConv &&  nOutSize != nBufferSize)
+               bconvFailed = MS_FALSE;
+           iconv_close(cd);
+        } else {
+            msSetError(MS_MISCERR, "Encoding not supported by libiconv (%s).", 
+                                   "msConvertWideStringToUTF8()", 
+                                   encoding);
+            return NULL;
+        }
+   
+        if (bconvFailed) {
+            msFree(output);
+            output = NULL;
+            msSetError(MS_MISCERR, "Unable to convert string in encoding '%s' to UTF8",
+                                   "msConvertWideStringToUTF8()",
+                                   encoding);
+        }
+    } else {
+        /* we were given a NULL wide string, nothing we can do here */
+        return NULL;
+    }
+    
+    /* NULL-terminate the output string */
+    output[nBufferSize - nOutSize] = '\0';
+    return output;
+#else
+  msSetError(MS_MISCERR, "Not implemented since Iconv is not enabled.", "msConvertWideStringToUTF8()");
+  return NULL;
+#endif
+}
+
+/*
+** Returns the next UTF-8 char in string and advances *in_ptr to the next
+** character.
+**
+** If out_string is not NULL then the character (bytes) is copied to this 
+** buffer and null-terminated. out_string must be a pre-allocated buffer of
+** at least 7 bytes.
+**
+** The function returns the number of bytes in this UTF-8 character.
+**
+** In UTF-8, the number of leading 1 bits in the first byte specifies the 
+** number of bytes in the entire sequence.
+** Source: http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+**
+** U-00000000 U-0000007F: 0xxxxxxx
+** U-00000080 U-000007FF: 110xxxxx 10xxxxxx
+** U-00000800 U-0000FFFF: 1110xxxx 10xxxxxx 10xxxxxx
+** U-00010000 U-001FFFFF: 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
+** U-00200000 U-03FFFFFF: 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
+** U-04000000 U-7FFFFFFF: 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
+*/
+int msGetNextUTF8Char(const char **in_ptr, char *out_string)
+{
+    unsigned char in;
+    int numbytes=0;
+    int i;
+
+    in = (unsigned char)**in_ptr;
+
+    if (in == 0)
+        return -1;  /* Empty string */
+
+    if (in < 0x80)
+    {
+        numbytes = 1;   /* 0xxxxxxx */
+    }
+    else if ((in & 0xC0) == 0x80)
+    {
+        /* 10xxxxxx - this is not the first byte of a UTF-8 char, we need to 
+         * resynchronize . For now just return -1.
+         */
+        numbytes = 1;
+        return -1;
+    }
+    else if ((in & 0xE0) == 0xC0)
+    {
+        numbytes = 2;   /* 110xxxxx 10xxxxxx */
+    }
+    else if ((in & 0xF0) == 0xE0)
+    {
+        numbytes = 3;   /* 1110xxxx 10xxxxxx 10xxxxxx */
+    }
+    else if ((in & 0xF8) == 0xF0)
+    {
+        numbytes = 4;   /* 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx */
+    }
+    else if ((in & 0xFC) == 0xF8)
+    {
+        numbytes = 5;   /* 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx */
+    }
+    else if ((in & 0xFE) == 0xFC)
+    {
+        numbytes = 6;   /* 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx */
+    }
+
+    /* Advance input buffer and copy the bytes to out_string if one is set */
+    for(i=0; i<numbytes && **in_ptr != '\0'; i++)
+    {
+        if (out_string)
+            out_string[i] = **in_ptr;
+        (*in_ptr)++;
+    }
+    if (out_string)
+        out_string[i] = '\0';
+
+    return numbytes;
+}
+
+/*
+** Returns the number of UTF-8 chars in string
+*/
+int msGetNumUTF8Chars(const char *in_ptr)
+{
+    int numchars=0;
+
+    while( msGetNextUTF8Char(&in_ptr, NULL) != -1 )
+        numchars++;
+
+    return numchars;
+}
