@@ -34,7 +34,7 @@
 #include "mapcopy.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapsymbol.c 6789 2007-09-05 14:04:06Z sdlime $")
+MS_CVSID("$Id: mapsymbol.c 7017 2007-11-08 15:19:29Z Assefa $")
 
 extern int msyylex(void); /* lexer globals */
 extern void msyyrestart(FILE *);
@@ -216,7 +216,7 @@ int loadSymbol(symbolObj *s, char *symbolpath)
       break;
     case(IMAGE):
       if(msyylex() != MS_STRING) { /* get image location from next token */
-	msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()", msyylineno);
+	msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()", msyytext, msyylineno);
 	return(-1);
       }
       
@@ -977,9 +977,9 @@ int msCopySymbol(symbolObj *dst, symbolObj *src, mapObj *map) {
     
     if (gdImageTrueColor(src->img)) {
       dst->img = gdImageCreateTrueColor(gdImageSX(src->img), gdImageSY(src->img));
-      gdImageCopy(dst->img, src->img, 0, 0, 0, 0, gdImageSX(src->img), gdImageSY(src->img));
-      gdImageAlphaBlending(dst->img, 0);
       gdImageFilledRectangle(dst->img, 0, 0, gdImageSX(src->img), gdImageSY(src->img), gdImageColorAllocateAlpha(dst->img, 0, 0, 0, gdAlphaTransparent)); 
+      gdImageAlphaBlending(dst->img, 0);
+      gdImageCopy(dst->img, src->img, 0, 0, 0, 0, gdImageSX(src->img), gdImageSY(src->img));
     } else {
       int tc = gdImageGetTransparent(src->img);
 
