@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsymbol.h 5060 2005-12-15 05:42:05Z sdlime $
+ * $Id: mapsymbol.h 6428 2007-07-31 19:58:45Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  symbolObj related declarations.
@@ -25,32 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.23  2005/12/15 05:42:05  sdlime
- * Fixed a problem in the symbol reader that basically turned TYPE SIMPLE into TYPE TRUETYPE. Fix what to start the symbol type enum at some larger value (e.g. 1000).
- *
- * Revision 1.22  2005/11/24 04:35:50  dan
- * Use dynamic allocation for ellipse symbol's STYLE array, avoiding the
- * static limitation on the STYLE argument values. (bug 1539)
- *
- * Revision 1.21  2005/10/14 05:05:19  sdlime
- * Added MS_MAXPATTERNSIZE.
- *
- * Revision 1.20  2005/09/23 03:47:56  sdlime
- * Made symbol->imagepath readonly within SWIG-based scripts. Use setImagepath instead. (bug 1472)
- *
- * Revision 1.19  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.18  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.17  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
 #ifndef MAPSYMBOL_H
 #define MAPSYMBOL_H
@@ -59,9 +34,9 @@
 
 enum MS_SYMBOL_TYPE {MS_SYMBOL_SIMPLE=1000, MS_SYMBOL_VECTOR, MS_SYMBOL_ELLIPSE, MS_SYMBOL_PIXMAP, MS_SYMBOL_TRUETYPE, MS_SYMBOL_CARTOLINE, MS_SYMBOL_HATCH};
 
-#define MS_MAXSYMBOLS 64            /* maximum number of symbols in a symbol file */
+#define MS_SYMBOL_ALLOCSIZE 64      /* number of symbolObj ptrs to allocate for a symbolset at once */
 #define MS_MAXVECTORPOINTS 100      /* shade, marker and line symbol parameters */
-#define MS_MAXSTYLELENGTH 10
+#define MS_MAXPATTERNLENGTH 10
 
 #define MS_IMAGECACHESIZE 6
 
@@ -112,14 +87,15 @@ typedef struct {
 #ifdef SWIG
   %immutable;
 #endif /* SWIG */
+  int refcount;
   int numpoints;
 #ifdef SWIG
   %mutable;
 #endif /* SWIG */
   int filled;
 
-  int stylelength;                      /* Number of intervals (eg. dashes) in the style */
-  int style[MS_MAXSTYLELENGTH];
+  int patternlength;                      /* Number of intervals (eg. dashes) in the pattern (was style, see bug 2119) */
+  int pattern[MS_MAXPATTERNLENGTH];
 
   /*
   ** MS_SYMBOL_PIXMAP options
