@@ -24,42 +24,18 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.19  2006/03/14 03:43:30  assefa
- * Move msValidateTimeValue to maptime so it can be used by WMS and SOS.
- * (Bug 1710)
- *
- * Revision 1.18  2005/09/26 14:51:40  assefa
- * Correct typo in  ms_timeFormats (Bug 1478).
- *
- * Revision 1.17  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.16  2005/05/27 15:00:12  dan
- * New regex wrappers to solve issues with previous version (bug 1354)
- *
- * Revision 1.15  2005/02/18 03:06:47  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.14  2004/10/21 10:54:17  assefa
- * Add postgis date_trunc support.
- *
- * Revision 1.13  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
+#define _GNU_SOURCE  /* glibc2 needs this for strptime() */
 #include <stdlib.h>
 #include <stdio.h>
 #include <time.h>
 
-#include "map.h"
+#include "mapserver.h"
 #include "maptime.h"
 #include "maperror.h"
 
-MS_CVSID("$Id: maptime.c 5258 2006-03-14 03:43:30Z assefa $")
+MS_CVSID("$Id: maptime.c 6756 2007-08-30 22:46:11Z Assefa $")
 
 typedef struct {
   char pattern[64];
@@ -165,13 +141,7 @@ char *strptime( const char *buf, const char *format, struct tm *timeptr );
 
 char *msStrptime(const char *s, const char *format, struct tm *tm)
 {
-#if defined(_WIN32) && !defined(__CYGWIN__)
-    /* we are now using a local strptime found strptime.c */
     return strptime(s, format, tm);
-#else
-    /* Use system strptime() on non-windows systems */
-    return strptime(s, format, tm);
-#endif
 }
 
 /**
@@ -227,7 +197,7 @@ void msSetLimitedPattersToUse(char *patternstring)
 
     if (patternstring)
     {
-        patterns = split(patternstring, ',', &ntmp);
+        patterns = msStringSplit(patternstring, ',', &ntmp);
         if (patterns && ntmp >= 1)
         {
             
@@ -351,7 +321,7 @@ int _msValidateTime(char *timestring,  const char *timeextent)
        or a range */
     
     numelements = 0;
-    atimeelements = split (timestring, '/', &numelements);
+    atimeelements = msStringSplit (timestring, '/', &numelements);
     msTimeInit(&tmtimestart);
     msTimeInit(&tmtimeend);
 
@@ -394,7 +364,7 @@ int _msValidateTime(char *timestring,  const char *timeextent)
     */
     
     numextents = 0;
-    atimeextents = split (timeextent, ',', &numextents);
+    atimeextents = msStringSplit (timeextent, ',', &numextents);
     if (atimeextents == NULL || numextents <= 0)
       return MS_FALSE;
     
@@ -408,7 +378,7 @@ int _msValidateTime(char *timestring,  const char *timeextent)
         msTimeInit(&tmend);
         
         numranges = 0;
-        atimerange = split (atimeextents[i], '/', &numranges);
+        atimerange = msStringSplit (atimeextents[i], '/', &numranges);
         /* - one value 2004-09-21 */
         if (numranges == 1)
         {
@@ -448,8 +418,8 @@ int _msValidateTime(char *timestring,  const char *timeextent)
 
 int msValidateTimeValue(char *timestring, const char *timeextent)
 {
-    char **atimes, **tokens =  NULL;
-    int i, numtimes, ntmp = 0;
+    char **atimes =  NULL;
+    int i, numtimes=0;
 
     /* we need to validate the time passsed in the request */
     /* against the time extent defined */
@@ -469,13 +439,12 @@ int msValidateTimeValue(char *timestring, const char *timeextent)
     }
     else
     {
-        atimes = split (timestring, ',', &numtimes);
+        atimes = msStringSplit (timestring, ',', &numtimes);
         if (numtimes >=1) /* multiple times */
         {
-            tokens = split(atimes[0],  '/', &ntmp);
-            if (ntmp == 1) /* multiple descrete times */
+
+            if (strstr(atimes[0], "/") == NULL) /* multiple descrete times */
             {
-              /*msFreeCharArray(tokens, ntmp);*/
                 for (i=0; i<numtimes; i++)
                 {
                     if (_msValidateTime(atimes[i], timeextent) == MS_FALSE)
@@ -487,11 +456,10 @@ int msValidateTimeValue(char *timestring, const char *timeextent)
                 msFreeCharArray(atimes, numtimes);
                 return MS_TRUE;
             }
-            else if (ntmp >= 2)/* multiple ranges */
+            else /* multiple ranges */
             {
                 for (i=0; i<numtimes; i++)
                 {
-                  /*msFreeCharArray(tokens, ntmp);*/
                     if (_msValidateTime(atimes[i], timeextent) == MS_FALSE)
                     {
                         msFreeCharArray(atimes, numtimes);
