@@ -30,7 +30,7 @@
 #include "maperror.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapwcs.c 6637 2007-08-17 21:30:26Z warmerdam $")
+MS_CVSID("$Id: mapwcs.c 7285 2008-01-21 17:09:01Z tomkralidis $")
 
 #ifdef USE_WCS_SVR
 
@@ -594,7 +594,7 @@ static int msWCSGetCapabilities_CoverageOfferingBrief(layerObj *layer, wcsParams
   msOWSPrintEncodeMetadata(stdout, &(layer->metadata), "COM", "label", OWS_WARN, "  <label>%s</label>\n", NULL);
 
   /* TODO: add elevation and temporal ranges to lonLatEnvelope (optional) */
-  msIO_printf("    <lonLatEnvelope srsName=\"WGS84(DD)\">\n");
+  msIO_printf("    <lonLatEnvelope srsName=\"EPSG:4326\">\n");
   msIO_printf("      <gml:pos>%.15g %.15g</gml:pos>\n", cm.llextent.minx, cm.llextent.miny); /* TODO: don't know if this is right */
   msIO_printf("      <gml:pos>%.15g %.15g</gml:pos>\n", cm.llextent.maxx, cm.llextent.maxy);
   msIO_printf("    </lonLatEnvelope>\n");
@@ -793,7 +793,7 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
   msOWSPrintEncodeMetadata(stdout, &(layer->metadata), "COM", "label", OWS_WARN, "  <label>%s</label>\n", NULL);
 
   /* TODO: add elevation and temporal ranges to lonLatEnvelope (optional) */
-  msIO_printf("    <lonLatEnvelope srsName=\"WGS84(DD)\">\n");
+  msIO_printf("    <lonLatEnvelope srsName=\"EPSG:4326\">\n");
   msIO_printf("      <gml:pos>%.15g %.15g</gml:pos>\n", cm.llextent.minx, cm.llextent.miny);
   msIO_printf("      <gml:pos>%.15g %.15g</gml:pos>\n", cm.llextent.maxx, cm.llextent.maxy);
   msIO_printf("    </lonLatEnvelope>\n");
@@ -808,7 +808,7 @@ static int msWCSDescribeCoverage_CoverageOffering(layerObj *layer, wcsParamsObj
   msIO_printf("      <spatialDomain>\n");
   
   /* envelope in lat/lon */
-  msIO_printf("        <gml:Envelope srsName=\"WGS84(DD)\">\n");
+  msIO_printf("        <gml:Envelope srsName=\"EPSG:4326\">\n");
   msIO_printf("          <gml:pos>%.15g %.15g</gml:pos>\n", cm.llextent.minx, cm.llextent.miny);
   msIO_printf("          <gml:pos>%.15g %.15g</gml:pos>\n", cm.llextent.maxx, cm.llextent.maxy);
   msIO_printf("        </gml:Envelope>\n");
