@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapwfs.c 5749 2006-09-06 17:26:24Z sdlime $
+ * $Id: mapwfs.c 6579 2007-08-14 14:36:38Z sdlime $
  *
  * Name:     mapwfs.c
  * Project:  MapServer
@@ -17,8 +17,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  * 
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  * 
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -26,101 +26,11 @@
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
- * DEALINGS IN THE SOFTWARE.
- **********************************************************************
- * $Log$
- * Revision 1.87  2006/09/06 17:26:24  sdlime
- * Removed advertising GML3 support in WFS 1.0.0 getCapabilities output.
- *
- * Revision 1.86  2006/08/22 04:45:06  sdlime
- * Fixed a bug that did not allow for seperate metadata namespaces to be used for WMS vs. WFS for GML transformations (e.g. WFS_GEOMETRIES, WMS_GEOMETRIES).
- *
- * Revision 1.85  2006/08/14 21:52:20  sdlime
- * Updated WFS GetFeature to reference external namespaces if available.
- *
- * Revision 1.84  2006/08/14 21:15:00  sdlime
- * Added support to WFS  DescribeFeatureType for external schemas (initial coding).
- *
- * Revision 1.83  2006/08/14 20:19:54  dan
- * Produce a warning in GetCapabilities if ???_featureid not set (bug 1782)
- *
- * Revision 1.82  2006/08/14 18:47:28  sdlime
- * Some more implementation of external application schema support.
- *
- * Revision 1.81  2006/08/11 21:44:48  sdlime
- * Added namespace reading (using is the hard part) to WFS server.
- *
- * Revision 1.80  2006/04/08 05:58:48  frank
- * fix various memory leaks
- *
- * Revision 1.79  2006/04/08 03:38:18  frank
- * Ensure that an error in FLTApplyFilterToLayer() is reported as a
- * WFS exception.
- *
- * Revision 1.78  2006/03/30 21:43:47  sdlime
- * Upgraded GML 3 version from 3.1.0 to 3.1.1 based on discussions with Tom Kralidis.
- *
- * Revision 1.77  2006/03/22 21:40:47  sdlime
- * Added support to allow a service provider (WFS or WMS) tonot expose feature geometries. (bug 1718)
- *
- * Revision 1.76  2006/03/15 22:54:07  sdlime
- * Fixed my brain-dead gml_group schema writing support.
- *
- * Revision 1.75  2006/02/24 02:55:03  assefa
- * Add the possiblity to set wfs_maxfeatures to 0 (Bug 1678)
- *
- * Revision 1.74  2006/02/17 22:59:47  sdlime
- * Updated WFS schema production code to ignore items with templates defined since we can't readily produce schema elements for them.
- *
- * Revision 1.73  2005/12/27 17:36:27  sdlime
- * Fixed a typo in msWFSGetGeometryType().
- *
- * Revision 1.72  2005/12/06 16:42:25  assefa
- * WFS : TYPENAME is manadatory for GetFeature request (Bug 1554).
- *
- * Revision 1.71  2005/10/25 20:29:52  sdlime
- * Completed work to add constants to GML output. For example gml_constants 'aConstant'  gml_aConstant_value 'this is a constant', which results in output like <aConstant>this is a constant</aConstant>. Constants can appear in groups and can havespecific types (default is string). Constants are NOT queryable so their use should be limited untilsome extensions to wfs 1.1 appear that will allow us to mark certain elements as queryable or not in capabilities output.
- *
- * Revision 1.70  2005/10/12 15:34:27  sdlime
- * Updated the gmlGroupObj to allow you to set the group type. This impacts schema location. If not set the complex type written by the schema generator is 'groupnameType' and via metadata you can override that (e.g. gml_groupname_type 'MynameType').
- *
- * Revision 1.69  2005/10/11 17:48:35  sdlime
- * Fixed complex type writer to create set the type name when writing a schema. I think we may need more control though (e.g. group types). Also trimmed the revision history to a cover only the more recent mods.
- *
- * Revision 1.68  2005/09/08 20:57:50  assefa
- * Changed default namespace used (Bug 1461).
- *
- * Revision 1.67  2005/06/02 20:42:14  sdlime
- * Fixed a small initialization problem when no namespace metadata supplied.
- *
- * Revision 1.66  2005/06/02 20:32:01  sdlime
- * Changed metadata reference from ...getfeature_collection to ...feature_collection.
- *
- * Revision 1.65  2005/06/02 20:25:04  sdlime
- * Updated WFS output to not use wfs:FeatureCollection as the main container for GML3 output. A default container of msFeatureCollection is provided or the user may define one explicitly.
- *
- * Revision 1.64  2005/05/31 18:24:49  sdlime
- * Updated GML3 writer to use the new gmlGeometryListObj. This allows you to package geometries from WFS in a pretty flexible manner. Will port GML2 writer once testing on GML3 code is complete.
- *
- * Revision 1.63  2005/05/27 17:24:56  sdlime
- * Changed WFS layer type name from [layer name]_Type to [layer name]Type in keeping with schema convension.
- *
- * Revision 1.62  2005/05/26 16:09:15  sdlime
- * Updated mapwfs.c to produce schema compliant with the GML for Simple Feature Exchange proposed standard. Changes are relatively minor with the exception of naming the geometry container and handling of mixed geometry types. The previous version defaulted to a GML type. We need more control for application specific schema. We now package the GML geometry in an element named by default geometry, users can override using gml_geometry_name metadata. We also advertise a *very* generic GMLPropertyType by default again which can be overriden using gml_geometry_type. That metadata *can* contain a list of valid types which are offered as a xsd:choice.
- *
- * Revision 1.61  2005/04/22 15:50:46  assefa
- * Bug 1262 : the SERVICE parameter is now required for wms and wfs
- * GetCapbilities request. It is not required for other WMS requests.
- * It is required for all WFS requests.
- *
- * Revision 1.60  2005/04/21 21:10:38  sdlime
- * Adjusted WFS support to allow for a new output format (GML3).
- *
- **********************************************************************/
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: mapwfs.c 5749 2006-09-06 17:26:24Z sdlime $")
+MS_CVSID("$Id: mapwfs.c 6579 2007-08-14 14:36:38Z sdlime $")
 
 #if defined(USE_WFS_SVR)
 
@@ -156,6 +66,9 @@ static int msWFSException(mapObj *map, const char *wmtversion)
     msIO_printf("  </ServiceException>\n");
     msIO_printf("</ServiceExceptionReport>\n");
 
+    /* clear error since we have already reported it */
+    msResetErrorList();
+
     return MS_FAILURE; /* so we can call 'return msWFSException();' anywhere */
 }
 
@@ -475,7 +388,7 @@ int msWFSGetCapabilities(mapObj *map, const char *wmtver, cgiRequestObj *req)
   for(i=0; i<map->numlayers; i++)
   {
       layerObj *lp;
-      lp = &(map->layers[i]);
+      lp = GET_LAYER(map, i);
 
       /* List only vector layers in which DUMP=TRUE */
       if (msWFSIsLayerSupported(lp))
@@ -494,8 +407,17 @@ int msWFSGetCapabilities(mapObj *map, const char *wmtver, cgiRequestObj *req)
   msIO_printf("<ogc:Filter_Capabilities>\n");
   msIO_printf("  <ogc:Spatial_Capabilities>\n");
   msIO_printf("    <ogc:Spatial_Operators>\n");
+#ifdef USE_GEOS
+  msIO_printf("      <ogc:Equals/>\n");
+  msIO_printf("      <ogc:Disjoint/>\n");
+  msIO_printf("      <ogc:Touches/>\n");
+  msIO_printf("      <ogc:Within/>\n");
+  msIO_printf("      <ogc:Overlaps/>\n");
+  msIO_printf("      <ogc:Crosses/>\n");
   msIO_printf("      <ogc:Intersect/>\n");
+  msIO_printf("      <ogc:Contains/>\n");
   msIO_printf("      <ogc:DWithin/>\n");
+#endif
   msIO_printf("      <ogc:BBOX/>\n");
   msIO_printf("    </ogc:Spatial_Operators>\n");
   msIO_printf("  </ogc:Spatial_Capabilities>\n");
@@ -729,22 +651,26 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
     /*  */
     /* __TODO__ Need to handle type grouping, e.g. "(l1,l2),l3,l4" */
     /*  */
-    layers = split(paramsObj->pszTypeName, ',', &numlayers);
+    layers = msStringSplit(paramsObj->pszTypeName, ',', &numlayers);
     if (numlayers > 0) {
       /* strip namespace if there is one :ex TYPENAME=cdf:Other */
-      tokens = split(layers[0], ':', &n);
+      tokens = msStringSplit(layers[0], ':', &n);
       if (tokens && n==2 && msGetLayerIndex(map, layers[0]) < 0) {
         msFreeCharArray(tokens, n);
+        tokens = NULL;
         for (i=0; i<numlayers; i++) {
-            tokens = split(layers[i], ':', &n);
+            tokens = msStringSplit(layers[i], ':', &n);
             if (tokens && n==2) {
                 free(layers[i]);
                 layers[i] = strdup(tokens[1]);
             }
             if (tokens)
                 msFreeCharArray(tokens, n);
+            tokens = NULL;
         }
       }
+      if (tokens)
+          msFreeCharArray(tokens, n);
     }
   } 
 
@@ -772,7 +698,7 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
   /*
   ** retrieve any necessary external namespace/schema configuration information
   */
-  namespaceList = msGMLGetNamespaces(&(map->web), "OFG");
+  namespaceList = msGMLGetNamespaces(&(map->web), "G");
 
   /*
   ** DescribeFeatureType response
@@ -858,7 +784,7 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
     layerObj *lp;
     int j, bFound = 0;
     
-    lp = &(map->layers[i]);
+    lp = GET_LAYER(map, i);
 
     for (j=0; j<numlayers && !bFound; j++) {
       if ( lp->name && strcasecmp(lp->name, layers[j]) == 0)
@@ -883,10 +809,10 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
           const char *layer_namespace_prefix;
           char *encoded_type=NULL;
 
-          itemList = msGMLGetItems(lp, "OFG"); /* GML-related metadata */
-          constantList = msGMLGetConstants(lp, "OFG");
-          groupList = msGMLGetGroups(lp, "OFG");
-          geometryList = msGMLGetGeometries(lp, "OFG");
+          itemList = msGMLGetItems(lp, "G"); /* GML-related metadata */
+          constantList = msGMLGetConstants(lp, "G");
+          groupList = msGMLGetGroups(lp, "G");
+          geometryList = msGMLGetGeometries(lp, "G");
 
           value = msOWSLookupMetadata(&(lp->metadata), "OFG", "namespace_prefix");
           if(value) 
@@ -974,6 +900,8 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
   if(layers)
     msFreeCharArray(layers, numlayers);
     
+  msGMLFreeNamespaces(namespaceList);
+
   return MS_SUCCESS;
 }
 
@@ -996,6 +924,8 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
   char   *pszFilter = NULL;
   int bFilterSet = 0;
   int bBBOXSet = 0;
+  int bFeatureIdSet = 0;
+
   char *pszNameSpace = NULL;
   const char *value;
   const char *user_namespace_prefix = "ms";
@@ -1007,7 +937,11 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 
   const char *output_schema_format = "XMLSCHEMA";  
   int outputformat = OWS_GML2; /* default output is GML 2.1 */
-    
+   
+  char **aFIDLayers = NULL;
+  char **aFIDValues = NULL;
+  int iFIDLayers = 0;
+
   gmlNamespaceListObj *namespaceList=NULL; /* for external application schema support */
 
   /* Default filter is map extents */
@@ -1021,7 +955,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 
   /* typename is mandatory unlsess featureid is specfied. We do not
      support featureid */
-  if (paramsObj->pszTypeName==NULL)
+  if (paramsObj->pszTypeName==NULL && paramsObj->pszFeatureId == NULL)
   {
       msSetError(MS_WFSERR, 
                  "Incomplete WFS request: TYPENAME parameter missing", 
@@ -1043,7 +977,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     /*  */
     /* __TODO__ Need to handle type grouping, e.g. "(l1,l2),l3,l4" */
     /*  */
-    layers = split(typename, ',', &numlayers);
+    layers = msStringSplit(typename, ',', &numlayers);
     
     /* ==================================================================== */
     /*      TODO: check if the typename contains namespaces (ex cdf:Other), */
@@ -1055,12 +989,12 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       return msWFSException(map, paramsObj->pszVersion);
     }
     
-    tokens = split(layers[0], ':', &n);
+    tokens = msStringSplit(layers[0], ':', &n);
     if (tokens && n==2 && msGetLayerIndex(map, layers[0]) < 0) {
       /* pszNameSpace = strdup(tokens[0]); */
       msFreeCharArray(tokens, n);
       for (i=0; i<numlayers; i++) {
-	tokens = split(layers[i], ':', &n);
+	tokens = msStringSplit(layers[i], ':', &n);
 	if (tokens && n==2) {
 	  free(layers[i]);
 	  layers[i] = strdup(tokens[1]);
@@ -1078,7 +1012,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     for(j=0; j<map->numlayers; j++) {
       layerObj *lp;
       
-      lp = &(map->layers[j]);
+      lp = GET_LAYER(map, j);
       
       /* Keep only selected layers, set to OFF by default. */
       lp->status = MS_OFF;
@@ -1090,7 +1024,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       for (j=0; j<map->numlayers; j++) {
 	layerObj *lp;
 	
-	lp = &(map->layers[j]);
+	lp = GET_LAYER(map, j);
 	
 	if (msWFSIsLayerSupported(lp) && lp->name && strcasecmp(lp->name, layers[k]) == 0) {
 	  const char *pszThisLayerSRS;
@@ -1181,10 +1115,11 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
   if (paramsObj->pszFilter) {
     bFilterSet = 1;
     pszFilter = paramsObj->pszFilter;
-  } else if (paramsObj->pszBbox) {
+  } 
+  if (paramsObj->pszBbox) {
     char **tokens;
     int n;
-    tokens = split(paramsObj->pszBbox, ',', &n);
+    tokens = msStringSplit(paramsObj->pszBbox, ',', &n);
     if (tokens==NULL || n != 4) {
       msSetError(MS_WFSERR, "Wrong number of arguments for BBOX.", "msWFSGetFeature()");
       return msWFSException(map, paramsObj->pszVersion);
@@ -1194,10 +1129,13 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     bbox.maxx = atof(tokens[2]);
     bbox.maxy = atof(tokens[3]);
     msFreeCharArray(tokens, n);
-    
+    bBBOXSet = 1;
     /* Note: BBOX SRS is implicit, it is the SRS of the selected */
     /* feature types, see pszOutputSRS in TYPENAMES above. */
   }
+  if (paramsObj->pszFeatureId) {
+    bFeatureIdSet = 1;
+  }
 
 #ifdef USE_OGR
     if (bFilterSet && pszFilter && strlen(pszFilter) > 0) {
@@ -1229,6 +1167,13 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	return msWFSException(map, paramsObj->pszVersion);
       }
 
+      if (bFeatureIdSet) {
+	msSetError(MS_WFSERR, 
+                   "FEATUREID parameter and FILTER parameter are mutually exclusive in GetFeature.", 
+                   "msWFSGetFeature()");
+	return msWFSException(map, paramsObj->pszVersion);
+      }
+
       /* -------------------------------------------------------------------- */
       /*      Parse the Filter parameter. If there are several Filter         */
       /*      parameters, each Filter is inside a parantheses. Eg :           */
@@ -1241,7 +1186,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       /* -------------------------------------------------------------------- */
       nFilters = 0;
       if (strlen(pszFilter) > 0 && pszFilter[0] == '(') {
-	tokens = split(pszFilter+1, '(', &nFilters);
+	tokens = msStringSplit(pszFilter+1, '(', &nFilters);
 	
 	if (tokens == NULL || nFilters <=0 || nFilters != numlayers) {
 	  msSetError(MS_WFSERR, "Wrong number of FILTER attributes",
@@ -1291,17 +1236,132 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       if (paszFilter)
 	free(paszFilter);
     }/* end if filter set */
+
+    if (bFeatureIdSet)
+    {
+        char **tokens = NULL, **tokens1=NULL ;
+        int nTokens = 0, j=0, nTokens1=0, k=0;
+        FilterEncodingNode *psNode = NULL;
+
+        /* Keep only selected layers, set to OFF by default. */
+        for(j=0; j<map->numlayers; j++) 
+        {
+            layerObj *lp;
+            lp = GET_LAYER(map, j);
+            lp->status = MS_OFF;
+        }
+        /*featureid can be a list INWATERA_1M.1234, INWATERA_1M.1235
+        We will keep all the feature id from the same layer together
+        so that an OR would be applied if several of them are present
+        */
+         tokens = msStringSplit(paramsObj->pszFeatureId, ',', &nTokens);
+         iFIDLayers = 0;
+         if (tokens && nTokens >=1)
+         {
+             aFIDLayers = (char **)malloc(sizeof(char *)*nTokens);
+             aFIDValues = (char **)malloc(sizeof(char *)*nTokens);
+             for (j=0; j<nTokens; j++)
+             {
+                 aFIDLayers[j] = NULL;
+                 aFIDValues[j] = NULL;
+             }
+             for (j=0; j<nTokens; j++)
+             {
+                 tokens1 = msStringSplit(tokens[j], '.', &nTokens1);
+                 if (tokens1 && nTokens1 == 2)
+                 {
+                     for (k=0; k<iFIDLayers; k++)
+                     {
+                         if (strcasecmp(aFIDLayers[k], tokens1[0]) == 0)
+                           break;
+                     }
+                     if (k == iFIDLayers)
+                     {
+                         aFIDLayers[iFIDLayers] = strdup(tokens1[0]);
+                         iFIDLayers++;
+                     }
+                     if (aFIDValues[k] != NULL)
+                       aFIDValues[k] = msStringConcatenate(aFIDValues[k], ",");
+                     aFIDValues[k] = msStringConcatenate( aFIDValues[k], tokens1[1]);
+                 }
+                 else
+                 {
+                     msSetError(MS_WFSERR, 
+		     "Invalid FeatureId in GetFeature. Expecting layername.value : %s", 
+		     "msWFSGetFeature()", pszFilter);
+                     if (tokens)
+                       msFreeCharArray(tokens, nTokens);
+                     if (tokens1)
+                       msFreeCharArray(tokens1, nTokens1);
+                     return msWFSException(map, paramsObj->pszVersion);
+                 }
+                 if (tokens1)
+                    msFreeCharArray(tokens1, nTokens1);
+             }
+         }
+         if (tokens)
+           msFreeCharArray(tokens, nTokens);
+
+         for (j=0; j< iFIDLayers; j++)
+         {
+             for (k=0; k<map->numlayers; k++) 
+             {
+                 layerObj *lp;
+                 lp = GET_LAYER(map, k);
+                 if (msWFSIsLayerSupported(lp) && lp->name && 
+                     strcasecmp(lp->name, aFIDLayers[j]) == 0)
+                 {
+                     lp->status = MS_ON;
+                     if (lp->template == NULL) {
+                         /* Force setting a template to enable query. */
+                         lp->template = strdup("ttt.html");
+                     }
+                     psNode = FLTCreateFeatureIdFilterEncoding(aFIDValues[j]);
+
+                     if( FLTApplyFilterToLayer(psNode, map, lp->index, MS_FALSE) != MS_SUCCESS )
+                       return msWFSException(map, paramsObj->pszVersion);
+
+                     FLTFreeFilterEncodingNode( psNode );
+                     psNode = NULL;
+                     break;
+                 }
+             }
+             if (k == map->numlayers)/*layer not found*/
+             {
+                 msSetError(MS_WFSERR, 
+                            "Invalid typename given with FeatureId in GetFeature : %s", "msWFSGetFeature()", 
+                            aFIDLayers[j]);
+
+                 if (aFIDLayers && aFIDValues)
+                 {      
+                     for (j=0; j<iFIDLayers; j++)
+                     {
+                         msFree(aFIDLayers[j]);
+                         msFree(aFIDValues[j]);
+                     }
+                     msFree(aFIDLayers);
+                     msFree(aFIDValues);
+                 }
+                 return msWFSException(map, paramsObj->pszVersion);
+             }
+         }
+
+         if (aFIDLayers && aFIDValues)
+         {
+             for (j=0; j<iFIDLayers; j++)
+             {
+                 msFree(aFIDLayers[j]);
+                 msFree(aFIDValues[j]);
+             }
+             msFree(aFIDLayers);
+             msFree(aFIDValues);
+         }
+    }
 #endif
 
     if(layers)
       msFreeCharArray(layers, numlayers);
 
-    if(typename == NULL)  {
-      msSetError(MS_WFSERR, 
-		 "Required TYPENAME parameter missing for GetFeature.", 
-		 "msWFSGetFeature()");
-      return msWFSException(map, paramsObj->pszFilter);
-    }
 
     /* Set map output projection to which output features should be reprojected */
     if (pszOutputSRS && strncasecmp(pszOutputSRS, "EPSG:", 5) == 0) {
@@ -1339,7 +1399,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     /*
     ** retrieve any necessary external namespace/schema configuration information
     */
-    namespaceList = msGMLGetNamespaces(&(map->web), "OFG");
+    namespaceList = msGMLGetNamespaces(&(map->web), "G");
 
     msIO_printf("Content-type: text/xml%c%c",10,10);
 
@@ -1425,7 +1485,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     
     /* if no results where written (TODO: this needs to be GML2/3 specific I imagine */
     for(i=0; i<map->numlayers; i++) {
-      if (map->layers[i].resultcache && map->layers[i].resultcache->numresults > 0)
+      if (GET_LAYER(map, i)->resultcache && GET_LAYER(map, i)->resultcache->numresults > 0)
         break;
     }
     if ((i==map->numlayers) || (maxfeatures == 0)) {
@@ -1448,6 +1508,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     if (pszNameSpace)
       free(pszNameSpace);
     msFree(user_namespace_uri_encoded);
+    msGMLFreeNamespaces(namespaceList);
 
     return MS_SUCCESS;
 }
@@ -1658,6 +1719,8 @@ void msWFSFreeParamsObj(wfsParamsObj *wfsparams)
           free(wfsparams->pszTypeName);
         if (wfsparams->pszFilter)
           free(wfsparams->pszFilter);
+        if (wfsparams->pszFeatureId)
+          free(wfsparams->pszFeatureId);
     }
 }
 
@@ -1703,11 +1766,15 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
                 
                 else if (strcasecmp(request->ParamNames[i], "OUTPUTFORMAT") == 0)
                   wfsparams->pszOutputFormat = strdup(request->ParamValues[i]);
+
+                 else if (strcasecmp(request->ParamNames[i], "FEATUREID") == 0)
+                  wfsparams->pszFeatureId = strdup(request->ParamValues[i]);
             }
         }
         /* version is optional is the GetCapabilities. If not */
         /* provided, set it. */
-        if (wfsparams->pszRequest && 
+        if (wfsparams->pszVersion == NULL &&
+            wfsparams->pszRequest && 
             strcasecmp(wfsparams->pszRequest, "GetCapabilities") == 0)
           wfsparams->pszVersion = strdup("1.0.0");
     }
