@@ -14,8 +14,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  * 
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  * 
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -23,130 +23,9 @@
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
- * DEALINGS IN THE SOFTWARE.
- **********************************************************************
- *
- * $Log$
- * Revision 1.37  2006/05/19 20:53:27  dan
- * Use lp->layerinfo for OGR connections (instead of ogrlayerinfo) (bug 331)
- *
- * Revision 1.36  2005/10/28 01:09:42  jani
- * MS RFC 3: Layer vtable architecture (bug 1477)
- *
- * Revision 1.35  2005/05/12 18:41:54  assefa
- * Use %f instead of %lf for bbox output : Bug 1239.
- *
- * Revision 1.34  2005/04/22 15:47:10  assefa
- * Bug 1302: the wfs/ows_service parameter is not used any more. The
- * service is always set to WFS for WFS layers.
- *
- * Revision 1.33  2005/04/21 14:39:47  dan
- * Fixed typo in error messages (bug 1302)
- *
- * Revision 1.32  2005/02/18 03:06:48  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.31  2005/02/14 04:33:32  sdlime
- * Changed %.17g to %.15g for WMS/WFS server code.
- *
- * Revision 1.30  2005/01/28 06:16:54  sdlime
- * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
- *
- * Revision 1.29  2004/11/25 06:19:05  dan
- * Add trailing "?" or "&" to connection string when required in WFS
- * client layers using GET method (bug 1082)
- *
- * Revision 1.28  2004/11/23 23:16:29  dan
- * Fixed build error when WFS was not enabled (bug 1083)
- *
- * Revision 1.27  2004/11/16 21:57:49  dan
- * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
- * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
- *
- * Revision 1.26  2004/11/15 20:35:02  dan
- * Added msLayerIsOpen() to all vector layer types (bug 1051)
- *
- * Revision 1.25  2004/11/12 17:05:01  assefa
- * Use %.17g instrad of %f when output bbox values (Bug 678)
- *
- * Revision 1.24  2004/10/21 04:30:57  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- * Revision 1.23  2004/07/13 20:39:37  dan
- * Made msTmpFile() more robust using msBuildPath() to return absolute paths (bug 771)
- *
- * Revision 1.22  2004/06/22 20:55:21  sean
- * Towards resolving issue 737 changed hashTableObj to a structure which contains a hashObj **items.  Changed all hash table access functions to operate on the target table by reference.  msFreeHashTable should not be used on the hashTableObj type members of mapserver structures, use msFreeHashItems instead.
- *
- * Revision 1.21  2004/04/27 16:16:11  assefa
- * Use wfs_request_method instead of wms_request_method.
- *
- * Revision 1.20  2004/03/15 07:43:42  frank
- * avoid warnings about usused variables when building without USE_WFS_LYR
- *
- * Revision 1.19  2004/03/11 22:45:39  dan
- * Added pszPostContentType in httpRequestObj instead of using hardcoded
- * text/html mime type for all post requests.
- *
- * Revision 1.18  2003/10/30 22:37:01  assefa
- * Add function msWFSExecuteGetFeature on a wfs layer.
- *
- * Revision 1.17  2003/10/22 20:20:09  assefa
- * Test if wfs_filter metada is empty.
- *
- * Revision 1.16  2003/09/19 21:54:19  assefa
- * Add support fot the Post request.
- *
- * Revision 1.15  2003/09/10 19:53:19  assefa
- * Use local hash function instead of md5.
- *
- * Revision 1.14  2003/09/10 03:52:53  assefa
- * The <Filter ...> and </Filter> is now generated here intstead of
- * comming from the wfs_filter metadata parameter.
- *
- * Revision 1.13  2003/09/04 17:47:15  assefa
- * Add filterencoding tests.
- *
- * Revision 1.12  2003/03/26 20:24:38  dan
- * Do not call msDebug() unless debug flag is turned on
- *
- * Revision 1.11  2003/02/19 14:19:02  frank
- * cleanup warnings
- *
- * Revision 1.10  2002/12/19 07:35:53  dan
- * Call msWFSLayerWhichShapes() inside open() to force downloading layer and
- * enable msOGRLayerGetItems() to work for queries.  i.e.WFS queries work now.
- *
- * Revision 1.9  2002/12/19 06:30:59  dan
- * Enable caching WMS/WFS request using tmp filename built from URL
- *
- * Revision 1.8  2002/12/19 05:17:09  dan
- * Report WFS exceptions, and do not fail on WFS requests returning 0 features
- *
- * Revision 1.7  2002/12/17 21:33:54  dan
- * Enable following redirections with libcurl (requires libcurl 7.10.1+)
- *
- * Revision 1.6  2002/12/17 05:30:17  dan
- * Fixed HTTP timeout value (in secs, not msecs) for WMS/WFS requests
- *
- * Revision 1.5  2002/12/17 04:48:25  dan
- * Accept version 0.0.14 in addition to 1.0.0
- *
- * Revision 1.4  2002/12/16 20:35:00  dan
- * Flush libwww and use libcurl instead for HTTP requests in WMS/WFS client
- *
- * Revision 1.3  2002/12/13 01:32:53  dan
- * OOOpps.. lp->wfslayerinfo was not set in msWFSLayerOpen()
- *
- * Revision 1.2  2002/12/13 00:57:31  dan
- * Modified WFS implementation to behave more as a real vector data source
- *
- * Revision 1.1  2002/10/09 02:29:03  dan
- * Initial implementation of WFS client support.
- *
- **********************************************************************/
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include "maperror.h"
 #include "mapows.h"
 
@@ -157,7 +36,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: mapwfslayer.c 5445 2006-05-19 20:53:27Z dan $")
+MS_CVSID("$Id: mapwfslayer.c 6373 2007-07-24 02:37:18Z dmorissette $")
 
 #define WFS_V_0_0_14  14
 #define WFS_V_1_0_0  100
@@ -704,7 +583,7 @@ int msPrepareWFSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
         int iLayer;
         for(iLayer=0; iLayer < map->numlayers; iLayer++)
         {
-            if (&(map->layers[iLayer]) == lp)
+            if (GET_LAYER(map, iLayer) == lp)
             {
                 nLayerId = iLayer;
                 break;
@@ -842,6 +721,9 @@ int msWFSLayerOpen(layerObj *lp,
 #ifdef USE_WFS_LYR
     int status = MS_SUCCESS;
     msWFSLayerInfo *psInfo = NULL;
+    
+    if ( msCheckParentPointer(lp->map,"map")==MS_FAILURE )
+		return MS_FAILURE;
 
     if (lp->wfslayerinfo != NULL)
     {
@@ -1002,6 +884,10 @@ int msWFSLayerWhichShapes(layerObj *lp, rectObj rect)
     int status = MS_SUCCESS;
     const char *pszTmp;
     FILE *fp;
+    
+    if ( msCheckParentPointer(lp->map,"map")==MS_FAILURE )
+		return MS_FAILURE;
+    
 
     psInfo =(msWFSLayerInfo*)lp->wfslayerinfo;
 
@@ -1022,7 +908,7 @@ int msWFSLayerWhichShapes(layerObj *lp, rectObj rect)
         int n;
         rectObj ext;
 
-        tokens = split(pszTmp, ' ', &n);
+        tokens = msStringSplit(pszTmp, ' ', &n);
         if (tokens==NULL || n != 4) {
             msSetError(MS_WFSCONNERR, "Wrong number of values in 'wfs_latlonboundingbox' metadata.",
                        "msWFSLayerWhichShapes()");
