@@ -26,34 +26,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.23  2006/08/22 13:54:37  hobu
- * cast the second argument to char* that we're passing into strncopy
- *
- * Revision 1.22  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.21  2005/04/22 14:11:30  frank
- * Bug 791: initialize some fields in msDBFCreate()
- *
- * Revision 1.20  2005/02/18 03:06:48  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.19  2005/02/07 18:37:43  frank
- * Removed dangerous junk in leading space trimmer.
- *
- * Revision 1.18  2004/10/21 04:30:54  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
-#include "map.h"
+#include "mapserver.h"
 #include <stdlib.h> /* for atof() and atoi() */
 #include <math.h>
 
-MS_CVSID("$Id: mapxbase.c 5620 2006-08-22 13:54:37Z hobu $")
+MS_CVSID("$Id: mapxbase.c 6600 2007-08-16 14:48:27Z dmorissette $")
 
 /************************************************************************/
 /*                             SfRealloc()                              */
@@ -468,6 +447,38 @@ int	msDBFAddField(DBFHandle psDBF, const char * pszFieldName, DBFFieldType eType
     return( psDBF->nFields-1 );
 }
 
+/************************************************************************/
+/*                         DBFIsValueNULL()                             */
+/*                                                                      */
+/*      Return TRUE if value is NULL (in DBF terms).                    */
+/*                                                                      */
+/*      Based on DBFIsAttributeNULL of shapelib                         */
+/************************************************************************/
+
+int DBFIsValueNULL( const char* pszValue, char type )
+
+{   
+    switch(type)
+    {
+      case 'N':
+      case 'F':
+        /* NULL numeric fields have value "****************" */
+        return pszValue[0] == '*';
+
+      case 'D':
+        /* NULL date fields have value "00000000" */
+        return strncmp(pszValue,"00000000",8) == 0;
+
+      case 'L':
+        /* NULL boolean fields have value "?" */
+        return pszValue[0] == '?';
+
+      default:
+        /* empty string fields are considered NULL */
+        return strlen(pszValue) == 0;
+    }
+}
+
 /************************************************************************/
 /*                          msDBFReadAttribute()                        */
 /*                                                                      */
@@ -485,13 +496,13 @@ static char *msDBFReadAttribute(DBFHandle psDBF, int hEntity, int iField )
     /* -------------------------------------------------------------------- */
     if( iField < 0 || iField >= psDBF->nFields ) 
     {
-        msSetError(MS_DBFERR, "Invalid field index %d.", "msDBFGetItemIndex()",iField );
+        msSetError(MS_DBFERR, "Invalid field index %d.", "msDBFReadAttribute()",iField );
         return( NULL );
     }
 
     if( hEntity < 0 || hEntity >= psDBF->nRecords )
     {
-        msSetError(MS_DBFERR, "Invalid record number %d.", "msDBFGetItemIndex()",hEntity );
+        msSetError(MS_DBFERR, "Invalid record number %d.", "msDBFReadAttribute()",hEntity );
         return( NULL );
     }
 
@@ -511,6 +522,8 @@ static char *msDBFReadAttribute(DBFHandle psDBF, int hEntity, int iField )
     }
 
     pabyRec = (uchar *) psDBF->pszCurrentRecord;
+    /* DEBUG */
+    /* printf("CurrentRecord(%c):%s\n", psDBF->pachFieldType[iField], pabyRec); */
 
     /* -------------------------------------------------------------------- */
     /*	Ensure our field buffer is large enough to hold this buffer.	    */
@@ -552,6 +565,11 @@ static char *msDBFReadAttribute(DBFHandle psDBF, int hEntity, int iField )
     else
         pReturnField = psDBF->pszStringField;
 
+    /*  detect null values */
+    if ( DBFIsValueNULL( pReturnField, psDBF->pachFieldType[iField] )  ) {
+	if (psDBF->pachFieldType[iField] == 'N' || psDBF->pachFieldType[iField] == 'F' || psDBF->pachFieldType[iField] == 'D')
+		pReturnField="0";	
+    }	
     return( pReturnField );
 }
 
