@@ -24,30 +24,17 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.14  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.13  2005/04/26 03:37:50  frank
- * Fixed setting of this_rec from hDBF->nRecords, was screwing up dbf file.
- * http://mapserver.gis.umn.edu/bugs/show_bug.cgi?id=1332
- *
- * Revision 1.12  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
 
-#include "map.h"
+#include "mapserver.h"
 #include <string.h>
 #ifndef _WIN32
 #include <unistd.h>
 #endif
 #include <stdlib.h>
 
-MS_CVSID("$Id: shptreevis.c 4695 2005-06-14 16:03:36Z dan $")
+MS_CVSID("$Id: shptreevis.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 #ifdef SHPT_POLYGON
    #undef MAPSERVER
@@ -172,7 +159,7 @@ int main( int argc, char ** argv )
 #endif
 
     printf ("This %s %s index supports a shapefile with %d shapes, %d depth \n",
-	(qix->version ? "new": "old"), (qix->LSB_order? "LSB": "MSB"), qix->nShapes, qix->nDepth);
+	(qix->version ? "new": "old"), (qix->LSB_order? "LSB": "MSB"), (int) qix->nShapes, (int) qix->nDepth);
 
 
 /* -------------------------------------------------------------------- */
