@@ -24,26 +24,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.9  2005/06/14 16:03:35  dan
- * Updated copyright date to 2005
- *
- * Revision 1.8  2005/02/18 03:06:48  dan
- * Turned all C++ (//) comments into C comments (bug 1238)
- *
- * Revision 1.7  2004/10/21 04:30:55  frank
- * Added standardized headers.  Added MS_CVSID().
- *
- */
+ ****************************************************************************/
 
 #include <gd.h>
 #include <gdfontt.h>
 
-#include "map.h"
+#include "mapserver.h"
 
-MS_CVSID("$Id: sym2img.c 4695 2005-06-14 16:03:36Z dan $")
+MS_CVSID("$Id: sym2img.c 6428 2007-07-31 19:58:45Z dmorissette $")
 
 #define MAXCOLORS 256
 #define CELLSIZE 100
@@ -161,8 +149,8 @@ int main(int argc, char *argv[])
 	break;
       }
 
-      if(symbolSet.symbol[n].name)
-	sprintf(buffer, "%d - %s", n, symbolSet.symbol[n].name);
+      if(symbolSet.symbol[n]->name)
+	sprintf(buffer, "%d - %s", n, symbolSet.symbol[n]->name);
       else
 	sprintf(buffer, "%d", n);
       gdImageString(img, gdFontTiny, j+1, i+1, buffer, black);
