@@ -1,5 +1,5 @@
 /******************************************************************************
-* $Id: tile4ms.c 5173 2006-02-09 05:06:25Z frank $
+* $Id: tile4ms.c 6373 2007-07-24 02:37:18Z dmorissette $
 *
 * tile4ms.mc  
 *
@@ -38,10 +38,10 @@
 *
 */
 
-#include "map.h"
+#include "mapserver.h"
 #include <string.h>
 
-MS_CVSID("$Id: tile4ms.c 5173 2006-02-09 05:06:25Z frank $")
+MS_CVSID("$Id: tile4ms.c 6373 2007-07-24 02:37:18Z dmorissette $")
 
 /***********************************************************************/
 int process_shapefiles(char *metaFileNameP, char *tileFileNameP, 
